@@ -210,7 +210,7 @@ struct gui : g3d_gui
                 break;
             case 1:
                 if(ishorizontal()) cury -= max(ysize - l.h, 0);
-                else curx -= max(xsize - l.h, 0);
+                else curx -= max(xsize - l.w, 0);
                 break;
             }
         }
@@ -394,18 +394,20 @@ struct gui : g3d_gui
             e->draw(curx+FONTW/2, cury, color, hit && editing);
 
             lineshader->set();
+            glDisable(GL_BLEND);
             glDisable(GL_TEXTURE_2D);
             if(editing) glColor3ub(guirollovercolour >> 16, (guirollovercolour >> 8)&0xFF, guirollovercolour&0xFF);
             else glColor3ub(color>>16, (color>>8)&0xFF, color&0xFF);
             rect_(curx, cury, w, h, -1, true);
             glEnable(GL_TEXTURE_2D);
+            glEnable(GL_BLEND);
             defaultshader->set();
         }
         layout(w, h);
 
         if(e->maxy != 1)
         {
-            int slines = e->lines.length()-e->pixelheight/FONTH;
+            int slines = e->limitscrolly();
             if(slines > 0)
             {
                 int pos = e->scrolly;
@@ -445,7 +447,7 @@ struct gui : g3d_gui
 
     void text_(const char *text, int x, int y, int color, bool shadow)
     {
-        if(shadow) draw_text(text, x+SHADOW, y+SHADOW, 0x00, 0x00, 0x00, 0xC0);
+        if(shadow) draw_text(text, x+SHADOW, y+SHADOW, 0x00, 0x00, 0x00, -0xC0);
         draw_text(text, x, y, color>>16, (color>>8)&0xFF, color&0xFF);
     }
 
@@ -1089,7 +1091,7 @@ void g3d_limitscale(float scale)
     gui::maxscale = scale;
 }
 
-int g3d_sort(gui *a, gui *b) { return (int)(a->dist>b->dist)*2-1; }
+static inline bool g3d_sort(const gui &a, const gui &b) { return a.dist > b.dist; }
 
 bool g3d_windowhit(bool on, bool act)
 {
@@ -1188,26 +1190,10 @@ void g3d_render()
 
     mousebuttons = 0;
 }
-#endif
-
 
 void consolebox(int x1, int y1, int x2, int y2)
 {
     glPushMatrix();
-    #ifdef NEWGUI
-    glScalef(.5, .5, 1);
-    glTranslatef(x1, y1, 0);
-    glColor4f(1, 0, 0, .8);
-    glBegin(GL_TRIANGLE_STRIP);
-
-    glVertex2i(x1, y1);
-    glVertex2i(x2, y1);
-    glVertex2i(x1, y2);
-    glVertex2i(x2, y2);
-
-    glEnd();
-
-    #else
 
     glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
     glTranslatef(x1, y1, 0);
@@ -1220,7 +1206,7 @@ void consolebox(int x1, int y1, int x2, int y2)
     gui::drawskin(-gui::skinx[1]*SKIN_SCALE, -gui::skiny[4]*SKIN_SCALE, int(bw), int(bh), 0, 9, 1, vec(1, 1, 1), 0.60f);
     gui::drawskin((-gui::skinx[1] + gui::skinx[2] - gui::skinx[5])*SKIN_SCALE, -gui::skiny[4]*SKIN_SCALE, int(bw), 0, 9, 1, 1, vec(1, 1, 1), 0.60f);
 
-    #endif
     glPopMatrix();
 }
+#endif
 
