@@ -13,19 +13,14 @@ struct animmodel : model
         int priority;
     };
 
-    //__tha__ see custom animations
-    struct vecanimspec
-    {
-        vector<animspec> spec;
-    };
-
     struct animpos
     {
-        int fr1, fr2;
+        int anim, fr1, fr2;
         float t;
 
         void setframes(const animinfo &info)
         {
+            anim = info.anim;
             if(info.range<=1)
             {
                 fr1 = 0;
@@ -64,7 +59,6 @@ struct animmodel : model
     struct animstate
     {
         part *owner;
-        int anim;
         animpos cur, prev;
         float interp;
 
@@ -80,12 +74,10 @@ struct animmodel : model
         part *owner;
         Texture *tex, *masks, *envmap, *unlittex, *normalmap;
         Shader *shader;
-        float spec, ambient, glow, specglare, glowglare, fullbright, envmapmin, envmapmax, scrollu, scrollv, alphatest;
+        float spec, ambient, glow, glowdelta, glowpulse, specglare, glowglare, fullbright, envmapmin, envmapmax, scrollu, scrollv, alphatest;
         bool alphablend, cullface;
-        bool skinanim; //__offtools__ skin animations
-    hashtable<int, GLuint> anim; //frame, tex id
 
-        skin() : owner(0), tex(notexture), masks(notexture), envmap(NULL), unlittex(NULL), normalmap(NULL), shader(NULL), spec(1.0f), ambient(0.3f), glow(3.0f), specglare(1), glowglare(1), fullbright(0), envmapmin(0), envmapmax(0), scrollu(0), scrollv(0), alphatest(0.9f), alphablend(true), cullface(true), skinanim(false) {}
+        skin() : owner(0), tex(notexture), masks(notexture), envmap(NULL), unlittex(NULL), normalmap(NULL), shader(NULL), spec(1.0f), ambient(0.3f), glow(3.0f), glowdelta(0), glowpulse(0), specglare(1), glowglare(1), fullbright(0), envmapmin(0), envmapmax(0), scrollu(0), scrollv(0), alphatest(0.9f), alphablend(true), cullface(true) {}
 
         bool multitextured() { return enableglow; }
         bool envmapped() { return hasCM && envmapmax>0 && envmapmodels && (renderpath!=R_FIXEDFUNCTION || maxtmus >= 3); }
@@ -106,7 +98,7 @@ struct animmodel : model
                 if(!enablerescale) { glEnable(hasRN ? GL_RESCALE_NORMAL_EXT : GL_NORMALIZE); enablerescale = true; }
             }
             if(masked!=enableglow) lasttex = lastmasks = NULL;
-            float mincolor = as->anim&ANIM_FULLBRIGHT ? fullbrightmodels/100.0f : 0.0f;
+            float mincolor = as->cur.anim&ANIM_FULLBRIGHT ? fullbrightmodels/100.0f : 0.0f;
             vec color = vec(lightcolor).max(mincolor), matcolor(1, 1, 1);
             if(masked)
             {
@@ -164,20 +156,29 @@ struct animmodel : model
             if(fullbright)
             {
                 glColor4f(fullbright/2, fullbright/2, fullbright/2, transparent);
-                setenvparamf("lightscale", SHPARAM_VERTEX, 2, 0, 2, glow);
-                setenvparamf("lightscale", SHPARAM_PIXEL, 2, 0, 2, glow);
+                setenvparamf("lightscale", SHPARAM_VERTEX, 2, 0, 0, 2);
+                setenvparamf("lightscale", SHPARAM_PIXEL, 2, 0, 0, 2);
             }
             else
             {
-                float mincolor = as->anim&ANIM_FULLBRIGHT ? fullbrightmodels/100.0f : 0.0f, minshade = max(ambient, mincolor);
+                float mincolor = as->cur.anim&ANIM_FULLBRIGHT ? fullbrightmodels/100.0f : 0.0f,
+                      bias = max(mincolor-1.0f, 0.2f), scale = 0.5f*max(0.8f-bias, 0.0f),
+                      minshade = scale*max(ambient, mincolor);
                 vec color = vec(lightcolor).max(mincolor);
                 glColor4f(color.x, color.y, color.z, transparent);
-                setenvparamf("lightscale", SHPARAM_VERTEX, 2, spec, minshade, glow);
-                setenvparamf("lightscale", SHPARAM_PIXEL, 2, spec, minshade, glow);
+                setenvparamf("lightscale", SHPARAM_VERTEX, 2, scale - minshade, scale, minshade + bias);
+                setenvparamf("lightscale", SHPARAM_PIXEL, 2, scale - minshade, scale, minshade + bias);
             }
+            float curglow = glow;
+            if(glowpulse > 0)
+            {
+                float curpulse = lastmillis*glowpulse;
+                curpulse -= floor(curpulse);
+                curglow += glowdelta*2*fabs(curpulse - 0.5f);
+            }
+            setenvparamf("maskscale", SHPARAM_PIXEL, 4, 0.5f*spec*lightmodels, 0.5f*curglow*glowmodels, 16*specglare, 4*glowglare);
             setenvparamf("texscroll", SHPARAM_VERTEX, 5, lastmillis/1000.0f, scrollu*lastmillis/1000.0f, scrollv*lastmillis/1000.0f);
             if(envmaptmu>=0 && envmapmax>0) setenvparamf("envmapscale", bumpmapped() ? SHPARAM_PIXEL : SHPARAM_VERTEX, 3, envmapmin-envmapmax, envmapmax);
-            if(glaring) setenvparamf("glarescale", SHPARAM_PIXEL, 4, 16*specglare, 4*glowglare);
         }
 
         Shader *loadshader(bool shouldenvmap, bool masked)
@@ -242,7 +243,7 @@ struct animmodel : model
             if(!cullface && enablecullface) { glDisable(GL_CULL_FACE); enablecullface = false; }
             else if(cullface && !enablecullface) { glEnable(GL_CULL_FACE); enablecullface = true; }
 
-            if(as->anim&ANIM_NOSKIN)
+            if(as->cur.anim&ANIM_NOSKIN)
             {
                 if(enablealphatest) { glDisable(GL_ALPHA_TEST); enablealphatest = false; }
                 if(enablealphablend) { glDisable(GL_BLEND); enablealphablend = false; }
@@ -251,7 +252,7 @@ struct animmodel : model
                 if(enablelighting) { glDisable(GL_LIGHTING); enablelighting = false; }
                 if(enablerescale) { glDisable(hasRN ? GL_RESCALE_NORMAL_EXT : GL_NORMALIZE); enablerescale = false; }
                 if(shadowmapping) SETMODELSHADER(b, shadowmapcaster);
-                else /*if(as->anim&ANIM_SHADOW)*/ SETMODELSHADER(b, notexturemodel);
+                else /*if(as->cur.anim&ANIM_SHADOW)*/ SETMODELSHADER(b, notexturemodel);
                 return;
             }
             Texture *s = bumpmapped() && unlittex ? unlittex : tex,
@@ -266,17 +267,10 @@ struct animmodel : model
                 setshaderparams(b, as, m!=notexture);
                 setshader(b, as, m!=notexture);
             }
-            if(s!=lasttex || skinanim) //__offtools__ skin animation)
+            if(s!=lasttex)
             {
                 if(enableglow) glActiveTexture_(GL_TEXTURE1_ARB);
-                if (anim.access(as->cur.fr1))
-		{
-			glBindTexture(GL_TEXTURE_2D, anim[as->cur.fr1]); //__offtools__ change texture depending on frame
-		 }
-		else
-		{
-                	glBindTexture(GL_TEXTURE_2D, s->id);
-                }
+                glBindTexture(GL_TEXTURE_2D, s->id);
                 if(enableglow) glActiveTexture_(GL_TEXTURE0_ARB);
                 lasttex = s;
             }
@@ -440,7 +434,7 @@ struct animmodel : model
             }
             loopi(numverts) verts[i].norm.normalize();
         }
-        
+
         template<class B, class V, class TC, class T> void calctangents(B *bumpverts, V *verts, TC *tcverts, int numverts, T *tris, int numtris, bool areaweight)
         {
             vec *tangent = new vec[2*numverts], *bitangent = tangent+numverts;
@@ -455,12 +449,16 @@ struct animmodel : model
                          &tc1 = tcverts[t.vert[1]],
                          &tc2 = tcverts[t.vert[2]];
                 float u1 = tc1.u - tc0.u, v1 = tc1.v - tc0.v,
-                      u2 = tc2.u - tc0.u, v2 = tc2.v - tc0.v,
-                      scale = u1*v2 - u2*v1;
-                if(scale!=0) scale = 1.0f / scale;
-                vec u(e1), v(e2);
-                u.mul(v2).sub(vec(e2).mul(v1)).mul(scale);
-                v.mul(u1).sub(vec(e1).mul(u2)).mul(scale);
+                      u2 = tc2.u - tc0.u, v2 = tc2.v - tc0.v;
+                vec u(e2), v(e2);
+                u.mul(v1).sub(vec(e1).mul(v2));
+                v.mul(u1).sub(vec(e1).mul(u2));
+
+                if(vec().cross(e2, e1).dot(vec().cross(v, u)) < 0)
+                {
+                    u.neg();
+                    v.neg();
+                }
 
                 if(!areaweight)
                 {
@@ -485,7 +483,7 @@ struct animmodel : model
             }
             delete[] tangent;
         }
-    
+
     };
 
     struct meshgroup
@@ -527,7 +525,7 @@ struct animmodel : model
         int clipframes(int i, int n) const { return min(n, totalframes() - i); }
 
         virtual void cleanup() {}
-        virtual void render(const animstate *as, float pitch, const vec &axis, dynent *d, part *p) {}
+        virtual void render(const animstate *as, float pitch, const vec &axis, const vec &forward, dynent *d, part *p) {}
     };
 
     virtual meshgroup *loadmeshes(char *name, va_list args) { return NULL; }
@@ -565,24 +563,18 @@ struct animmodel : model
         meshgroup *meshes;
         vector<linkedpart> links;
         vector<skin> skins;
-        //vector<animspec> *anims[MAXANIMPARTS];
-        vector<vecanimspec> *anims; //vector for custom animation instead fixed array
+        vector<animspec> *anims[MAXANIMPARTS];
         int numanimparts;
         float pitchscale, pitchoffset, pitchmin, pitchmax;
         vec translate;
 
         part() : meshes(NULL), numanimparts(1), pitchscale(1), pitchoffset(0), pitchmin(0), pitchmax(0), translate(0, 0, 0)
         {
-            //loopk(MAXANIMPARTS) anims[k] = NULL;
-            anims = new vector<vecanimspec>[MAXANIMPARTS];
+            loopk(MAXANIMPARTS) anims[k] = NULL;
         }
         virtual ~part()
         {
-             loopi(MAXANIMPARTS)
-		{
-			anims[i].setsize(0);
-		}
-		DELETEA( anims );
+            loopk(MAXANIMPARTS) DELETEA(anims[k]);
         }
 
         virtual void cleanup()
@@ -625,7 +617,7 @@ struct animmodel : model
             int i = meshes ? meshes->findtag(tag) : -1;
             if(i<0)
             {
-                loopv(links) if(links[i].p && links[i].p->link(p, tag, translate, anim, basetime, pos)) return true;
+                loopvrev(links) if(links[i].p && links[i].p->link(p, tag, translate, anim, basetime, pos)) return true;
                 return false;
             }
             linkedpart &l = links.add();
@@ -677,7 +669,7 @@ struct animmodel : model
             info.range = 1;
         }
 
-        bool calcanim(int animpart, int anim, int basetime, int basetime2, dynent *d, int interp, animinfo &info)
+        bool calcanim(int animpart, int anim, int basetime, int basetime2, dynent *d, int interp, animinfo &info, int &aitime)
         {
             uint varseed = uint((size_t)d);
             info.anim = anim;
@@ -692,15 +684,14 @@ struct animmodel : model
             else
             {
                 animspec *spec = NULL;
-                //if(anims[animpart])
-                if(anims[animpart].length() > 0)
+                if(anims[animpart])
                 {
-                    vector<animspec> &primary = anims[animpart][anim&ANIM_INDEX].spec;
-                    if(primary.length()) spec = &primary[uint(varseed + basetime)%primary.length()];
+                    vector<animspec> &primary = anims[animpart][anim&ANIM_INDEX];
+                    if(&primary < &anims[animpart][NUMANIMS] && primary.length()) spec = &primary[uint(varseed + basetime)%primary.length()];
                     if((anim>>ANIM_SECONDARY)&(ANIM_INDEX|ANIM_DIR))
                     {
-                        vector<animspec> &secondary = anims[animpart][(anim>>ANIM_SECONDARY)&ANIM_INDEX].spec;
-                        if(secondary.length())
+                        vector<animspec> &secondary = anims[animpart][(anim>>ANIM_SECONDARY)&ANIM_INDEX];
+                        if(&secondary < &anims[animpart][NUMANIMS] && secondary.length())
                         {
                             animspec &spec2 = secondary[uint(varseed + basetime2)%secondary.length()];
                             if(!spec || spec2.priority > spec->priority)
@@ -723,15 +714,18 @@ struct animmodel : model
 
             info.anim &= (1<<ANIM_SECONDARY)-1;
             info.anim |= anim&ANIM_FLAGS;
-            if(info.anim&(ANIM_LOOP|ANIM_START|ANIM_END))
+            if((info.anim&ANIM_CLAMP) != ANIM_CLAMP)
             {
-                info.anim &= ~ANIM_SETTIME;
-                if(!info.basetime) info.basetime = -((int)(size_t)d&0xFFF);
-            }
-            if(info.anim&(ANIM_START|ANIM_END))
-            {
-                if(info.anim&ANIM_END) info.frame += info.range-1;
-                info.range = 1;
+                if(info.anim&(ANIM_LOOP|ANIM_START|ANIM_END))
+                {
+                    info.anim &= ~ANIM_SETTIME;
+                    if(!info.basetime) info.basetime = -((int)(size_t)d&0xFFF);
+                }
+                if(info.anim&(ANIM_START|ANIM_END))
+                {
+                    if(info.anim&ANIM_END) info.frame += info.range-1;
+                    info.range = 1;
+                }
             }
 
             if(!meshes->hasframes(info.frame, info.range))
@@ -743,19 +737,20 @@ struct animmodel : model
             if(d && interp>=0)
             {
                 animinterpinfo &ai = d->animinterp[interp];
-                if(d->ragdoll && !(anim&ANIM_RAGDOLL))
+                if((info.anim&ANIM_CLAMP)==ANIM_CLAMP) aitime = min(aitime, int(info.range*info.speed*0.5e-3f));
+                if(d->ragdoll && !(anim&ANIM_RAGDOLL)) 
                 {
                     ai.prev.range = ai.cur.range = 0;
                     ai.lastswitch = -1;
                 }
-                else if(ai.lastmodel!=this || ai.lastswitch<0 || lastmillis-d->lastrendered>animationinterpolationtime)
+                else if(ai.lastmodel!=this || ai.lastswitch<0 || lastmillis-d->lastrendered>aitime)
                 {
                     ai.prev = ai.cur = info;
-                    ai.lastswitch = lastmillis-animationinterpolationtime*2;
+                    ai.lastswitch = lastmillis-aitime*2;
                 }
                 else if(ai.cur!=info)
                 {
-                    if(lastmillis-ai.lastswitch>animationinterpolationtime/2) ai.prev = ai.cur;
+                    if(lastmillis-ai.lastswitch>aitime/2) ai.prev = ai.cur;
                     ai.cur = info;
                     ai.lastswitch = lastmillis;
                 }
@@ -765,61 +760,47 @@ struct animmodel : model
             return true;
         }
 
-        float calcpitchaxis(int anim, float pitch, vec &axis, vec &dir, vec &campos)
-        {
-            float angle = pitchscale*pitch + pitchoffset;
-            if(pitchmin || pitchmax) angle = max(pitchmin, min(pitchmax, angle));
-            if(!angle) return 0;
-
-            float c = cosf(-angle*RAD), s = sinf(-angle*RAD);
-            vec d(axis);
-            axis.rotate(c, s, d);
-            if(!(anim&ANIM_NOSKIN))
-            {
-                dir.rotate(c, s, d);
-                campos.rotate(c, s, d);
-            }
-
-            return angle;
-        }
-
-        void render(int anim, int basetime, int basetime2, float pitch, const vec &axis, dynent *d, const vec &dir, const vec &campos)
+        void render(int anim, int basetime, int basetime2, float pitch, const vec &axis, const vec &forward, dynent *d)
         {
             animstate as[MAXANIMPARTS];
-            render(anim, basetime, basetime2, pitch, axis, d, dir, campos, as);
+            render(anim, basetime, basetime2, pitch, axis, forward, d, as);
         }
 
-        void render(int anim, int basetime, int basetime2, float pitch, const vec &axis, dynent *d, const vec &dir, const vec &campos, animstate *as)
+        void render(int anim, int basetime, int basetime2, float pitch, const vec &axis, const vec &forward, dynent *d, animstate *as)
         {
             if(!(anim&ANIM_REUSE)) loopi(numanimparts)
             {
                 animinfo info;
-                int interp = d && index+numanimparts<=MAXANIMPARTS ? index+i : -1;
-                if(!calcanim(i, anim, basetime, basetime2, d, interp, info)) return;
+                int interp = d && index+numanimparts<=MAXANIMPARTS ? index+i : -1, aitime = animationinterpolationtime;
+                if(!calcanim(i, anim, basetime, basetime2, d, interp, info, aitime)) return;
                 animstate &p = as[i];
                 p.owner = this;
-                p.anim = info.anim;
                 p.cur.setframes(info);
                 p.interp = 1;
                 if(interp>=0 && d->animinterp[interp].prev.range>0)
                 {
                     int diff = lastmillis-d->animinterp[interp].lastswitch;
-                    if(diff<animationinterpolationtime)
+                    if(diff<aitime)
                     {
                         p.prev.setframes(d->animinterp[interp].prev);
-                        p.interp = diff/float(animationinterpolationtime);
+                        p.interp = diff/float(aitime);
                     }
                 }
             }
 
-            vec raxis(axis), rdir(dir), rcampos(campos);
-            float pitchamount = calcpitchaxis(anim, pitch, raxis, rdir, rcampos);
+            vec oaxis, oforward;
+            matrixstack[matrixpos].transposedtransformnormal(axis, oaxis);
+            float pitchamount = pitchscale*pitch + pitchoffset;
+            if(pitchmin || pitchmax) pitchamount = clamp(pitchamount, pitchmin, pitchmax);
+            if(as->cur.anim&ANIM_NOPITCH || (as->interp < 1 && as->prev.anim&ANIM_NOPITCH))
+                pitchamount *= (as->cur.anim&ANIM_NOPITCH ? 0 : as->interp) + (as->interp < 1 && as->prev.anim&ANIM_NOPITCH ? 0 : 1-as->interp);
             if(pitchamount)
             {
                 ++matrixpos;
                 matrixstack[matrixpos] = matrixstack[matrixpos-1];
-                matrixstack[matrixpos].rotate(pitchamount*RAD, axis);
+                matrixstack[matrixpos].rotate(pitchamount*RAD, oaxis);
             }
+            matrixstack[matrixpos].transposedtransformnormal(forward, oforward);
 
             if(!(anim&ANIM_NORENDER))
             {
@@ -839,15 +820,17 @@ struct animmodel : model
             {
                 if(renderpath!=R_FIXEDFUNCTION)
                 {
-                    setenvparamf("lightdir", SHPARAM_VERTEX, 0, rdir.x, rdir.y, rdir.z);
-                    setenvparamf("lightdir", SHPARAM_PIXEL, 0, rdir.x, rdir.y, rdir.z);
-                    vec ocampos(rcampos);
+                    vec odir, ocampos;
+                    matrixstack[matrixpos].transposedtransformnormal(lightdir, odir);
+                    setenvparamf("lightdir", SHPARAM_VERTEX, 0, odir.x, odir.y, odir.z);
+                    setenvparamf("lightdir", SHPARAM_PIXEL, 0, odir.x, odir.y, odir.z);
+                    matrixstack[matrixpos].transposedtransform(camera1->o, ocampos);
                     ocampos.div(model->scale).sub(translate);
                     setenvparamf("camera", SHPARAM_VERTEX, 1, ocampos.x, ocampos.y, ocampos.z, 1);
                 }
             }
 
-            meshes->render(as, pitch, axis, d, this);
+            meshes->render(as, pitch, oaxis, oforward, d, this);
 
             if(!(anim&ANIM_NORENDER))
             {
@@ -872,14 +855,6 @@ struct animmodel : model
                         continue;
                     }
 
-                    vec naxis(raxis), ndir(rdir), ncampos(rcampos);
-                    link.matrix.invertnormal(naxis);
-                    if(!(anim&(ANIM_NOSKIN|ANIM_NORENDER)))
-                    {
-                        link.matrix.invertnormal(ndir);
-                        link.matrix.invertvertex(ncampos);
-                    }
-
                     int nanim = anim, nbasetime = basetime, nbasetime2 = basetime2;
                     if(link.anim>=0)
                     {
@@ -887,7 +862,7 @@ struct animmodel : model
                         nbasetime = link.basetime;
                         nbasetime2 = 0;
                     }
-                    link.p->render(nanim, nbasetime, nbasetime2, pitch, naxis, d, ndir, ncampos);
+                    link.p->render(nanim, nbasetime, nbasetime2, pitch, axis, forward, d);
 
                     matrixpos--;
                 }
@@ -896,39 +871,23 @@ struct animmodel : model
             if(pitchamount) matrixpos--;
         }
 
-	int setanim(int animpart, int num, int frame, int range, float speed, int priority = 0)
+        void setanim(int animpart, int num, int frame, int range, float speed, int priority = 0)
         {
-            if(animpart<0 || animpart>=MAXANIMPARTS) return -1;
+            if(animpart<0 || animpart>=MAXANIMPARTS) return;
             if(frame<0 || range<=0 || !meshes || !meshes->hasframes(frame, range))
             {
                 conoutf("invalid frame %d, range %d in model %s", frame, range, model->loadname);
-                return -1;
+                return;
             }
-            //add/reserve vector elements to for standard animations
-			if (anims[animpart].length() == 0)
-			{
-				for (int i = 0; i < NUMANIMS; i++) { anims[animpart].add(); }
-			}
-			else if (num == NUMANIMS) //__offtools__ add new element for custom animation
-			{
-				anims[animpart].add();
-				num = anims[animpart].length()-1;
-			}
-			//add new animspec
-			animspec &spec = anims[animpart][num].spec.add();
+            if(!anims[animpart]) anims[animpart] = new vector<animspec>[NUMANIMS];
+            animspec &spec = anims[animpart][num].add();
             spec.frame = frame;
             spec.range = range;
             spec.speed = speed;
             spec.priority = priority;
-            return num;
         }
     };
 
-	int numanims()
-	{
-		return parts[0]->anims[0].length();
-	}
-
     enum
     {
         LINK_TAG = 0,
@@ -938,7 +897,7 @@ struct animmodel : model
 
     virtual int linktype(animmodel *m) const { return LINK_TAG; }
 
-    void render(int anim, int basetime, int basetime2, float pitch, const vec &axis, dynent *d, modelattach *a, const vec &dir, const vec &campos)
+    void render(int anim, int basetime, int basetime2, float pitch, const vec &axis, const vec &forward, dynent *d, modelattach *a)
     {
         if(!loaded) return;
 
@@ -975,7 +934,7 @@ struct animmodel : model
         }
 
         animstate as[MAXANIMPARTS];
-        parts[0]->render(anim, basetime, basetime2, pitch, axis, d, dir, campos, as);
+        parts[0]->render(anim, basetime, basetime2, pitch, axis, forward, d, as);
 
         if(a) for(int i = numtags-1; i >= 0; i--)
         {
@@ -994,12 +953,12 @@ struct animmodel : model
                     break;
 
                 case LINK_COOP:
-                    p->render(anim, basetime, basetime2, pitch, axis, d, dir, campos);
+                    p->render(anim, basetime, basetime2, pitch, axis, forward, d);
                     p->index = 0;
                     break;
 
                 case LINK_REUSE:
-                    p->render(anim | ANIM_REUSE, basetime, basetime2, pitch, axis, d, dir, campos, as);
+                    p->render(anim | ANIM_REUSE, basetime, basetime2, pitch, axis, forward, d, as);
                     break;
             }
         }
@@ -1009,11 +968,11 @@ struct animmodel : model
     {
         if(!loaded) return;
 
-        vec rdir, campos;
-
-        yaw += offsetyaw + spinyaw*lastmillis/1000.0f;
+        yaw += spinyaw*lastmillis/1000.0f;
         pitch += offsetpitch + spinpitch*lastmillis/1000.0f;
-	roll += offsetroll + spinroll*lastmillis/1000.0f;
+        roll += offsetroll + spinroll*lastmillis/1000.0f;
+
+        vec axis(0, -1, 0), forward(1, 0, 0);
 
         matrixpos = 0;
         matrixstack[0].identity();
@@ -1021,13 +980,16 @@ struct animmodel : model
         {
             matrixstack[0].translate(o);
             matrixstack[0].rotate_around_z(yaw*RAD);
-	    matrixstack[0].rotate_around_x(-roll * RAD);
+            matrixstack[0].rotate_around_x(-roll * RAD);
+            matrixstack[0].transformnormal(vec(axis), axis);
+            matrixstack[0].transformnormal(vec(forward), forward);
+            if(offsetyaw) matrixstack[0].rotate_around_z(offsetyaw*RAD);
         }
         else pitch = 0;
 
         if(anim&ANIM_NORENDER)
         {
-            render(anim, basetime, basetime2, pitch, vec(0, -1, 0), d, a, rdir, campos);
+            render(anim, basetime, basetime2, pitch, axis, forward, d, a);
             if(d) d->lastrendered = lastmillis;
             return;
         }
@@ -1041,17 +1003,9 @@ struct animmodel : model
             }
 
             transparent = trans;
+            lightdir = dir;
             lightcolor = color;
 
-            rdir = dir;
-            campos = camera1->o;
-            if(!d || !d->ragdoll || anim&ANIM_RAGDOLL)
-            {
-                rdir.rotate_around_z(-yaw*RAD);
-                campos.sub(o);
-                campos.rotate_around_z(-yaw*RAD);
-            }
-
             if(envmapped()) envmaptmu = 2;
             else if(a) for(int i = 0; a[i].tag; i++) if(a[i].m && a[i].m->envmapped())
             {
@@ -1072,7 +1026,7 @@ struct animmodel : model
             if(renderpath==R_FIXEDFUNCTION)
             {
                 glActiveTexture_(GL_TEXTURE0_ARB+envmaptmu);
-                setuptmu(envmaptmu, "T , P @ Pa", "= Ca");
+                setuptmu(envmaptmu, "T , P @ Pa", hasTEX || hasTE4 ? "Ca * $1a" : "= Ca");
 
                 glMatrixMode(GL_TEXTURE);
                 glLoadMatrixf(envmatrix.v);
@@ -1095,7 +1049,7 @@ struct animmodel : model
             else if(alphadepth)
             {
                 glColorMask(GL_FALSE, GL_FALSE, GL_FALSE, GL_FALSE);
-                render(anim|ANIM_NOSKIN, basetime, basetime2, pitch, vec(0, -1, 0), d, a, rdir, campos);
+                render(anim|ANIM_NOSKIN, basetime, basetime2, pitch, axis, forward, d, a);
                 glColorMask(GL_TRUE, GL_TRUE, GL_TRUE, fading ? GL_FALSE : GL_TRUE);
 
                 glDepthFunc(GL_LEQUAL);
@@ -1109,7 +1063,7 @@ struct animmodel : model
             }
         }
 
-        render(anim, basetime, basetime2, pitch, vec(0, -1, 0), d, a, rdir, campos);
+        render(anim, basetime, basetime2, pitch, axis, forward, d, a);
 
         if(envmaptmu>=0)
         {
@@ -1156,7 +1110,7 @@ struct animmodel : model
     {
         m.identity();
         if(offsetyaw) m.rotate_around_z(offsetyaw*RAD);
-	if(offsetroll) m.rotate_around_x(-offsetroll*RAD);
+        if(offsetroll) m.rotate_around_x(-offsetroll*RAD);
         if(offsetpitch) m.rotate_around_y(-offsetpitch*RAD);
     }
 
@@ -1244,10 +1198,16 @@ struct animmodel : model
         loopv(parts) loopvj(parts[i]->skins) parts[i]->skins[j].ambient = ambient;
     }
 
-    void setglow(float glow)
+    void setglow(float glow, float delta, float pulse)
     {
         if(parts.empty()) loaddefaultparts();
-        loopv(parts) loopvj(parts[i]->skins) parts[i]->skins[j].glow = glow;
+        loopv(parts) loopvj(parts[i]->skins)
+        {
+            skin &s = parts[i]->skins[j];
+            s.glow = glow;
+            s.glowdelta = delta;
+            s.glowpulse = pulse;
+        }
     }
 
     void setglare(float specglare, float glowglare)
@@ -1300,7 +1260,7 @@ struct animmodel : model
     }
 
     static bool enabletc, enablemtc, enablealphatest, enablealphablend, enableenvmap, enableglow, enableoverbright, enablelighting, enablelight0, enablecullface, enablenormals, enabletangents, enablebones, enablerescale, enabledepthoffset;
-    static vec lightcolor;
+    static vec lightdir, lightcolor;
     static float transparent, lastalphatest;
     static void *lastvbuf, *lasttcbuf, *lastmtcbuf, *lastnbuf, *lastxbuf, *lastbbuf, *lastsdata, *lastbdata;
     static GLuint lastebuf, lastenvmaptex, closestenvmaptex;
@@ -1431,7 +1391,7 @@ struct animmodel : model
 bool animmodel::enabletc = false, animmodel::enablemtc = false, animmodel::enablealphatest = false, animmodel::enablealphablend = false,
      animmodel::enableenvmap = false, animmodel::enableglow = false, animmodel::enableoverbright = false, animmodel::enablelighting = false, animmodel::enablelight0 = false, animmodel::enablecullface = true,
      animmodel::enablenormals = false, animmodel::enabletangents = false, animmodel::enablebones = false, animmodel::enablerescale = false, animmodel::enabledepthoffset = false;
-vec animmodel::lightcolor;
+vec animmodel::lightdir(0, 0, 1), animmodel::lightcolor(1, 1, 1);
 float animmodel::transparent = 1, animmodel::lastalphatest = -1;
 void *animmodel::lastvbuf = NULL, *animmodel::lasttcbuf = NULL, *animmodel::lastmtcbuf = NULL, *animmodel::lastnbuf = NULL, *animmodel::lastxbuf = NULL, *animmodel::lastbbuf = NULL, *animmodel::lastsdata = NULL, *animmodel::lastbdata = NULL;
 GLuint animmodel::lastebuf = 0, animmodel::lastenvmaptex = 0, animmodel::closestenvmaptex = 0;
@@ -1446,7 +1406,7 @@ template<class MDL> struct modelloader
 
     static bool animated() { return true; }
     static bool multiparted() { return true; }
-    static bool multimeshed() { return true; } 
+    static bool multimeshed() { return true; }
 };
 
 template<class MDL> MDL *modelloader<MDL>::loading = NULL;
@@ -1477,7 +1437,7 @@ template<class MDL, class MESH> struct modelcommands
         }
 
     #define loopskins(meshname, s, body) loopmeshes(meshname, m, { skin &s = mdl.skins[i]; body; })
-    
+
     static void setskin(char *meshname, char *tex, char *masks, float *envmapmax, float *envmapmin)
     {
         loopskins(meshname, s,
@@ -1490,7 +1450,7 @@ template<class MDL, class MESH> struct modelcommands
             }
         );
     }
-    
+
     static void setspec(char *meshname, int *percent)
     {
         float spec = 1.0f;
@@ -1498,7 +1458,7 @@ template<class MDL, class MESH> struct modelcommands
         else if(*percent<0) spec = 0.0f;
         loopskins(meshname, s, s.spec = spec);
     }
-    
+
     static void setambient(char *meshname, int *percent)
     {
         float ambient = 0.3f;
@@ -1506,41 +1466,42 @@ template<class MDL, class MESH> struct modelcommands
         else if(*percent<0) ambient = 0.0f;
         loopskins(meshname, s, s.ambient = ambient);
     }
-    
-    static void setglow(char *meshname, int *percent)
+
+    static void setglow(char *meshname, int *percent, int *delta, float *pulse)
     {
-        float glow = 3.0f;
+        float glow = 3.0f, glowdelta = *delta/100.0f, glowpulse = *pulse > 0 ? *pulse/1000.0f : 0;
         if(*percent>0) glow = *percent/100.0f;
         else if(*percent<0) glow = 0.0f;
-        loopskins(meshname, s, s.glow = glow);
+        glowdelta -= glow;
+        loopskins(meshname, s, {s.glow = glow; s.glowdelta = glowdelta; s.glowpulse = glowpulse;});
     }
-    
+
     static void setglare(char *meshname, float *specglare, float *glowglare)
     {
         loopskins(meshname, s, { s.specglare = *specglare; s.glowglare = *glowglare; });
     }
-    
+
     static void setalphatest(char *meshname, float *cutoff)
     {
         loopskins(meshname, s, s.alphatest = max(0.0f, min(1.0f, *cutoff)));
     }
-    
+
     static void setalphablend(char *meshname, int *blend)
     {
         loopskins(meshname, s, s.alphablend = *blend!=0);
     }
-    
+
     static void setcullface(char *meshname, int *cullface)
     {
         loopskins(meshname, s, s.cullface = *cullface!=0);
     }
-    
+
     static void setenvmap(char *meshname, char *envmap)
     {
         Texture *tex = cubemapload(envmap);
         loopskins(meshname, s, s.envmap = tex);
     }
-    
+
     static void setbumpmap(char *meshname, char *normalmapfile, char *skinfile)
     {
         Texture *normalmaptex = NULL, *skintex = NULL;
@@ -1548,34 +1509,34 @@ template<class MDL, class MESH> struct modelcommands
         if(skinfile[0]) skintex = textureload(makerelpath(MDL::dir, skinfile, "<noff>"), 0, true, false);
         loopskins(meshname, s, { s.unlittex = skintex; s.normalmap = normalmaptex; m.calctangents(); });
     }
-    
+
     static void setfullbright(char *meshname, float *fullbright)
     {
         loopskins(meshname, s, s.fullbright = *fullbright);
     }
-    
+
     static void setshader(char *meshname, char *shader)
     {
         loopskins(meshname, s, s.shader = lookupshaderbyname(shader));
     }
-    
+
     static void setscroll(char *meshname, float *scrollu, float *scrollv)
     {
         loopskins(meshname, s, { s.scrollu = *scrollu; s.scrollv = *scrollv; });
     }
-    
+
     static void setnoclip(char *meshname, int *noclip)
     {
         loopmeshes(meshname, m, m.noclip = *noclip!=0);
     }
-  
+
     static void setlink(int *parent, int *child, char *tagname, float *x, float *y, float *z)
     {
         if(!MDL::loading) { conoutf("not loading an %s", MDL::formatname()); return; }
         if(!MDL::loading->parts.inrange(*parent) || !MDL::loading->parts.inrange(*child)) { conoutf("no models loaded to link"); return; }
         if(!MDL::loading->parts[*parent]->link(MDL::loading->parts[*child], tagname, vec(*x, *y, *z))) conoutf("could not link model %s", MDL::loading->loadname);
     }
- 
+
     template<class F> void modelcommand(F *fun, const char *suffix, const char *args)
     {
         defformatstring(name)("%s%s", MDL::formatname(), suffix);
@@ -1590,7 +1551,7 @@ template<class MDL, class MESH> struct modelcommands
             modelcommand(setskin, "skin", "sssff");
             modelcommand(setspec, "spec", "si");
             modelcommand(setambient, "ambient", "si");
-            modelcommand(setglow, "glow", "si");
+            modelcommand(setglow, "glow", "siif");
             modelcommand(setglare, "glare", "sff");
             modelcommand(setalphatest, "alphatest", "sf");
             modelcommand(setalphablend, "alphablend", "si");
