@@ -300,7 +300,7 @@ ident *newident(const char *name, int flags)
     {
         if(isinteger(name))
         {
-            debugcode("integer %s is not a valid identifier name", name);
+            debugcode("number %s is not a valid identifier name", name);
             return newident("//dummy", IDF_UNKNOWN);
         }
         ident init(ID_ALIAS, newstring(name), flags);
@@ -341,7 +341,7 @@ static void setalias(const char *name, tagval &v)
     }
     else if(isinteger(name))
     {
-        debugcode("cannot alias integer %s", name);
+        debugcode("cannot alias number %s", name);
         freearg(v);
     }
     else
@@ -1366,14 +1366,14 @@ static const uint *runcode(const uint *code, tagval &result)
             case CODE_IDENTU:
             {
                 tagval &arg = args[numargs-1];
-		ident *id = newident(arg.type == VAL_STR || arg.type == VAL_MACRO ? arg.s : "//dummy", IDF_UNKNOWN);
+                ident *id = newident(arg.type == VAL_STR || arg.type == VAL_MACRO ? arg.s : "//dummy", IDF_UNKNOWN);
                 freearg(arg);
                 arg.setident(id);
                 continue;
             }
 
             case CODE_LOOKUPU|RET_STR:
-                #define LOOKUPU(aval, sval, ival, fval) { \
+                #define LOOKUPU(aval, sval, ival, fval, nval) { \
                     tagval &arg = args[numargs-1]; \
                     if(arg.type != VAL_STR && arg.type != VAL_MACRO) continue; \
                     id = idents.access(arg.s); \
@@ -1384,14 +1384,16 @@ static const uint *runcode(const uint *code, tagval &result)
                         case ID_VAR: freearg(arg); ival; continue; \
                         case ID_FVAR: freearg(arg); fval; continue; \
                     } \
-                    if(arg.type == VAL_MACRO) arg.setstr(newstring(arg.s, arg.code[-1]>>8)); \
                     debugcode("unknown alias lookup: %s", arg.s); \
+                    freearg(arg); \
+                    nval; \
                     continue; \
                 }
                 LOOKUPU(arg.setstr(newstring(id->getstr())),
                         arg.setstr(newstring(*id->storage.s)),
                         arg.setstr(newstring(intstr(*id->storage.i))),
-                        arg.setstr(newstring(floatstr(*id->storage.f))));
+                        arg.setstr(newstring(floatstr(*id->storage.f))),
+                        arg.setstr(newstring("")));
             case CODE_LOOKUP|RET_STR:
                 #define LOOKUP(aval) { \
                     id = identmap[op>>8]; \
@@ -1404,21 +1406,24 @@ static const uint *runcode(const uint *code, tagval &result)
                 LOOKUPU(arg.setint(id->getint()),
                         arg.setint(parseint(*id->storage.s)),
                         arg.setint(*id->storage.i),
-                        arg.setint(int(*id->storage.f)));
+                        arg.setint(int(*id->storage.f)),
+                        arg.setint(0));
             case CODE_LOOKUP|RET_INT:
                 LOOKUP(args[numargs++].setint(id->getint()));
             case CODE_LOOKUPU|RET_FLOAT:
                 LOOKUPU(arg.setfloat(id->getfloat()),
                         arg.setfloat(parsefloat(*id->storage.s)),
                         arg.setfloat(float(*id->storage.i)),
-                        arg.setfloat(*id->storage.f));
+                        arg.setfloat(*id->storage.f),
+                        arg.setfloat(0.0f));
             case CODE_LOOKUP|RET_FLOAT:
                 LOOKUP(args[numargs++].setfloat(id->getfloat()));
             case CODE_LOOKUPU|RET_NULL:
                 LOOKUPU(id->getval(arg),
                         arg.setstr(newstring(*id->storage.s)),
                         arg.setint(*id->storage.i),
-                        arg.setfloat(*id->storage.f));
+                        arg.setfloat(*id->storage.f),
+                        arg.setnull());
             case CODE_LOOKUP|RET_NULL:
                 LOOKUP(id->getval(args[numargs++]));
 
@@ -1547,16 +1552,13 @@ static const uint *runcode(const uint *code, tagval &result)
                     _numargs = oldargs; \
                     numargs = 0; \
                 }
+                forcenull(result);
                 id = identmap[op>>8];
                 if(id->flags&IDF_UNKNOWN)
                 {
                     debugcode("unknown command: %s", id->name);
-                    freearg(result);
-                    freeargs(args, numargs, 0);
-                    result.setstr(newstring(id->name));
-                    continue;
+                    goto forceresult;
                 }
-                forcenull(result);
                 CALLALIAS(0);
                 continue;
 
@@ -1566,8 +1568,10 @@ static const uint *runcode(const uint *code, tagval &result)
                 if(!id)
                 {
                 noid:
-                    if(!isinteger(args[0].s)) debugcode("unknown command: %s", args[0].s);
-                    goto litval;
+                    if(isinteger(args[0].s)) goto litval;
+                    debugcode("unknown command: %s", args[0].s);
+                    forcenull(result);
+                    goto forceresult;
                 }
                 forcenull(result);
                 switch(id->type)
@@ -1770,7 +1774,7 @@ static bool validatealias(const char *s)
 void writecfg(const char *name)
 {
     defformatstring(confname)("config_%s.cfg", game::gameident());
-    stream *f = openfile(path((name && name[0]) ? name : confname, true), "w");
+    stream *f = openutf8file(path((name && name[0]) ? name : confname, true), "wb");
 
     if(!f) return;
     f->printf("// automatically written on exit, DO NOT MODIFY\n// delete this file to have data/defaults.cfg and data/%s/defaults.cfg overwrite these settings\n// modify settings in game, or put settings in %s to override anything\n\n", game::gameident(), game::autoexec());
@@ -2012,11 +2016,15 @@ void substr(char *s, int *start, char *count)
 void sublist(char *s, int *start, char *count)
 {
     int offset = max(*start, 0), len = count[0] ? max(parseint(count), 0) : -1;
-    loopi(offset)
+    if(offset > 0)
     {
+        loopi(offset)
+        {
+            whitespaceskip;
+            elementskip;
+            if(!*s) break;
+        }
         whitespaceskip;
-        elementskip;
-        if(!*s) break;
     }
     if(len < 0) { commandret->setstr(newstring(s)); return; }
     const char *e = s;
@@ -2142,9 +2150,9 @@ char *listdel(const char *s, const char *del)
     whitespaceskip;
     while(*s)
     {
-        const char *elem = s;
+        const char *elem = s, *pelem = elem;
         elementskip;
-        int len = s-elem;
+        int len = s-elem, plen = len;
         if(*elem=='"')
         {
             elem++;
@@ -2153,7 +2161,7 @@ char *listdel(const char *s, const char *del)
         if(listincludes(del, elem, len) < 0)
         {
             if(!p.empty()) p.add(' ');
-            p.put(elem, len);
+            p.put(pelem, plen);
         }
         whitespaceskip;
     }
@@ -2322,6 +2330,8 @@ ICOMMAND(maxf, "V", (tagval *args, int numargs),
     loopi(numargs - 1) val = max(val, args[i].getfloat());
     floatret(val);
 });
+ICOMMAND(abs, "i", (int *n), intret(abs(*n)));
+ICOMMAND(absf, "f", (float *n), floatret(fabs(*n)));
 
 ICOMMAND(cond, "ee2V", (tagval *args, int numargs),
 {
