@@ -2,10 +2,10 @@
 #define __ENGINE_H__
 
 #include "cube.h"
+#include "world.h"
 
 #ifndef STANDALONE
 
-#include "world.h"
 #include "octa.h"
 #include "lightmap.h"
 #include "bih.h"
@@ -21,7 +21,7 @@
 #endif
 #endif
 
-#define CONFVERSION 8
+#define CONFVERSION 9
 
 // GL_ARB_multitexture
 extern PFNGLACTIVETEXTUREARBPROC       glActiveTexture_;
@@ -116,6 +116,7 @@ extern char *maptitle;
 extern vector<ushort> texmru;
 extern int xtraverts, xtravertsva;
 extern const ivec cubecoords[8];
+extern const ivec facecoords[6][4];
 extern const uchar fv[6][4];
 extern const uchar fvmasks[64];
 extern const uchar faceedgesidx[6][4];
@@ -131,14 +132,13 @@ struct font
 {
     struct charinfo
     {
-        short x, y, w, h;
+        short x, y, w, h, offsetx, offsety, advance, tex;
     };
 
     char *name;
-    Texture *tex;
+    vector<Texture *> texs;
     vector<charinfo> chars;
     int charoffset, defaultw, defaulth;
-    int offsetx, offsety, offsetw, offseth;
 };
 
 #define FONTH (curfont->defaulth)
@@ -239,15 +239,17 @@ extern void writecrosshairs(stream *f);
 extern void render3dbox(vec &o, float tofloor, float toceil, float xradius, float yradius = 0);
 
 // octa
-extern cube *newcubes(uint face = F_EMPTY);
-extern cubeext *newcubeext(cube &c);
+extern cube *newcubes(uint face = F_EMPTY, int mat = MAT_AIR);
+extern cubeext *growcubeext(cubeext *ext, int maxverts);
+extern void setcubeext(cube &c, cubeext *ext);
+extern cubeext *newcubeext(cube &c, int maxverts = 0, bool init = true);
 extern void getcubevector(cube &c, int d, int x, int y, int z, ivec &p);
 extern void setcubevector(cube &c, int d, int x, int y, int z, const ivec &p);
 extern int familysize(cube &c);
 extern void freeocta(cube *c);
 extern void discardchildren(cube &c, bool fixtex = false, int depth = 0);
 extern void optiface(uchar *p, cube &c);
-extern void validatec(cube *c, int size);
+extern void validatec(cube *c, int size = 0);
 extern bool isvalidcube(cube &c);
 extern ivec lu;
 extern int lusize;
@@ -255,40 +257,35 @@ extern cube &lookupcube(int tx, int ty, int tz, int tsize = 0, ivec &ro = lu, in
 extern cube *neighbourstack[32];
 extern int neighbourdepth;
 extern cube &neighbourcube(cube &c, int orient, int x, int y, int z, int size, ivec &ro = lu, int &rsize = lusize);
-extern void newclipplanes(cube &c);
-extern void freeclipplanes(cube &c);
+extern void resetclipplanes();
 extern int getmippedtexture(cube &p, int orient);
 extern void forcemip(cube &c, bool fixtex = true);
 extern bool subdividecube(cube &c, bool fullcheck=true, bool brighten=true);
 extern void converttovectorworld();
-extern int faceverts(cube &c, int orient, int vert);
+extern int faceconvexity(ivec v[4]);
+extern int faceconvexity(ivec v[4], int &vis);
+extern int faceconvexity(vertinfo *verts, int numverts);
 extern int faceconvexity(cube &c, int orient);
 extern void calcvert(cube &c, int x, int y, int z, int size, ivec &vert, int i, bool solid = false);
 extern void calcvert(cube &c, int x, int y, int z, int size, vec &vert, int i, bool solid = false);
 extern uint faceedges(cube &c, int orient);
-extern bool collapsedface(uint cfe);
+extern bool collapsedface(cube &c, int orient);
 extern bool touchingface(cube &c, int orient);
 extern bool flataxisface(cube &c, int orient);
+extern bool collideface(cube &c, int orient);
 extern int genclipplane(cube &c, int i, vec *v, plane *clip);
 extern void genclipplanes(cube &c, int x, int y, int z, int size, clipplanes &p);
 extern bool visibleface(cube &c, int orient, int x, int y, int z, int size, uchar mat = MAT_AIR, uchar nmat = MAT_AIR, uchar matmask = MATF_VOLUME);
-extern int visibletris(cube &c, int orient, int x, int y, int z, int size);
+extern int visibletris(cube &c, int orient, int x, int y, int z, int size, uchar nmat = MAT_AIR, uchar matmask = MAT_AIR);
 extern int visibleorient(cube &c, int orient);
 extern bool threeplaneintersect(plane &pl1, plane &pl2, plane &pl3, vec &dest);
-extern void genvectorvert(const ivec &p, cube &c, ivec &v);
-extern void freemergeinfo(cube &c);
-extern void genmergedverts(cube &cu, int orient, const ivec &co, int size, const mergeinfo &m, vec *vv, plane *p = NULL);
-extern int calcmergedsize(int orient, const ivec &co, int size, const mergeinfo &m, const vec *vv);
-extern void invalidatemerges(cube &c, bool msg);
+extern void genfaceverts(cube &c, int orient, ivec v[4]);
+extern int calcmergedsize(int orient, const ivec &co, int size, const vertinfo *verts, int numverts);
+extern void invalidatemerges(cube &c, const ivec &co, int size, bool msg);
 extern void calcmerges();
 
-struct cubeface : mergeinfo
-{
-    cube *c;
-};
-
-extern int mergefaces(int orient, cubeface *m, int sz);
-extern void mincubeface(cube &cu, int orient, const ivec &o, int size, const mergeinfo &orig, mergeinfo &cf, uchar nmat = MAT_AIR, uchar matmask = MATF_VOLUME);
+extern int mergefaces(int orient, facebounds *m, int sz);
+extern void mincubeface(cube &cu, int orient, const ivec &o, int size, const facebounds &orig, facebounds &cf, uchar nmat = MAT_AIR, uchar matmask = MATF_VOLUME);
 
 static inline uchar octantrectangleoverlap(const ivec &c, int size, const ivec &o, const ivec &s)
 {
@@ -304,6 +301,11 @@ static inline uchar octantrectangleoverlap(const ivec &c, int size, const ivec &
     return p;
 }
 
+static inline cubeext &ext(cube &c)
+{
+    return *(c.ext ? c.ext : newcubeext(c));
+}
+
 // ents
 extern char *entname(entity &e);
 extern bool haveselent();
@@ -319,6 +321,12 @@ extern void rendereditcursor();
 extern void tryedit();
 
 // octarender
+extern vector<tjoint> tjoints;
+
+extern ushort encodenormal(const vec &n);
+extern vec decodenormal(ushort norm);
+extern void reduceslope(ivec &n);
+extern void findtjoints();
 extern void octarender();
 extern void allchanged(bool load = false);
 extern void clearvas(cube *c);
@@ -371,7 +379,7 @@ extern bool getdynlight(int n, vec &o, float &radius, vec &color);
 extern int showmat;
 
 extern int findmaterial(const char *name);
-extern void genmatsurfs(cube &c, int cx, int cy, int cz, int size, vector<materialsurface> &matsurfs, uchar &vismask, uchar &clipmask);
+extern void genmatsurfs(cube &c, int cx, int cy, int cz, int size, vector<materialsurface> &matsurfs);
 extern void rendermatsurfs(materialsurface *matbuf, int matsurfs);
 extern void rendermatgrid(materialsurface *matbuf, int matsurfs);
 extern int optimizematsurfs(materialsurface *matbuf, int matsurfs);
@@ -484,6 +492,7 @@ extern void renderprogress(float bar, const char *text, GLuint tex = 0, bool bac
 
 extern void getfps(int &fps, int &bestdiff, int &worstdiff);
 extern void swapbuffers();
+extern int getclockmillis();
 
 // menu
 #ifndef NEWGUI
@@ -581,8 +590,8 @@ extern void g3d_mainmenu();
 namespace UI
 {
     extern void resetcursor();
-    extern bool movecursor(int dx, int dy);
-    extern bool hascursor(bool targeting = false);
+    extern bool movecursor(int &dx, int &dy);
+    extern bool hascursor();
     extern void getcursorpos(float &x, float &y);
     extern bool keypress(int code, bool isdown, int cooked);
     extern void setup();
