@@ -123,10 +123,10 @@ void drawmaterial(int orient, int x, int y, int z, int csize, int rsize, float o
     int dim = dimension(orient), c = C[dim], r = R[dim];
     loopi(4)
     {
-        int coord = fv[orient][i];
+        const ivec &cc = facecoords[orient][i];
         vec v(x, y, z);
-        v[c] += cubecoords[coord][c]/8*csize;
-        v[r] += cubecoords[coord][r]/8*rsize;
+        v[c] += cc[c]/8*csize;
+        v[r] += cc[r]/8*rsize;
         v[dim] += dimcoord(orient) ? -offset : offset;
         varray::attribv<3>(v.v);
     }
@@ -185,7 +185,7 @@ int visiblematerial(cube &c, int orient, int x, int y, int z, int size, uchar ma
     return MATSURF_NOT_VISIBLE;
 }
 
-void genmatsurfs(cube &c, int cx, int cy, int cz, int size, vector<materialsurface> &matsurfs, uchar &vismask, uchar &clipmask)
+void genmatsurfs(cube &c, int cx, int cy, int cz, int size, vector<materialsurface> &matsurfs)
 {
     loopi(6)
     {
@@ -204,12 +204,6 @@ void genmatsurfs(cube &c, int cx, int cy, int cz, int size, vector<materialsurfa
                 m.csize = m.rsize = size;
                 if(dimcoord(i)) m.o[dimension(i)] += size;
                 matsurfs.add(m);
-                if(isclipped(c.material&matmask))
-                {
-                    clipmask |= 1<<i;
-                    if(vis == MATSURF_VISIBLE) vismask |= 1<<i;
-                    else vismask &= ~(1<<i);
-                }
                 break;
             }
         }
@@ -342,6 +336,7 @@ void setupmaterials(int start, int len)
     for(int i = start; i < len; i++)
     {
         vtxarray *va = valist[i];
+        materialsurface *skip = NULL;
         loopj(va->matsurfs)
         {
             materialsurface &m = va->matbuf[j];
@@ -408,6 +403,11 @@ void setupmaterials(int start, int len)
                 }
             }
             if(m.material&MATF_VOLUME) hasmat |= 1<<m.material;
+            m.skip = 0;
+            if(skip && m.material == skip->material && m.orient == skip->orient && skip->skip < 0xFF)
+                skip->skip++;
+            else 
+                skip = &m;
         }
     }
     loopv(water)
@@ -497,9 +497,9 @@ void sortmaterials(vector<materialsurface *> &vismats)
             materialsurface &m = va->matbuf[i];
             if(!editmode || !showmat || envmapping)
             {
-                if(m.material==MAT_WATER && (m.orient==O_TOP || (refracting<0 && reflectz>worldsize))) continue;
-                if(m.flags&materialsurface::F_EDIT) continue;
-                if(glaring && m.material!=MAT_LAVA) continue;
+                if(m.material==MAT_WATER && (m.orient==O_TOP || (refracting<0 && reflectz>worldsize))) { i += m.skip; continue; }
+                if(m.flags&materialsurface::F_EDIT) { i += m.skip; continue; }
+                if(glaring && m.material!=MAT_LAVA) { i += m.skip; continue; }
             }
             else if(glaring) continue;
             vismats.add(&m);
@@ -554,10 +554,10 @@ void drawglass(int orient, int x, int y, int z, int csize, int rsize, float offs
     int dim = dimension(orient), c = C[dim], r = R[dim];
     loopi(4)
     {
-        int coord = fv[orient][i];
+        const ivec &cc = facecoords[orient][i];
         vec v(x, y, z);
-        v[c] += cubecoords[coord][c]/8*csize;
-        v[r] += cubecoords[coord][r]/8*rsize;
+        v[c] += cc[c]/8*csize;
+        v[r] += cc[r]/8*rsize;
         v[dim] += dimcoord(orient) ? -offset : offset;
 
         vec reflect(v);
