@@ -221,6 +221,14 @@ void guionclear(char *action)
     if(action[0]) m->onclear = compilecode(action);
 }
 
+void guifont(const char *name, uint *contents)
+{
+    pushfont();
+    setfont(name);
+    execute(contents);
+    popfont();
+}
+
 void guistayopen(uint *contents)
 {
     bool oldclearmenu = shouldclearmenu;
@@ -530,6 +538,7 @@ COMMAND(guiservers, "s");
 ICOMMAND(cleargui, "i", (int *n), intret(cleargui(*n)));
 COMMAND(showgui, "s");
 COMMAND(guionclear, "s");
+COMMAND(guifont, "se");
 COMMAND(guistayopen, "e");
 COMMAND(guinoautotab, "e");
 
