@@ -511,7 +511,7 @@ void readychanges(block3 &b, cube *c, const ivec &cor, int size)
                 int hasmerges = c[i].ext->va->hasmerges;
                 destroyva(c[i].ext->va);
                 c[i].ext->va = NULL;
-                if(hasmerges) invalidatemerges(c[i], true);
+                if(hasmerges) invalidatemerges(c[i], o, size, true);
             }
             freeoctaentities(c[i]);
             c[i].ext->tjoints = -1;
@@ -572,6 +572,7 @@ static inline void copycube(const cube &src, cube &dst)
 {
     dst = src;
     dst.visible = 0;
+    dst.collide = 0;
     dst.merged = 0;
     dst.ext = NULL; // src cube is responsible for va destruction
     if(src.children)
@@ -1769,7 +1770,6 @@ void filltexlist()
         loopv(vslots) if(texmru.find(i)<0) texmru.add(i);
     }
 }
-COMMAND(filltexlist, "");
 
 void compactmruvslots()
 {
@@ -1835,7 +1835,7 @@ void gettex()
 
 void getcurtex()
 {
-    if(noedit()) return;
+    if(noedit(true)) return;
     filltexlist();
     int index = curtexindex < 0 ? 0 : curtexindex;
     if(!texmru.inrange(index)) return;
@@ -1852,13 +1852,25 @@ void getseltex()
 
 void gettexname(int *tex, int *subslot)
 {
-    if(!editmode || *tex<0) return;
-    VSlot &vslot = lookupvslot(*tex);
+    if(noedit(true) || *tex<0) return;
+    VSlot &vslot = lookupvslot(*tex, false);
     Slot &slot = *vslot.slot;
     if(!slot.sts.inrange(*subslot)) return;
     result(slot.sts[*subslot].name);
 }
 
+ICOMMAND(texmrunum, "", (),
+    filltexlist();
+    intret(texmru.length())
+);
+ICOMMAND(texmruresolve, "i", (int *slot),
+    filltexlist();
+    if(texmru.inrange(*slot))
+        intret(texmru[*slot]);
+    else
+        intret(*slot);
+);
+ICOMMAND(settex, "i", (int *slot), edittex(*slot));
 COMMANDN(edittex, edittex_, "i");
 COMMAND(gettex, "");
 COMMAND(getcurtex, "");
@@ -2189,7 +2201,7 @@ void setmat(cube &c, uchar mat, uchar matmask, uchar filtermat, uchar filtermask
         loopi(8) setmat(c.children[i], mat, matmask, filtermat, filtermask);
     else if((c.material&filtermask) == filtermat)
     {
-        if(mat!=MAT_AIR) 
+        if(mat!=MAT_AIR)
         {
             c.material &= matmask;
             c.material |= mat;
@@ -2290,15 +2302,15 @@ struct texturegui : g3d_callback
                             lastthumbnail = totalmillis;
                         }
                         int test = g.texture(vslot, 1.0f, true);
-			if(test & G3D_UP && (slot.loaded || slot.thumbnail!=notexture))
+                        if(test & G3D_UP && (slot.loaded || slot.thumbnail!=notexture))
                             edittex(vslot.index);
-			if(test & G3D_ROLLOVER)
-			{
-				if(!slot.loaded && !slot.thumbnail)
-					loadthumbnail(slot);
+                        if(test & G3D_ROLLOVER)
+                        {
+                            if(!slot.loaded && !slot.thumbnail)
+                                loadthumbnail(slot);
 
-				texhoveridx = ti;
-			}
+                            texhoveridx = ti;
+                        }
                     }
                     else
                     {
@@ -2308,38 +2320,38 @@ struct texturegui : g3d_callback
                 g.poplist();
             }
 
-		Slot &prev = lookupslot(texhoveridx, false);
-		VSlot &vprev = *prev.variants;
+            Slot &prev = lookupslot(texhoveridx, false);
+            VSlot &vprev = *prev.variants;
 
-		g.pushlist();
-		g.texture(vprev, 2.0f, true);
+            g.pushlist();
+            g.texture(vprev, 2.0f, true);
 
-		extern int guitextcolour;
-		g.pushlist();
-		defformatstring(ds)("Texture: %s", prev.sts[0].name);
-		if(ds[60]) //shorten strings, to avoid a jittery interface
-		{
-			ds[60] = '\0';
-			ds[59] = ds[58] = ds[57] = '.';
-		}
-		g.text(ds, guitextcolour, "info");
+            extern int guitextcolour;
+            g.pushlist();
+            defformatstring(ds)("Texture: %s", prev.sts[0].name);
+            if(ds[60]) //shorten strings, to avoid a jittery interface
+            {
+                ds[60] = '\0';
+                ds[59] = ds[58] = ds[57] = '.';
+            }
+            g.text(ds, guitextcolour, "info");
 
-		formatstring(ds)("layer: %i\tShader: %s", vprev.layer, prev.shader->name);
-		if(ds[60]) //shorten strings, to avoid a jittery interface
-		{
-			ds[60] = '\0';
-			ds[59] = ds[58] = ds[57] = '.';
-		}
-		g.text(ds, guitextcolour, "info");
+            formatstring(ds)("layer: %i\tShader: %s", vprev.layer, prev.shader->name);
+            if(ds[60]) //shorten strings, to avoid a jittery interface
+            {
+                ds[60] = '\0';
+                ds[59] = ds[58] = ds[57] = '.';
+            }
+            g.text(ds, guitextcolour, "info");
 
-		formatstring(ds)("Scale: %.2f\tAlpha %.2f %.2f", vprev.scale, vprev.alphafront, vprev.alphaback);
-		g.text(ds, guitextcolour, "info");
+            formatstring(ds)("Scale: %.2f\tAlpha %.2f %.2f", vprev.scale, vprev.alphafront, vprev.alphaback);
+            g.text(ds, guitextcolour, "info");
 
-		formatstring(ds)("Scroll: %.2f %.2f\toffset %i %i", vprev.scrollS * 1000.0f, vprev.scrollT * 1000.0f, vprev.xoffset, vprev.yoffset);
-		g.text(ds, guitextcolour, "info");
+            formatstring(ds)("Scroll: %.2f %.2f\toffset %i %i", vprev.scrollS * 1000.0f, vprev.scrollT * 1000.0f, vprev.xoffset, vprev.yoffset);
+            g.text(ds, guitextcolour, "info");
 
-		g.poplist();
-		g.poplist();
+            g.poplist();
+            g.poplist();
         }
         g.end();
     }
