@@ -13,7 +13,7 @@ static int clipcacheversion = 0;
 static inline clipplanes &getclipplanes(cube &c, const ivec &o, int size)
 {
     clipplanes &p = clipcache[int(&c - worldroot)&(MAXCLIPPLANES-1)];
-    if(p.owner != &c || p.version != clipcacheversion) 
+    if(p.owner != &c || p.version != clipcacheversion)
     {
         p.owner = &c;
         p.version = clipcacheversion;
@@ -701,43 +701,11 @@ bool plcollide(physent *d, const vec &dir)    // collide with player or monster
     return true;
 }
 
-#define MMROT(x, y) \
-    vec2(x, y), vec2(x, y), vec2(x, y), vec2(x, y), vec2(x, y), vec2(x, y), vec2(x, y), vec2(x, y), \
-    vec2(x, y), vec2(x, y), vec2(x, y), vec2(x, y), vec2(x, y), vec2(x, y), vec2(x, y)
-extern const vec2 mmrots[((360/15)+1)*15] =
-{
-    MMROT(1.00000000, 0.00000000), // 0
-    MMROT(0.96592583, 0.25881905), // 15
-    MMROT(0.86602540, 0.50000000), // 30
-    MMROT(0.70710678, 0.70710678), // 45
-    MMROT(0.50000000, 0.86602540), // 60
-    MMROT(0.25881905, 0.96592583), // 75
-    MMROT(0.00000000, 1.00000000), // 90
-    MMROT(-0.25881905, 0.96592583), // 105
-    MMROT(-0.50000000, 0.86602540), // 120
-    MMROT(-0.70710678, 0.70710678), // 135
-    MMROT(-0.86602540, 0.50000000), // 150
-    MMROT(-0.96592583, 0.25881905), // 165
-    MMROT(-1.00000000, 0.00000000), // 180
-    MMROT(-0.96592583, -0.25881905), // 195
-    MMROT(-0.86602540, -0.50000000), // 210
-    MMROT(-0.70710678, -0.70710678), // 225
-    MMROT(-0.50000000, -0.86602540), // 240
-    MMROT(-0.25881905, -0.96592583), // 255
-    MMROT(-0.00000000, -1.00000000), // 270
-    MMROT(0.25881905, -0.96592583), // 285
-    MMROT(0.50000000, -0.86602540), // 300
-    MMROT(0.70710678, -0.70710678), // 315
-    MMROT(0.86602540, -0.50000000), // 330
-    MMROT(0.96592583, -0.25881905), // 345
-    MMROT(1.00000000, 0.00000000) // 360
-};
-
 void rotatebb(vec &center, vec &radius, int yaw)
 {
     if(yaw < 0) yaw = 360 + yaw%360;
     else if(yaw >= 360) yaw %= 360;
-    const vec2 &rot = mmrots[yaw + 7];
+    const vec2 &rot = sincos360[yaw];
     vec2 oldcenter(center), oldradius(radius);
     center.x = oldcenter.x*rot.x - oldcenter.y*rot.y;
     center.y = oldcenter.y*rot.x + oldcenter.x*rot.y;
@@ -772,7 +740,7 @@ bool mmcollide(physent *d, const vec &dir, octaentities &oc)               // co
         if(!m || !m->collide) continue;
         vec center, radius;
         m->collisionbox(0, center, radius);
-        float yaw = float((e.attr[0]+7)-(e.attr[0]+7)%15);
+        float yaw = e.attr[0];
         switch(d->collidetype)
         {
             case COLLIDE_ELLIPSE:
@@ -812,8 +780,8 @@ static bool fuzzycollidesolid(physent *d, const vec &dir, float cutoff, cube &c,
     E entvol(d);
     wall = vec(0, 0, 0);
     float bestdist = -1e10f;
-    int vis = isentirelysolid(c) ? c.visible : 0xFF;
-    loopi(6) if(vis&(1<<i))
+    int visible = isentirelysolid(c) ? c.collide : 0xFF;
+    loopi(6) if(visible&(1<<i))
      {
         int dim = dimension(i), dc = dimcoord(i), dimdir = 2*dc - 1;
         plane w(0, 0, 0, -(dimdir*co[dim] + dc*size));
@@ -842,6 +810,33 @@ static bool fuzzycollidesolid(physent *d, const vec &dir, float cutoff, cube &c,
     return false;
 }
 
+template<class E>
+static inline bool clampcollide(const clipplanes &p, const E &entvol, const plane &w, const vec &pw)
+{
+    if(w.x && (w.y || w.z) && fabs(pw.x - p.o.x) > p.r.x)
+    {
+        vec c = entvol.center();
+        float fv = pw.x < p.o.x ? p.o.x-p.r.x : p.o.x+p.r.x, fdist = (w.x*fv + w.y*c.y + w.z*c.z + w.offset) / (w.y*w.y + w.z*w.z);
+        vec fdir(fv - c.x, -w.y*fdist, -w.z*fdist);
+        if(entvol.supportpoint(fdir).squaredist(c) < fdir.squaredlen()) return true;
+    }
+    if(w.y && (w.x || w.z) && fabs(pw.y - p.o.y) > p.r.y)
+    {
+        vec c = entvol.center();
+        float fv = pw.y < p.o.y ? p.o.y-p.r.y : p.o.y+p.r.y, fdist = (w.x*c.x + w.y*fv + w.z*c.z + w.offset) / (w.x*w.x + w.z*w.z);
+        vec fdir(-w.x*fdist, fv - c.y, -w.z*fdist);
+        if(entvol.supportpoint(fdir).squaredist(c) < fdir.squaredlen()) return true;
+    }
+    if(w.z && (w.x || w.y) && fabs(pw.z - p.o.z) > p.r.z)
+    {
+        vec c = entvol.center();
+        float fv = pw.z < p.o.z ? p.o.z-p.r.z : p.o.z+p.r.z, fdist = (w.x*c.x + w.y*c.y + w.z*fv + w.offset) / (w.x*w.x + w.y*w.y);
+        vec fdir(-w.x*fdist, -w.y*fdist, fv - c.z);
+        if(entvol.supportpoint(fdir).squaredist(c) < fdir.squaredlen()) return true;
+    }
+    return false;
+}
+    
 template<class E>
 static bool fuzzycollideplanes(physent *d, const vec &dir, float cutoff, cube &c, const ivec &co, int size) // collide with deformed cube geometry
 {
@@ -875,6 +870,7 @@ static bool fuzzycollideplanes(physent *d, const vec &dir, float cutoff, cube &c
         wall = w;
         bestdist = dist;
     }
+    int bestplane = -1;
     loopi(p.size)
     {
         plane &w = p.p[i];
@@ -882,6 +878,8 @@ static bool fuzzycollideplanes(physent *d, const vec &dir, float cutoff, cube &c
         float dist = w.dist(pw);
         if(dist >= 0) return true;
         if(dist <= bestdist) continue;
+        bestplane = -1;
+        bestdist = dist;
         if(!dir.iszero())
         {
             if(w.dot(dir) >= -cutoff*dir.magnitude()) continue;
@@ -891,10 +889,11 @@ static bool fuzzycollideplanes(physent *d, const vec &dir, float cutoff, cube &c
                     ((dir.x*w.x < 0 || dir.y*w.y < 0) ? -d->radius : 0)))
                 continue;
         }
-        wall = w;
-        bestdist = dist;
+        if(clampcollide(p, entvol, w, pw)) continue;
+        bestplane = i;
     }
-    if(wall.iszero())
+    if(bestplane >= 0) wall = p.p[bestplane];
+    else if(wall.iszero())
     {
         inside = true;
         return true;
@@ -916,8 +915,8 @@ static bool cubecollidesolid(physent *d, const vec &dir, float cutoff, cube &c,
 
     wall = vec(0, 0, 0);
     float bestdist = -1e10f;
-    int vis = isentirelysolid(c) ? c.visible : 0xFF;
-    loopi(6) if(vis&(1<<i))
+    int visible = isentirelysolid(c) ? c.collide : 0xFF;
+    loopi(6) if(visible&(1<<i))
      {
         int dim = dimension(i), dc = dimcoord(i), dimdir = 2*dc - 1;
         plane w(0, 0, 0, -(dimdir*co[dim] + dc*size));
@@ -934,7 +933,6 @@ static bool cubecollidesolid(physent *d, const vec &dir, float cutoff, cube &c,
                     ((dir.x*w.x < 0 || dir.y*w.y < 0) ? -d->radius : 0)))
                 continue;
         }
-        //wall.add(w);
         wall = w;
         bestdist = dist;
     }
@@ -943,7 +941,6 @@ static bool cubecollidesolid(physent *d, const vec &dir, float cutoff, cube &c,
         inside = true;
         return true;
     }
-    //wall.normalize();
     return false;
 }
 
@@ -979,16 +976,18 @@ static bool cubecollideplanes(physent *d, const vec &dir, float cutoff, cube &c,
                     ((dir.x*w.x < 0 || dir.y*w.y < 0) ? -d->radius : 0)))
                 continue;
         }
-        //wall.add(w);
         wall = w;
         bestdist = dist;
     }
+    int bestplane = -1;
     loopi(p.size)
     {
         plane &w = p.p[i];
         vec pw = entvol.supportpoint(vec(w).neg());
         float dist = w.dist(pw);
         if(dist <= bestdist) continue;
+        bestplane = -1;
+        bestdist = dist;
         if(!dir.iszero())
         {
             if(w.dot(dir) >= -cutoff*dir.magnitude()) continue;
@@ -998,16 +997,15 @@ static bool cubecollideplanes(physent *d, const vec &dir, float cutoff, cube &c,
                     ((dir.x*w.x < 0 || dir.y*w.y < 0) ? -d->radius : 0)))
                 continue;
         }
-        //wall.add(w);
-        wall = w;
-        bestdist = dist;
+        if(clampcollide(p, entvol, w, pw)) continue;
+        bestplane = i;
     }
-    if(wall.iszero())
+    if(bestplane >= 0) wall = p.p[bestplane];
+    else if(wall.iszero())
     {
         inside = true;
         return true;
     }
-    //wall.normalize();
     return false;
 }
 
@@ -1021,7 +1019,7 @@ static inline bool cubecollide(physent *d, const vec &dir, float cutoff, cube &c
             if(cutoff <= 0)
             {
                 int crad = size/2;
-                return rectcollide(d, dir, vec(co.x + crad, co.y + crad, co.z), crad, crad, size, 0, isentirelysolid(c) ? c.visible : 0xFF);
+                return rectcollide(d, dir, vec(co.x + crad, co.y + crad, co.z), crad, crad, size, 0, isentirelysolid(c) ? c.collide : 0xFF);
             }
 #if 0
             else return cubecollidesolid<mpr::EntAABB>(d, dir, cutoff, c, co, size);
@@ -1646,7 +1644,7 @@ void vectoyawpitch(const vec &v, float &yaw, float &pitch)
 
 VARP(maxroll, 0, 3, 20);
 FVAR(straferoll, 0, 0.033f, 90);
-VAR(floatspeed, 10, 100, 1000);
+VAR(floatspeed, 10, 100, 10000);
 
 void modifyvelocity(physent *pl, bool local, bool water, bool floating, int curtime)
 {
