@@ -137,7 +137,6 @@ VARP(ati_skybox_bug, 0, 0, 1);
 VAR(ati_oq_bug, 0, 0, 1);
 VAR(ati_minmax_bug, 0, 0, 1);
 VAR(ati_dph_bug, 0, 0, 1);
-VAR(ati_teximage_bug, 0, 0, 1);
 VAR(ati_line_bug, 0, 0, 1);
 VAR(ati_cubemap_bug, 0, 0, 1);
 VAR(ati_ubo_bug, 0, 0, 1);
@@ -172,6 +171,16 @@ static bool checkseries(const char *s, int low, int high)
 
 VAR(dbgexts, 0, 0, 1);
 
+bool hasext(const char *exts, const char *ext)
+{
+    int len = strlen(ext);
+    for(const char *cur = exts; (cur = strstr(cur, ext)); cur += len)
+    {
+        if((cur == exts || cur[-1] == ' ') && (cur[len] == ' ' || !cur[len])) return true;
+    }
+    return false;
+}
+
 void gl_checkextensions()
 {
     const char *vendor = (const char *)glGetString(GL_VENDOR);
@@ -192,18 +201,18 @@ void gl_checkextensions()
     // char *weakcards[] = { "GeForce FX", "Quadro FX", "6200", "9500", "9550", "9600", "9700", "9800", "X300", "X600", "FireGL", "Intel", "Chrome", NULL }
     // if(shaderprecision==2) for(char **wc = weakcards; *wc; wc++) if(strstr(renderer, *wc)) shaderprecision = 1;
 
-    if(strstr(exts, "GL_EXT_texture_env_combine") || strstr(exts, "GL_ARB_texture_env_combine"))
+    if(hasext(exts, "GL_EXT_texture_env_combine") || hasext(exts, "GL_ARB_texture_env_combine"))
     {
         hasTE = true;
-        if(strstr(exts, "GL_ARB_texture_env_crossbar")) hasTEX = true;
-        if(strstr(exts, "GL_ATI_texture_env_combine3")) hasTE3 = true;
-        if(strstr(exts, "GL_NV_texture_env_combine4")) hasTE4 = true;
-        if(strstr(exts, "GL_EXT_texture_env_dot3") || strstr(exts, "GL_ARB_texture_env_dot3")) hasD3 = true;
+        if(hasext(exts, "GL_ARB_texture_env_crossbar")) hasTEX = true;
+        if(hasext(exts, "GL_ATI_texture_env_combine3")) hasTE3 = true;
+        if(hasext(exts, "GL_NV_texture_env_combine4")) hasTE4 = true;
+        if(hasext(exts, "GL_EXT_texture_env_dot3") || hasext(exts, "GL_ARB_texture_env_dot3")) hasD3 = true;
         if(dbgexts) conoutf(CON_INIT, "Using GL_ARB_texture_env_combine extension.");
     }
     else conoutf(CON_WARN, "WARNING: No texture_env_combine extension! (your video card is WAY too old)");
 
-    if(strstr(exts, "GL_ARB_multitexture"))
+    if(hasext(exts, "GL_ARB_multitexture"))
     {
         glActiveTexture_       = (PFNGLACTIVETEXTUREARBPROC)      getprocaddress("glActiveTextureARB");
         glClientActiveTexture_ = (PFNGLCLIENTACTIVETEXTUREARBPROC)getprocaddress("glClientActiveTextureARB");
@@ -216,7 +225,7 @@ void gl_checkextensions()
     else conoutf(CON_WARN, "WARNING: No multitexture extension!");
 
 
-    if(strstr(exts, "GL_ARB_vertex_buffer_object"))
+    if(hasext(exts, "GL_ARB_vertex_buffer_object"))
     {
         hasVBO = true;
         if(dbgexts) conoutf(CON_INIT, "Using GL_ARB_vertex_buffer_object extension.");
@@ -228,7 +237,7 @@ void gl_checkextensions()
     if(osversion < 0x1060) maxvbosize = min(maxvbosize, 8192);
 #endif
 
-    if(strstr(exts, "GL_ARB_pixel_buffer_object"))
+    if(hasext(exts, "GL_ARB_pixel_buffer_object"))
     {
         hasPBO = true;
         if(dbgexts) conoutf(CON_INIT, "Using GL_ARB_pixel_buffer_object extension.");
@@ -246,14 +255,14 @@ void gl_checkextensions()
         glGetBufferSubData_ = (PFNGLGETBUFFERSUBDATAARBPROC)getprocaddress("glGetBufferSubDataARB");
     }
 
-    if(strstr(exts, "GL_EXT_draw_range_elements"))
+    if(hasext(exts, "GL_EXT_draw_range_elements"))
     {
         glDrawRangeElements_ = (PFNGLDRAWRANGEELEMENTSEXTPROC)getprocaddress("glDrawRangeElementsEXT");
         hasDRE = true;
         if(dbgexts) conoutf(CON_INIT, "Using GL_EXT_draw_range_elements extension.");
     }
 
-    if(strstr(exts, "GL_EXT_multi_draw_arrays"))
+    if(hasext(exts, "GL_EXT_multi_draw_arrays"))
     {
         glMultiDrawArrays_   = (PFNGLMULTIDRAWARRAYSEXTPROC)  getprocaddress("glMultiDrawArraysEXT");
         glMultiDrawElements_ = (PFNGLMULTIDRAWELEMENTSEXTPROC)getprocaddress("glMultiDrawElementsEXT");
@@ -265,7 +274,7 @@ void gl_checkextensions()
     // floating point FBOs not fully supported until 10.5
     if(osversion>=0x1050)
 #endif
-    if(strstr(exts, "GL_ARB_texture_float") || strstr(exts, "GL_ATI_texture_float"))
+    if(hasext(exts, "GL_ARB_texture_float") || hasext(exts, "GL_ATI_texture_float"))
     {
         hasTF = true;
         if(dbgexts) conoutf(CON_INIT, "Using GL_ARB_texture_float extension.");
@@ -274,13 +283,13 @@ void gl_checkextensions()
         smoothshadowmappeel = 1;
     }
 
-    if(strstr(exts, "GL_NV_float_buffer"))
+    if(hasext(exts, "GL_NV_float_buffer"))
     {
         hasNVFB = true;
         if(dbgexts) conoutf(CON_INIT, "Using GL_NV_float_buffer extension.");
     }
 
-    if(strstr(exts, "GL_EXT_framebuffer_object"))
+    if(hasext(exts, "GL_EXT_framebuffer_object"))
     {
         glBindRenderbuffer_        = (PFNGLBINDRENDERBUFFEREXTPROC)       getprocaddress("glBindRenderbufferEXT");
         glDeleteRenderbuffers_     = (PFNGLDELETERENDERBUFFERSEXTPROC)    getprocaddress("glDeleteRenderbuffersEXT");
@@ -296,7 +305,7 @@ void gl_checkextensions()
         hasFBO = true;
         if(dbgexts) conoutf(CON_INIT, "Using GL_EXT_framebuffer_object extension.");
 
-        if(strstr(exts, "GL_EXT_framebuffer_blit"))
+        if(hasext(exts, "GL_EXT_framebuffer_blit"))
         {
             glBlitFramebuffer_     = (PFNGLBLITFRAMEBUFFEREXTPROC)        getprocaddress("glBlitFramebufferEXT");
             hasFBB = true;
@@ -305,7 +314,11 @@ void gl_checkextensions()
     }
     else conoutf(CON_WARN, "WARNING: No framebuffer object support. (reflective water may be slow)");
 
-    if(strstr(exts, "GL_ARB_occlusion_query"))
+#if defined(__APPLE__)
+    // Intel HD3000 broke occlusion queries - either causing software fallback, or returning wrong results
+    if(!strstr(vendor, "Intel"))
+#endif
+    if(hasext(exts, "GL_ARB_occlusion_query"))
     {
         GLint bits;
         glGetQueryiv_ = (PFNGLGETQUERYIVARBPROC)getprocaddress("glGetQueryivARB");
@@ -323,7 +336,7 @@ void gl_checkextensions()
 #if defined(__APPLE__) && SDL_BYTEORDER == SDL_BIG_ENDIAN
             if(strstr(vendor, "ATI") && (osversion<0x1050)) ati_oq_bug = 1;
 #endif
-            if(ati_oq_bug) conoutf(CON_WARN, "WARNING: Using ATI occlusion query bug workaround. (use \"/ati_oq_bug 0\" to disable if unnecessary)");
+            //if(ati_oq_bug) conoutf(CON_WARN, "WARNING: Using ATI occlusion query bug workaround. (use \"/ati_oq_bug 0\" to disable if unnecessary)");
         }
     }
     if(!hasOQ)
@@ -334,7 +347,7 @@ void gl_checkextensions()
         waterreflect = 0;
     }
 
-    extern int reservedynlighttc, reserveshadowmaptc, batchlightmaps, ffdynlights;
+    extern int reservedynlighttc, reserveshadowmaptc, batchlightmaps, ffdynlights, fpdepthfx;
     if(strstr(vendor, "ATI"))
     {
         //conoutf(CON_WARN, "WARNING: ATI cards may show garbage in skybox. (use \"/ati_skybox_bug 1\" to fix)");
@@ -343,25 +356,14 @@ void gl_checkextensions()
         reserveshadowmaptc = 3;
         minimizetcusage = 1;
         emulatefog = 1;
-        extern int depthfxprecision;
-        if(hasTF) depthfxprecision = 1;
-
-#if 0
-        //causes problems with Catalyst AI advanced setting, hope this is fixed by now - 11-21-09
-#if !defined(WIN32) && !defined(__APPLE__)
-        // reported on ATI Radeon HD 4800, Gentoo Linux kernel 2.6.26, Catalyst 9.3, 4-29-09, driver overreads memory on mipmapped GL_RGB textures for base level once max level is specified
-        // ... doesn't seem to affect Radeon X1300 on Catalyst 9.3, however
-        // TODO: verify if this is fixed in newer Catalyst releases
-        if(strstr(renderer, "Radeon HD")) ati_teximage_bug = 1;
-#endif
-#endif
+		if(hasTF && hasNVFB) fpdepthfx = 1;
     }
     else if(strstr(vendor, "NVIDIA"))
     {
         reservevpparams = 10;
         rtsharefb = 0; // work-around for strange driver stalls involving when using many FBOs
         extern int filltjoints;
-        if(!strstr(exts, "GL_EXT_gpu_shader4")) filltjoints = 0; // DX9 or less NV cards seem to not cause many sparklies
+        if(!hasext(exts, "GL_EXT_gpu_shader4")) filltjoints = 0; // DX9 or less NV cards seem to not cause many sparklies
 
         if(hasFBO && !hasTF) nvidia_scissor_bug = 1; // 5200 bug, clearing with scissor on an FBO messes up on reflections, may affect lesser cards too
         extern int fpdepthfx;
@@ -394,7 +396,7 @@ void gl_checkextensions()
         if(!hasOQ) waterrefract = 0;
     }
 
-    if(strstr(exts, "GL_ARB_vertex_program") && strstr(exts, "GL_ARB_fragment_program"))
+    if(hasext(exts, "GL_ARB_vertex_program") && hasext(exts, "GL_ARB_fragment_program"))
     {
         hasVP = hasFP = true;
         glGenPrograms_ =              (PFNGLGENPROGRAMSARBPROC)              getprocaddress("glGenProgramsARB");
@@ -415,12 +417,12 @@ void gl_checkextensions()
         if(osversion>=0x1050) // fixed in 1055 for some hardware.. but not all..
         {
             apple_ff_bug = 1;
-            conoutf(CON_WARN, "WARNING: Using Leopard ARB_position_invariant bug workaround. (use \"/apple_ff_bug 0\" to disable if unnecessary)");
+            //conoutf(CON_WARN, "WARNING: Using Leopard ARB_position_invariant bug workaround. (use \"/apple_ff_bug 0\" to disable if unnecessary)");
         }
 #endif
     }
 
-    if(strstr(exts, "GL_ARB_shading_language_100") && strstr(exts, "GL_ARB_shader_objects") && strstr(exts, "GL_ARB_vertex_shader") && strstr(exts, "GL_ARB_fragment_shader"))
+    if(hasext(exts, "GL_ARB_shading_language_100") && hasext(exts, "GL_ARB_shader_objects") && hasext(exts, "GL_ARB_vertex_shader") && hasext(exts, "GL_ARB_fragment_shader"))
     {
         glCreateProgramObject_ =        (PFNGLCREATEPROGRAMOBJECTARBPROC)     getprocaddress("glCreateProgramObjectARB");
         glDeleteObject_ =               (PFNGLDELETEOBJECTARBPROC)            getprocaddress("glDeleteObjectARB");
@@ -460,7 +462,7 @@ void gl_checkextensions()
             //if(osversion<0x1050) ??
             if(hasVP && hasFP) apple_glsldepth_bug = 1;
 #endif
-            if(apple_glsldepth_bug) conoutf(CON_WARN, "WARNING: Using Apple GLSL depth bug workaround. (use \"/apple_glsldepth_bug 0\" to disable if unnecessary");
+            //if(apple_glsldepth_bug) conoutf(CON_WARN, "WARNING: Using Apple GLSL depth bug workaround. (use \"/apple_glsldepth_bug 0\" to disable if unnecessary");
         }
     }
 
@@ -471,10 +473,10 @@ void gl_checkextensions()
         if(!avoidshaders) matskel = 0;
     }
 
-    if(strstr(exts, "GL_NV_vertex_program2_option")) { usevp2 = 1; hasVP2 = true; }
-    if(strstr(exts, "GL_NV_vertex_program3")) { usevp3 = 1; hasVP3 = true; }
+    if(hasext(exts, "GL_NV_vertex_program2_option")) { usevp2 = 1; hasVP2 = true; }
+    if(hasext(exts, "GL_NV_vertex_program3")) { usevp3 = 1; hasVP3 = true; }
 
-    if(strstr(exts, "GL_EXT_gpu_program_parameters"))
+    if(hasext(exts, "GL_EXT_gpu_program_parameters"))
     {
         glProgramEnvParameters4fv_   = (PFNGLPROGRAMENVPARAMETERS4FVEXTPROC)  getprocaddress("glProgramEnvParameters4fvEXT");
         glProgramLocalParameters4fv_ = (PFNGLPROGRAMLOCALPARAMETERS4FVEXTPROC)getprocaddress("glProgramLocalParameters4fvEXT");
@@ -482,7 +484,7 @@ void gl_checkextensions()
         if(dbgexts) conoutf(CON_INIT, "Using GL_EXT_gpu_program_parameters extension.");
     }
 
-    if(strstr(exts, "GL_ARB_uniform_buffer_object"))
+    if(hasext(exts, "GL_ARB_uniform_buffer_object"))
     {
         glGetUniformIndices_       = (PFNGLGETUNIFORMINDICESPROC)      getprocaddress("glGetUniformIndices");
         glGetActiveUniformsiv_     = (PFNGLGETACTIVEUNIFORMSIVPROC)    getprocaddress("glGetActiveUniformsiv");
@@ -497,7 +499,7 @@ void gl_checkextensions()
         if(strstr(vendor, "ATI")) ati_ubo_bug = 1;
         if(dbgexts) conoutf(CON_INIT, "Using GL_ARB_uniform_buffer_object extension.");
     }
-    else if(strstr(exts, "GL_EXT_bindable_uniform"))
+    else if(hasext(exts, "GL_EXT_bindable_uniform"))
     {
         glUniformBuffer_        = (PFNGLUNIFORMBUFFEREXTPROC)       getprocaddress("glUniformBufferEXT");
         glGetUniformBufferSize_ = (PFNGLGETUNIFORMBUFFERSIZEEXTPROC)getprocaddress("glGetUniformBufferSizeEXT");
@@ -509,7 +511,7 @@ void gl_checkextensions()
         if(dbgexts) conoutf(CON_INIT, "Using GL_EXT_bindable_uniform extension.");
     }
 
-    if(strstr(exts, "GL_EXT_texture_rectangle") || strstr(exts, "GL_ARB_texture_rectangle"))
+    if(hasext(exts, "GL_EXT_texture_rectangle") || hasext(exts, "GL_ARB_texture_rectangle"))
     {
         usetexrect = 1;
         hasTR = true;
@@ -517,13 +519,13 @@ void gl_checkextensions()
     }
     else if(hasMT && hasshaders) conoutf(CON_WARN, "WARNING: No texture rectangle support. (no full screen shaders)");
 
-    if(strstr(exts, "GL_EXT_packed_depth_stencil") || strstr(exts, "GL_NV_packed_depth_stencil"))
+    if(hasext(exts, "GL_EXT_packed_depth_stencil") || hasext(exts, "GL_NV_packed_depth_stencil"))
     {
         hasDS = true;
         if(dbgexts) conoutf(CON_INIT, "Using GL_EXT_packed_depth_stencil extension.");
     }
 
-    if(strstr(exts, "GL_EXT_blend_minmax"))
+    if(hasext(exts, "GL_EXT_blend_minmax"))
     {
         glBlendEquation_ = (PFNGLBLENDEQUATIONEXTPROC) getprocaddress("glBlendEquationEXT");
         hasBE = true;
@@ -531,21 +533,21 @@ void gl_checkextensions()
         if(dbgexts) conoutf(CON_INIT, "Using GL_EXT_blend_minmax extension.");
     }
 
-    if(strstr(exts, "GL_EXT_blend_color"))
+    if(hasext(exts, "GL_EXT_blend_color"))
     {
         glBlendColor_ = (PFNGLBLENDCOLOREXTPROC) getprocaddress("glBlendColorEXT");
         hasBC = true;
         if(dbgexts) conoutf(CON_INIT, "Using GL_EXT_blend_color extension.");
     }
 
-    if(strstr(exts, "GL_EXT_fog_coord"))
+    if(hasext(exts, "GL_EXT_fog_coord"))
     {
         glFogCoordPointer_ = (PFNGLFOGCOORDPOINTEREXTPROC) getprocaddress("glFogCoordPointerEXT");
         hasFC = true;
         if(dbgexts) conoutf(CON_INIT, "Using GL_EXT_fog_coord extension.");
     }
 
-    if(strstr(exts, "GL_ARB_texture_cube_map"))
+    if(hasext(exts, "GL_ARB_texture_cube_map"))
     {
         GLint val;
         glGetIntegerv(GL_MAX_CUBE_MAP_TEXTURE_SIZE_ARB, &val);
@@ -558,14 +560,14 @@ void gl_checkextensions()
     else conoutf(CON_WARN, "WARNING: No cube map texture support. (no reflective glass)");
 
     extern int usenp2;
-    if(strstr(exts, "GL_ARB_texture_non_power_of_two"))
+    if(hasext(exts, "GL_ARB_texture_non_power_of_two"))
     {
         hasNP2 = true;
         if(dbgexts) conoutf(CON_INIT, "Using GL_ARB_texture_non_power_of_two extension.");
     }
     else if(usenp2) conoutf(CON_WARN, "WARNING: Non-power-of-two textures not supported!");
 
-    if(strstr(exts, "GL_ARB_texture_compression") && strstr(exts, "GL_EXT_texture_compression_s3tc"))
+    if(hasext(exts, "GL_ARB_texture_compression") && hasext(exts, "GL_EXT_texture_compression_s3tc"))
     {
         glCompressedTexImage3D_ =    (PFNGLCOMPRESSEDTEXIMAGE3DARBPROC)   getprocaddress("glCompressedTexImage3DARB");
         glCompressedTexImage2D_ =    (PFNGLCOMPRESSEDTEXIMAGE2DARBPROC)   getprocaddress("glCompressedTexImage2DARB");
@@ -579,7 +581,7 @@ void gl_checkextensions()
         if(dbgexts) conoutf(CON_INIT, "Using GL_EXT_texture_compression_s3tc extension.");
     }
 
-    if(strstr(exts, "GL_EXT_texture_filter_anisotropic"))
+    if(hasext(exts, "GL_EXT_texture_filter_anisotropic"))
     {
        GLint val;
        glGetIntegerv(GL_MAX_TEXTURE_MAX_ANISOTROPY_EXT, &val);
@@ -588,36 +590,36 @@ void gl_checkextensions()
        if(dbgexts) conoutf(CON_INIT, "Using GL_EXT_texture_filter_anisotropic extension.");
     }
 
-    if(strstr(exts, "GL_SGIS_generate_mipmap"))
+    if(hasext(exts, "GL_SGIS_generate_mipmap"))
     {
         hasGM = true;
         if(dbgexts) conoutf(CON_INIT, "Using GL_SGIS_generate_mipmap extension.");
     }
 
-    if(strstr(exts, "GL_ARB_depth_texture"))
+    if(hasext(exts, "GL_ARB_depth_texture"))
     {
         hasSGIDT = hasDT = true;
         if(dbgexts) conoutf(CON_INIT, "Using GL_ARB_depth_texture extension.");
     }
-    else if(strstr(exts, "GL_SGIX_depth_texture"))
+    else if(hasext(exts, "GL_SGIX_depth_texture"))
     {
         hasSGIDT = true;
         if(dbgexts) conoutf(CON_INIT, "Using GL_SGIX_depth_texture extension.");
     }
 
-    if(strstr(exts, "GL_ARB_shadow"))
+    if(hasext(exts, "GL_ARB_shadow"))
     {
         hasSGISH = hasSH = true;
         if(strstr(vendor, "NVIDIA") || strstr(renderer, "Radeon HD")) hasNVPCF = true;
         if(dbgexts) conoutf(CON_INIT, "Using GL_ARB_shadow extension.");
     }
-    else if(strstr(exts, "GL_SGIX_shadow"))
+    else if(hasext(exts, "GL_SGIX_shadow"))
     {
         hasSGISH = true;
         if(dbgexts) conoutf(CON_INIT, "Using GL_SGIX_shadow extension.");
     }
 
-    if(strstr(exts, "GL_EXT_rescale_normal"))
+    if(hasext(exts, "GL_EXT_rescale_normal"))
     {
         hasRN = true;
         if(dbgexts) conoutf(CON_INIT, "Using GL_EXT_rescale_normal extension.");
@@ -625,7 +627,7 @@ void gl_checkextensions()
 
     if(!hasSGIDT && !hasSGISH) shadowmap = 0;
 
-    if(strstr(exts, "GL_EXT_gpu_shader4") && !avoidshaders)
+    if(hasext(exts, "GL_EXT_gpu_shader4") && !avoidshaders)
     {
         // on DX10 or above class cards (i.e. GF8 or RadeonHD) enable expensive features
         extern int grass, glare, maxdynlights, depthfxsize, depthfxrect, depthfxfilter, blurdepthfx;
@@ -653,7 +655,7 @@ void gl_checkextensions()
 void glext(char *ext)
 {
     const char *exts = (const char *)glGetString(GL_EXTENSIONS);
-    intret(strstr(exts, ext) ? 1 : 0);
+    intret(hasext(exts, ext) ? 1 : 0);
 }
 COMMAND(glext, "s");
 
@@ -728,6 +730,9 @@ void cleanupgl()
 
 VAR(wireframe, 0, 0, 1);
 
+ICOMMAND(getcamyaw, "", (), floatret(camera1->yaw));
+ICOMMAND(getcampitch, "", (), floatret(camera1->pitch));
+ICOMMAND(getcamroll, "", (), floatret(camera1->roll));
 ICOMMAND(getcampos, "", (),
 {
     defformatstring(pos)("%s %s %s", floatstr(camera1->o.x), floatstr(camera1->o.y), floatstr(camera1->o.z));
@@ -738,9 +743,28 @@ vec worldpos, camdir, camright, camup;
 
 void findorientation()
 {
-    vecfromyawpitch(camera1->yaw, camera1->pitch, 1, 0, camdir);
-    vecfromyawpitch(camera1->yaw, 0, 0, -1, camright);
-    vecfromyawpitch(camera1->yaw, camera1->pitch+90, 1, 0, camup);
+    float yaw, pitch;
+    #ifdef NEWGUI
+    if(editmode)
+    {
+        float x, y;
+        UI::getcursorpos(x, y);
+
+        vec dir1 = invmvpmatrix.perspectivetransform(vec(x*2-1, 1-2*y, 2-1)),
+        dir2 = invmvpmatrix.perspectivetransform(vec(x*2-1, 1-2*y, -1));
+        camdir = dir1.sub(dir2).normalize();
+        vectoyawpitch(camdir, yaw, pitch);
+    }
+    else
+    #endif
+    {
+        yaw = camera1->yaw;
+        pitch = camera1->pitch;
+        vecfromyawpitch(yaw, pitch, 1, 0, camdir);
+    }
+
+    vecfromyawpitch(yaw, 0, 0, -1, camright);
+    vecfromyawpitch(yaw, pitch+90, 1, 0, camup);
 
     if(raycubepos(camera1->o, camdir, worldpos, 0, RAY_CLIPMAT|RAY_SKIPFIRST) == -1)
         worldpos = vec(camdir).mul(2*worldsize).add(camera1->o); //otherwise 3dgui won't work when outside of map
@@ -1152,9 +1176,10 @@ static float findsurface(int fogmat, const vec &v, int &abovemat)
     do
     {
         cube &c = lookupcube(o.x, o.y, o.z, 0, co, csize);
-        if(!c.ext || (c.ext->material&MATF_VOLUME) != fogmat)
+        int mat = c.material&MATF_VOLUME;
+        if(mat != fogmat)
         {
-            abovemat = c.ext && isliquid(c.ext->material&MATF_VOLUME) ? c.ext->material&MATF_VOLUME : MAT_AIR;
+            abovemat = isliquid(mat) ? mat : MAT_AIR;
             return o.z;
         }
         o.z = co.z + csize;
@@ -1491,8 +1516,8 @@ void drawcubemap(int size, const vec &o, float yaw, float pitch, const cubemapsi
     cmcamera.yaw = yaw;
     cmcamera.pitch = pitch;
     cmcamera.roll = 0;
-    setviewcell(camera1->o);
     camera1 = &cmcamera;
+    setviewcell(camera1->o);
 
     defaultshader->set();
 
@@ -1576,7 +1601,7 @@ void clipminimap(ivec &bbmin, ivec &bbmax, cube *c = worldroot, int x = 0, int y
     {
         ivec o(i, x, y, z, size);
         if(c[i].children) clipminimap(bbmin, bbmax, c[i].children, o.x, o.y, o.z, size>>1);
-        else if(!isentirelysolid(c[i]) && (!c[i].ext || (c[i].ext->material&MATF_CLIP)!=MAT_CLIP))
+        else if(!isentirelysolid(c[i]) && (c[i].material&MATF_CLIP)!=MAT_CLIP)
         {
             loopk(3) bbmin[k] = min(bbmin[k], o[k]);
             loopk(3) bbmax[k] = max(bbmax[k], o[k] + size);
@@ -2089,11 +2114,11 @@ SVARP(guicursortex, "hud/2.3/guicursor");
 COMMANDN(loadcrosshair, loadcrosshair_, "si");
 
 ICOMMAND(getcrosshair, "i", (int *i),
-    const char *name = "data/crosshair.png";
+    const char *name = "packages/crosshair/default.png";
     if(crosshairs.inrange(*i))
     {
         name = crosshairs[*i] ? crosshairs[*i]->name : game::defaultcrosshair(*i);
-        if(!name) name = "data/crosshair.png";
+	if(!name) name = "packages/crosshair/default.png";
     }
     result(name);
 )
@@ -2109,11 +2134,11 @@ void writecrosshairs(stream *f)
 void drawcrosshair(int w, int h)
 {
     bool windowhit =
-    	#ifdef NEWGUI
-    	UI::hascursor(false);
-    	#else
-    	g3d_windowhit(true, false);
-    	#endif
+    #ifdef NEWGUI
+    UI::hascursor();
+    #else
+    g3d_windowhit(true, false);
+    #endif
 
     if(!windowhit && hidehud) return; //(hidehud || player->state==CS_SPECTATOR || player->state==CS_DEAD)) return;
 
@@ -2125,6 +2150,11 @@ void drawcrosshair(int w, int h)
 
     float r = 1, g = 1, b = 1, cx = 0.5f, cy = 0.5f, chsize;
     Texture *crosshair;
+
+    #ifdef NEWGUI
+    UI::getcursorpos(cx, cy);
+    #endif
+
     if(windowhit)
     {
         defformatstring(tname)("data/%s", guicursortex);
@@ -2132,9 +2162,8 @@ void drawcrosshair(int w, int h)
         cursor = textureload(tname, 3, true);
         crosshair = cursor;
         chsize = cursorsize*w/900.0f;
-        #ifdef NEWGUI
-        UI::getcursorpos(cx, cy);
-        #else
+
+        #ifndef NEWGUI
         g3d_cursorpos(cx, cy);
         #endif
     }
@@ -2147,9 +2176,9 @@ void drawcrosshair(int w, int h)
             r = g = b = 1;
         }
         if(!crosshairimg)
-	{
-		index = 0;
-	}
+        {
+            index = 0;
+        }
         crosshair = crosshairs.inrange(index) ? crosshairs[index] : NULL;
         if(!crosshair)
         {
@@ -2346,8 +2375,14 @@ void gl_drawhud(int w, int h)
                 char *editinfo = executeret("edithud");
                 if(editinfo)
                 {
-                    abovehud -= 2*FONTH;
-                    draw_text(editinfo, FONTH/2, abovehud);
+                    if(editinfo[0])
+                    {
+                        int tw, th;
+                        text_bounds(editinfo, tw, th);
+                        th += FONTH-1; th -= th%FONTH;
+                        abovehud -= max(th, FONTH);
+                        draw_text(editinfo, FONTH/2, abovehud);
+                    }
                     DELETEA(editinfo);
                 }
             }
@@ -2356,9 +2391,15 @@ void gl_drawhud(int w, int h)
                 char *gameinfo = executeret("gamehud");
                 if(gameinfo)
                 {
-                    draw_text(gameinfo, conw-max(5*FONTH, 2*FONTH+text_width(gameinfo)), conh-FONTH*3/2-roffset);
+                    if(gameinfo[0])
+                    {
+                        int tw, th;
+                        text_bounds(gameinfo, tw, th);
+                        th += FONTH-1; th -= th%FONTH;
+                        roffset += max(th, FONTH);
+                        draw_text(gameinfo, conw-max(5*FONTH, 2*FONTH+tw), conh-FONTH/2-roffset);
+                    }
                     DELETEA(gameinfo);
-                    roffset += FONTH;
                 }
             }
             glPopMatrix();
@@ -2388,16 +2429,16 @@ extern int scr_w, scr_h;
 
 bool resettextures()
 {
-	if(
-		reloadtexture("data/loadingscreen/engine_badge.png")  &&
-		reloadtexture("data/loadingscreen/load_back.png")     &&
-		reloadtexture("data/loadingscreen/load_bar.png")      &&
-		reloadtexture("data/loadingscreen/load_frame.png")    &&
-		reloadtexture("data/loadingscreen/mapshot_frame.png") &&
-		reloadtexture("data/loadingscreen/title.png")         &&
-		reloadtexture(imagelogo)
-	)	return true;
-	return false;
+    if(
+        reloadtexture("data/loadingscreen/engine_badge.png")  &&
+        reloadtexture("data/loadingscreen/load_back.png")     &&
+        reloadtexture("data/loadingscreen/load_bar.png")      &&
+        reloadtexture("data/loadingscreen/load_frame.png")    &&
+        reloadtexture("data/loadingscreen/mapshot_frame.png") &&
+        reloadtexture("data/loadingscreen/title.png")         &&
+        reloadtexture(imagelogo)
+    ) return true;
+    return false;
 }
 
 void resetgl()
@@ -2407,7 +2448,7 @@ void resetgl()
 
     extern void cleanupva();
     extern void cleanupparticles();
-	extern void cleanupsky();
+    extern void cleanupsky();
     extern void cleanupmodels();
     extern void cleanuptextures();
     extern void cleanuplightmaps();
@@ -2422,7 +2463,7 @@ void resetgl()
     recorder::cleanup();
     cleanupva();
     cleanupparticles();
-	cleanupsky();
+    cleanupsky();
     cleanupmodels();
     cleanuptextures();
     cleanuplightmaps();
@@ -2445,14 +2486,14 @@ void resetgl()
     extern void reloadshaders();
     inbetweenframes = false;
 
-	if(!resettextures())
-		fatal("failed to reload core texture");
+    if(!resettextures())
+        fatal("failed to reload core texture");
 
     reloadfonts();
     inbetweenframes = true;
     renderbackground("initializing...");
-    extern void resetgamma();
-	resetgamma();
+    extern void restoregamma();
+    restoregamma();
     reloadshaders();
     reloadtextures();
     initlights();
