@@ -119,16 +119,17 @@ void mdldepthoffset(int *offset)
 
 COMMAND(mdldepthoffset, "i");
 
-void mdlglow(int *percent)
+void mdlglow(int *percent, int *delta, float *pulse)
 {
     checkmdl;
-    float glow = 3.0f;
+    float glow = 3.0f, glowdelta = *delta/100.0f, glowpulse = *pulse > 0 ? *pulse/1000.0f : 0;
     if(*percent>0) glow = *percent/100.0f;
     else if(*percent<0) glow = 0.0f;
-    loadingmodel->setglow(glow);
+    glowdelta -= glow;
+    loadingmodel->setglow(glow, glowdelta, glowpulse);
 }
 
-COMMAND(mdlglow, "i");
+COMMAND(mdlglow, "iif");
 
 void mdlglare(float *specglare, float *glowglare)
 {
@@ -250,8 +251,8 @@ void mdlname()
 COMMAND(mdlname, "");
 
 #define checkragdoll \
-    skelmodel *m = dynamic_cast<skelmodel *>(loadingmodel); \
-    if(!m) { conoutf(CON_ERROR, "not loading a skeletal model"); return; } \
+    if(!loadingmodel->skeletal()) { conoutf(CON_ERROR, "not loading a skeletal model"); return; } \
+    skelmodel *m = (skelmodel *)loadingmodel; \
     skelmodel::skelmeshgroup *meshes = (skelmodel::skelmeshgroup *)m->parts.last()->meshes; \
     if(!meshes) return; \
     skelmodel::skeleton *skel = meshes->skel; \
@@ -289,6 +290,7 @@ COMMAND(rdtri, "iii");
 void rdjoint(int *n, int *t, char *v1, char *v2, char *v3)
 {
     checkragdoll;
+    if(*n < 0 || *n >= skel->numbones) return;
     ragdollskel::joint &j = ragdoll->joints.add();
     j.bone = *n;
     j.tri = *t;
@@ -595,11 +597,9 @@ struct transparentmodel
     float dist;
 };
 
-static int sorttransparentmodels(const transparentmodel *x, const transparentmodel *y)
+static inline bool sorttransparentmodels(const transparentmodel &x, const transparentmodel &y)
 {
-    if(x->dist > y->dist) return -1;
-    if(x->dist < y->dist) return 1;
-    return 0;
+    return x.dist < y.dist;
 }
 
 void endmodelbatches()
@@ -761,7 +761,7 @@ void rendermodel(entitylight *light, const char *mdl, int anim, const vec &o, fl
     if(shadowmapping && !(flags&(MDL_SHADOW|MDL_DYNSHADOW))) return;
     model *m = loadmodel(mdl);
     if(!m) return;
-    vec center, bbradius;
+    vec center(0, 0, 0), bbradius(0, 0, 0);
     float radius = 0;
     bool shadow = !shadowmap && !glaring && (flags&(MDL_SHADOW|MDL_DYNSHADOW)) && showblobs,
          doOQ = flags&MDL_CULL_QUERY && hasOQ && oqfrags && oqdynent;
@@ -776,7 +776,7 @@ void rendermodel(entitylight *light, const char *mdl, int anim, const vec &o, fl
         }
         else
         {
-            center.rotate_around_z(-yaw*RAD);
+            center.rotate_around_z(yaw*RAD);
             center.add(o);
         }
         if(flags&MDL_CULL_DIST && center.dist(camera1->o)/radius>maxmodelradiusdistance) return;
@@ -996,6 +996,22 @@ void findanims(const char *pattern, vector<int> &anims)
     loopi(sizeof(animnames)/sizeof(animnames[0])) if(matchanim(animnames[i], pattern)) anims.add(i);
 }
 
+ICOMMAND(findanims, "s", (char *name),
+{
+    vector<int> anims;
+    findanims(name, anims);
+    vector<char> buf;
+    string num;
+    loopv(anims)
+    {
+        formatstring(num)("%d", anims[i]);
+        if(i > 0) buf.add(' ');
+        buf.put(num, strlen(num));
+    }
+    buf.add('\0');
+    result(buf.getbuf());
+});
+
 void loadskin(const char *dir, const char *altdir, Texture *&skin, Texture *&masks) // model skin sharing
 {
 #define ifnoload(tex, path) if((tex = textureload(path, 0, true, false))==notexture)
@@ -1029,17 +1045,13 @@ void renderclient(dynent *d, const char *mdlname, modelattach *attachments, int
     if(animoverride) anim = (animoverride<0 ? ANIM_ALL : animoverride)|ANIM_LOOP;
     else if(d->state==CS_DEAD)
     {
-        anim = ANIM_DYING;
+        anim = ANIM_DYING|ANIM_NOPITCH;
         basetime = lastpain;
         if(ragdoll)
         {
             if(!d->ragdoll || d->ragdoll->millis < basetime) anim |= ANIM_RAGDOLL;
         }
-        else
-        {
-            pitch *= max(1.0f - (lastmillis-basetime)/500.0f, 0.0f);
-            if(lastmillis-basetime>1000) anim = ANIM_DEAD|ANIM_LOOP;
-        }
+        else if(lastmillis-basetime>1000) anim = ANIM_DEAD|ANIM_LOOP|ANIM_NOPITCH;
     }
     else if(d->state==CS_EDITING || d->state==CS_SPECTATOR) anim = ANIM_EDIT|ANIM_LOOP;
     else if(d->state==CS_LAGGED)                            anim = ANIM_LAG|ANIM_LOOP;
@@ -1067,7 +1079,7 @@ void renderclient(dynent *d, const char *mdlname, modelattach *attachments, int
 
         if((anim&ANIM_INDEX)==ANIM_IDLE && (anim>>ANIM_SECONDARY)&ANIM_INDEX) anim >>= ANIM_SECONDARY;
     }
-    if(d->ragdoll && (!ragdoll || anim!=ANIM_DYING)) DELETEP(d->ragdoll);
+    if(d->ragdoll && (!ragdoll || (anim&ANIM_INDEX)!=ANIM_DYING)) DELETEP(d->ragdoll);
     if(!((anim>>ANIM_SECONDARY)&ANIM_INDEX)) anim |= (ANIM_IDLE|ANIM_LOOP)<<ANIM_SECONDARY;
     int flags = MDL_LIGHT;
     if(d!=player && !(anim&ANIM_RAGDOLL)) flags |= MDL_CULL_VFC | MDL_CULL_OCCLUDED | MDL_CULL_QUERY;
