@@ -65,13 +65,13 @@ static particleemitter *seedemitter = NULL;
 
 void clearparticleemitters()
 {
-    emitters.setsize(0);
+    emitters.shrink(0);
     regenemitters = true;
 }
 
 void addparticleemitters()
 {
-    emitters.setsize(0);
+    emitters.shrink(0);
     const vector<extentity *> &ents = entities::getents();
     loopv(ents)
     {
@@ -105,6 +105,7 @@ enum
     PT_VFLIP = 1<<15,
     PT_ROT   = 1<<16,
     PT_CULL  = 1<<17,
+    PT_ICON  = 1<<18,
     PT_FLIP  = PT_HFLIP | PT_VFLIP | PT_ROT
 };
 
@@ -145,11 +146,16 @@ struct partrenderer
 {
     Texture *tex;
     const char *texname;
+    int texclamp;
     uint type;
     int collide;
 
-    partrenderer(const char *texname, int type, int collide = 0)
-        : tex(NULL), texname(texname), type(type), collide(collide)
+    partrenderer(const char *texname, int texclamp, int type, int collide = 0)
+        : tex(NULL), texname(texname), texclamp(texclamp), type(type), collide(collide)
+    {
+    }
+    partrenderer(int type, int collide = 0)
+        : tex(NULL), texname(NULL), texclamp(0), type(type), collide(collide)
     {
     }
     virtual ~partrenderer()
@@ -157,14 +163,14 @@ struct partrenderer
     }
 
     virtual void init(int n) { }
-    virtual void reset() = NULL;
+    virtual void reset() = 0;
     virtual void resettracked(physent *owner) { }
-    virtual particle *addpart(const vec &o, const vec &d, int fade, int color, float size, int gravity = 0) = NULL;
+    virtual particle *addpart(const vec &o, const vec &d, int fade, int color, float size, int gravity = 0) = 0;
     virtual int adddepthfx(vec &bbmin, vec &bbmax) { return 0; }
     virtual void update() { }
-    virtual void render() = NULL;
-    virtual bool haswork() = NULL;
-    virtual int count() = NULL; //for debug
+    virtual void render() = 0;
+    virtual bool haswork() = 0;
+    virtual int count() = 0; //for debug
     virtual bool usesvertexarray() { return false; }
     virtual void cleanup() {}
 
@@ -223,8 +229,12 @@ struct listrenderer : partrenderer
     static listparticle *parempty;
     listparticle *list;
 
-    listrenderer(const char *texname, int type, int collide = 0)
-        : partrenderer(texname, type, collide), list(NULL)
+    listrenderer(const char *texname, int texclamp, int type, int collide = 0)
+        : partrenderer(texname, texclamp, type, collide), list(NULL)
+    {
+    }
+    listrenderer(int type, int collide = 0)
+        : partrenderer(type, collide), list(NULL)
     {
     }
 
@@ -313,7 +323,7 @@ struct listrenderer : partrenderer
         startrender();
         if(texname)
         {
-            if(!tex) tex = textureload(texname);
+            if(!tex) tex = textureload(texname, texclamp);
             glBindTexture(GL_TEXTURE_2D, tex->id);
         }
 
@@ -348,7 +358,7 @@ listparticle *listrenderer::parempty = NULL;
 struct meterrenderer : listrenderer
 {
     meterrenderer(int type)
-        : listrenderer(NULL, type)
+        : listrenderer(type)
     {}
 
     void startrender()
@@ -362,7 +372,6 @@ struct meterrenderer : listrenderer
     {
          glEnable(GL_BLEND);
          glEnable(GL_TEXTURE_2D);
-         if(fogging && renderpath!=R_FIXEDFUNCTION) setfogplane(1, reflectz);
          particleshader->set();
     }
 
@@ -372,8 +381,7 @@ struct meterrenderer : listrenderer
 
         glPushMatrix();
         glTranslatef(o.x, o.y, o.z);
-        if(fogging && renderpath!=R_FIXEDFUNCTION) setfogplane(0, reflectz - o.z, true);
-        glRotatef(camera1->yaw-180, 0, 0, 1);
+        glRotatef(camera1->yaw, 0, 0, 1);
         glRotatef(camera1->pitch-90, 1, 0, 0);
 
         float scale = p->size/80.0f;
@@ -436,7 +444,7 @@ static meterrenderer meters(PT_METER|PT_LERP), metervs(PT_METERVS|PT_LERP);
 struct textrenderer : listrenderer
 {
     textrenderer(int type)
-        : listrenderer(NULL, type)
+        : listrenderer(type)
     {}
 
     void startrender()
@@ -445,7 +453,6 @@ struct textrenderer : listrenderer
 
     void endrender()
     {
-        if(fogging && renderpath!=R_FIXEDFUNCTION) setfogplane(1, reflectz);
     }
 
     void cleanup(listparticle *p)
@@ -457,13 +464,8 @@ struct textrenderer : listrenderer
     {
         glPushMatrix();
         glTranslatef(o.x, o.y, o.z);
-        if(fogging)
-        {
-            if(renderpath!=R_FIXEDFUNCTION) setfogplane(0, reflectz - o.z, true);
-            else blend = (uchar)(blend * max(0.0f, min(1.0f, 1.0f - (reflectz - o.z)/waterfog)));
-        }
 
-        glRotatef(camera1->yaw-180, 0, 0, 1);
+        glRotatef(camera1->yaw, 0, 0, 1);
         glRotatef(camera1->pitch-90, 1, 0, 0);
 
         float scale = p->size/80.0f;
@@ -485,7 +487,6 @@ template<int T>
 static inline void modifyblend(const vec &o, int &blend)
 {
     blend = min(blend<<2, 255);
-    if(renderpath==R_FIXEDFUNCTION && fogging) blend = (uchar)(blend * max(0.0f, min(1.0f, 1.0f - (reflectz - o.z)/waterfog)));
 }
 
 template<>
@@ -595,7 +596,7 @@ struct varenderer : partrenderer
     int maxparts, numparts, lastupdate, rndmask;
 
     varenderer(const char *texname, int type, int collide = 0)
-        : partrenderer(texname, type, collide),
+        : partrenderer(texname, 3, type, collide),
           verts(NULL), parts(NULL), maxparts(0), numparts(0), lastupdate(-1), rndmask(0)
     {
         if(type & PT_HFLIP) rndmask |= 0x01;
@@ -693,11 +694,10 @@ struct varenderer : partrenderer
         {
             p->flags &= ~0x80;
 
-            #define SETTEXCOORDS(u1c, u2c, v1c, v2c) \
+            #define SETTEXCOORDS(u1c, u2c, v1c, v2c, body) \
             { \
                 float u1 = u1c, u2 = u2c, v1 = v1c, v2 = v2c; \
-                if(p->flags&0x01) swap(u1, u2); \
-                if(p->flags&0x02) swap(v1, v2); \
+                body; \
                 vs[0].u = u1; \
                 vs[0].v = v1; \
                 vs[1].u = u2; \
@@ -710,9 +710,18 @@ struct varenderer : partrenderer
             if(type&PT_RND4)
             {
                 float tx = 0.5f*((p->flags>>5)&1), ty = 0.5f*((p->flags>>6)&1);
-                SETTEXCOORDS(tx, tx + 0.5f, ty, ty + 0.5f);
+                SETTEXCOORDS(tx, tx + 0.5f, ty, ty + 0.5f,
+                {
+                    if(p->flags&0x01) swap(u1, u2);
+                    if(p->flags&0x02) swap(v1, v2);
+                });
             }
-            else SETTEXCOORDS(0, 1, 0, 1);
+            else if(type&PT_ICON)
+            {
+                float tx = 0.25f*(p->flags&3), ty = 0.25f*((p->flags>>2)&3);
+                SETTEXCOORDS(tx, tx + 0.25f, ty, ty + 0.25f, {});
+            }
+            else SETTEXCOORDS(0, 1, 0, 1, {});
 
             #define SETCOLOR(r, g, b, a) \
             do { \
@@ -756,7 +765,7 @@ struct varenderer : partrenderer
 
     void render()
     {
-        if(!tex) tex = textureload(texname);
+        if(!tex) tex = textureload(texname, texclamp);
         glBindTexture(GL_TEXTURE_2D, tex->id);
         glVertexPointer(3, GL_FLOAT, sizeof(partvert), &verts->pos);
         glTexCoordPointer(2, GL_FLOAT, sizeof(partvert), &verts->u);
@@ -807,27 +816,30 @@ struct softquadrenderer : quadrenderer
 
 static partrenderer *parts[] =
 {
-    new quadrenderer("data/items", PT_PART|PT_FLIP|PT_MOD|PT_RND4, 1),          // blood spats (note: rgb is inverted)
-    new trailrenderer("data/hirato/base", PT_TRAIL|PT_LERP, DECAL_RIPPLE),      // water, entity
-    new quadrenderer("data/hirato/smoke", PT_PART|PT_FLIP|PT_LERP),             // smoke
-    new quadrenderer("data/hirato/steam", PT_PART|PT_FLIP),                     // steam
-    new quadrenderer("data/hirato/flames", PT_PART|PT_HFLIP|PT_RND4|PT_GLARE),  // flames
-    new quadrenderer("data/hirato/ball1", PT_PART|PT_GLARE),                    // fireball1
-    new quadrenderer("data/hirato/ball2", PT_PART|PT_GLARE),                    // fireball2
-    new quadrenderer("data/hirato/ball3", PT_PART|PT_GLARE),                    // fireball3
-    new taperenderer("data/hirato/flare", PT_TAPE|PT_GLARE),                    // streak
-    &lightnings,                                                                // lightning
-    &fireballs,                                                                 // explosion fireball
-    &noglarefireballs,                                                          // explosion fireball no glare
-    new quadrenderer("data/hirato/spark", PT_PART|PT_FLIP|PT_GLARE),            // sparks
-    new quadrenderer("data/hirato/base",  PT_PART|PT_FLIP|PT_GLARE),            // edit mode entities
-    new quadrenderer("data/hirato/muzzleflash1", PT_PART|PT_FLIP|PT_GLARE|PT_TRACK), // muzzle flash
-    new quadrenderer("data/hirato/muzzleflash2", PT_PART|PT_FLIP|PT_GLARE|PT_TRACK), // muzzle flash
-    new quadrenderer("data/hirato/muzzleflash3", PT_PART|PT_FLIP|PT_GLARE|PT_TRACK), // muzzle flash
-    &texts,                                                                     // text
-    &meters,                                                                    // meter
-    &metervs,                                                                   // meter vs
-    new quadrenderer("data/hirato/snow", PT_PART|PT_GLARE|PT_RND4|PT_FLIP, DECAL_STAIN), // snow
+    new quadrenderer("data/items", PT_PART|PT_FLIP|PT_MOD|PT_RND4, 1),                            // blood spats (note: rgb is inverted)
+    new trailrenderer("packages/particles/base", PT_TRAIL|PT_LERP, DECAL_RIPPLE),                 // water, entity
+    new quadrenderer("packages/particles/smoke", PT_PART|PT_FLIP|PT_LERP),                        // smoke
+    new quadrenderer("packages/particles/steam", PT_PART|PT_FLIP),                                // steam
+    new quadrenderer("packages/particles/flames", PT_PART|PT_HFLIP|PT_RND4|PT_GLARE),             // flames
+    new quadrenderer("packages/particles/ball1", PT_PART|PT_GLARE),                               // fireball1
+    new quadrenderer("packages/particles/ball2", PT_PART|PT_GLARE),                               // fireball2
+    new quadrenderer("packages/particles/ball3", PT_PART|PT_GLARE),                               // fireball3
+    new taperenderer("packages/particles/flare", PT_TAPE|PT_GLARE),                               // streak
+    &lightnings,                                                                                  // lightning
+    &fireballs,                                                                                   // explosion fireball
+    &bluefireballs,                                                                               // bluish explosion fireball
+    new quadrenderer("packages/particles/spark", PT_PART|PT_FLIP|PT_GLARE),                       // sparks
+    new quadrenderer("packages/particles/base",  PT_PART|PT_FLIP|PT_GLARE),                       // edit mode entities
+    new quadrenderer("packages/particles/muzzleflash1", PT_PART|PT_FLIP|PT_GLARE|PT_TRACK),       // muzzle flash
+    new quadrenderer("packages/particles/muzzleflash2", PT_PART|PT_FLIP|PT_GLARE|PT_TRACK),       // muzzle flash
+    new quadrenderer("packages/particles/muzzleflash3", PT_PART|PT_FLIP|PT_GLARE|PT_TRACK),       // muzzle flash
+    new quadrenderer("data/items.png", PT_PART|PT_ICON),                                          // hud icon
+    new quadrenderer("<colorify:1/1/1>data/items.png", PT_PART|PT_ICON),                               // grey hud icon
+    &texts,                                                                                       // text
+    &meters,                                                                                      // meter
+    &metervs,                                                                                     // meter vs
+    new quadrenderer("packages/particles/snow", PT_PART|PT_GLARE|PT_RND4|PT_FLIP, DECAL_STAIN),   // snow
+    new quadrenderer("packages/particles/leaves", PT_PART|PT_RND4|PT_FLIP|PT_LERP, DECAL_LEAVES), //leaves
     &flares // must be done last
 };
 
@@ -835,7 +847,8 @@ void finddepthfxranges()
 {
     depthfxmin = vec(1e16f, 1e16f, 1e16f);
     depthfxmax = vec(0, 0, 0);
-    numdepthfxranges = fireballs.finddepthfxranges(depthfxowners, depthfxranges, MAXDFXRANGES, depthfxmin, depthfxmax);
+    numdepthfxranges = fireballs.finddepthfxranges(depthfxowners, depthfxranges, 0, MAXDFXRANGES, depthfxmin, depthfxmax);
+    numdepthfxranges = bluefireballs.finddepthfxranges(depthfxowners, depthfxranges, numdepthfxranges, MAXDFXRANGES, depthfxmin, depthfxmax);
     loopk(3)
     {
         depthfxmin[k] -= depthfxmargin;
@@ -1141,6 +1154,13 @@ void particle_textcopy(const vec &s, const char *t, int type, int fade, int colo
      p->flags = 1;
 }
 
+void particle_icon(const vec &s, int ix, int iy, int type, int fade, int color, float size, int gravity)
+{
+    if(!canaddparticles()) return;
+    particle *p = newparticle(s, vec(0, 0, 1), fade, type, color, size, gravity);
+    p->flags |= ix | (iy<<2);
+}
+
 void particle_meter(const vec &s, float val, int type, int fade, int color, int color2, float size)
 {
     if(!canaddparticles()) return;
@@ -1274,7 +1294,7 @@ void regularshape(int type, int radius, int color, int dir, int num, int fade, c
     }
 }
 
-void regularflame(int type, const vec &p, float radius, float height, int color, int density, float scale, float speed, float fade, int gravity)
+static void regularflame(int type, const vec &p, float radius, float height, int color, int density = 3, float scale = 2.0f, float speed = 200.0f, float fade = 600.0f, int gravity = -15)
 {
     if(!emit_particles()) return;
 
@@ -1289,6 +1309,12 @@ void regularflame(int type, const vec &p, float radius, float height, int color,
     }
 }
 
+void regular_particle_flame(int type, const vec &p, float radius, float height, int color, int density, float scale, float speed, float fade, int gravity)
+{
+    if(!canaddparticles()) return;
+    regularflame(type, p, radius, height, color, density, scale, speed, fade, gravity);
+}
+
 void regularlensflare(vec o, uchar r, uchar g, uchar b, bool sun, bool sparkle, int sizemod)
 {
 	flares.addflare(o, r, g, b, sun, sparkle, sizemod);
@@ -1298,79 +1324,81 @@ VARP(showentities, 0, 1, 2);
 
 static void makeparticles(entity &e)
 {
-    switch(e.attr1)
+    switch(e.attr[0])
     {
         case 0: //fire and smoke -  <radius> <height> <rgb> - 0 values default to compat for old maps
 	{
-            float radius = e.attr2 ? float(e.attr2)/100.0f : 1.5f,
-                  height = e.attr3 ? float(e.attr3)/100.0f : radius/3;
- 	    regularflame(PART_FLAME, e.o, radius, height, e.attr4 ? e.attr4 & 0xFFFFFF : 0x903020, 3, e.attr5 ? 0.01f * abs(e.attr5) : 2.0f, e.attr6 ? 0.02f * abs(e.attr6): 200.0f, e.attr7 ? abs(e.attr7) : 600.0f, e.attr8 ? e.attr8: -15.0f);
-            regularflame(PART_SMOKE, vec(e.o.x, e.o.y, e.o.z + 4.0f*min(radius, height)), radius, height, 0x303020, 1, e.attr5 ? .02f * abs(e.attr5) : 4.0f, e.attr6 ? 0.01f * abs(e.attr6) : 100.0f, e.attr7 ? abs(e.attr7) + 1400: 2000.0f, e.attr8 ? e.attr8 - 5 : -20.0f);
+            float radius = e.attr[1] ? float(e.attr[1])/100.0f : 1.5f,
+                  height = e.attr[2] ? float(e.attr[2])/100.0f : radius/3;
+ 	    regularflame(PART_FLAME, e.o, radius, height, e.attr[3] ? e.attr[3] & 0xFFFFFF : 0x903020, 3, e.attr[4] ? 0.01f * abs(e.attr[4]) : 2.0f, e.attr[5] ? 0.02f * abs(e.attr[5]): 200.0f, e.attr[6] ? abs(e.attr[6]) : 600.0f, e.attr[7] ? e.attr[7]: -15.0f);
+            regularflame(PART_SMOKE, vec(e.o.x, e.o.y, e.o.z + 4.0f*min(radius, height)), radius, height, 0x303020, 1, e.attr[4] ? .02f * abs(e.attr[4]) : 4.0f, e.attr[5] ? 0.01f * abs(e.attr[5]) : 100.0f, e.attr[6] ? abs(e.attr[6]) + 1400: 2000.0f, e.attr[7] ? e.attr[7] - 5 : -20.0f);
             break;
 	}
-        case 1: //steam vent - <dir>
-            regularsplash(PART_STEAM, 0x897661, e.attr3 ? abs(e.attr3) : 50, 1, e.attr4 ? abs(e.attr4) : 200, offsetvec(e.o, e.attr2, rnd(10)), e.attr5 ? abs(e.attr5) *.01f : 2.4f, e.attr6 ? e.attr6 : -20);
+	case 1: // flame <radius> <height> <rgb> - radius=100, height=100 is the classic size
+            regularflame(PART_FLAME, e.o, float(e.attr[1])/100.0f, float(e.attr[2])/100.0f, e.attr[3] & 0xFFFFFF, 3, e.attr[4] ? 0.01f * abs(e.attr[4]) : 2.0f, e.attr[5] ? 0.02f * abs(e.attr[5]): 200.0f, e.attr[6] ? abs(e.attr[6]) : 600.0f, e.attr[7] ? e.attr[7]: -15.0f);
             break;
-        case 2: //water fountain - <dir>
+        case 2: // smoke plume <radius> <height> <rgb>
+            regularflame(PART_SMOKE, e.o, float(e.attr[1])/100.0f, float(e.attr[2])/100.0f, e.attr[3] & 0xFFFFFF, 1, e.attr[4] ? .02f * abs(e.attr[4]) : 4.0f, e.attr[5] ? 0.01f * abs(e.attr[5]) : 100.0f, e.attr[6] ? abs(e.attr[6]): 2000.0f, e.attr[7] ? e.attr[7]: -20.0f);
+            break;
+        case 3: //steam vent - <dir>
+            regularsplash(PART_STEAM, 0x897661, e.attr[2] ? abs(e.attr[2]) : 50, 1, e.attr[3] ? abs(e.attr[3]) : 200, offsetvec(e.o, e.attr[1], rnd(10)), e.attr[4] ? abs(e.attr[4]) *.01f : 2.4f, e.attr[5] ? e.attr[5] : -20);
+            break;
+        case 4: //water fountain - <dir>
         {
             int color = (int(waterfallcolor[0])<<16) | (int(waterfallcolor[1])<<8) | int(waterfallcolor[2]);
             if(!color) color = (int(watercolor[0])<<16) | (int(watercolor[1])<<8) | int(watercolor[2]);
-            regularsplash(PART_WATER, color, e.attr3 ? e.attr3 : 150, 4, e.attr4 ? e.attr4 : 200, offsetvec(e.o, e.attr2, rnd(10)), e.attr5 ? e.attr5 *.01f : 0.6f, e.attr6 ? e.attr6 : 2);
+            regularsplash(PART_WATER, color, e.attr[2] ? abs(e.attr[2]) : 150, 4, e.attr[3] ? abs(e.attr[3]) : 200, offsetvec(e.o, e.attr[1], rnd(10)), e.attr[4] ? abs(e.attr[4]) *.01f : 0.6f, e.attr[5] ? e.attr[5] : 2);
             break;
         }
-        case 3: //fire ball - <size> <rgb>
-           newparticle(e.o, vec(0, 0, 1), 1, PART_EXPLOSION, e.attr3 & 0xFFFFFF, 4.0f)->val = 1+e.attr2;
+        case 5: //fire ball - <size> <rgb>
+           newparticle(e.o, vec(0, 0, 1), 1, PART_EXPLOSION, e.attr[2] & 0xFFFFFF, 4.0f)->val = 1+e.attr[1];
             break;
-        case 4:  //tape - <dir> <length> <rgb>
-        case 7:  //lightning
-        case 8:  //ball1
-        case 9:  //steam
-        case 10: //water
-        case 12: //snow
+	case 6: //meter, metervs - <percent> <rgb> <rgb2>
+        case 7:
         {
-            const int typemap[]   = { PART_STREAK, -1, -1, PART_LIGHTNING, PART_FIREBALL1, PART_STEAM, PART_WATER, -1, PART_SNOW };
-            const float sizemap[] = { 0.28, 0.0, 0.0, 0.28, 4.8, 2.4, 0.60, 0.0, 0.5 };
-            const float velmap[]  = {  200,   0,   0,  200, 200, 200,  200,   0,  40 };
-	    const int gravmap[] =   {    0,   0,   0,    0,  20, -20,    2,   0, 200 };
-            int type = typemap[e.attr1-4], gravity = e.attr8 ? e.attr8 : gravmap[e.attr1-4];
-            float size = e.attr6 ? abs(e.attr6) * .01f : sizemap[e.attr1-4], vel = e.attr7 ? e.attr7 * .1f : velmap[e.attr1-4];
-            if(e.attr2 >= 256) regularshape(type, max(1+e.attr3, 1), e.attr4 & 0xFFFFFF, e.attr2-256, 5, e.attr5 != 0 ? e.attr5 : 200, e.o, size*particlesize/100.0, vel, gravity);
-            else newparticle(e.o, offsetvec(e.o, e.attr2, max(1+e.attr3, 1)), 1, type, e.attr4 & 0xFFFFFF, size, gravity);
-            break;
-        }
-        case 11:
-        	if(editmode || (game::showenthelpers() && showentities >= 1) || showentities==2) break;
-
-		defformatstring(aliasname)("part_text_%d", e.attr2);
-		defformatstring(word)("@%s", identexists(aliasname) ?  getalias(aliasname) : "Nothing");
-		partent_text(e.o, word, e.attr3 & 0xFFFFFF, e.attr4 > 0 ? e.attr4 : 200);
-        	break;
-        case 5: //meter, metervs - <percent> <rgb> <rgb2>
-        case 6:
-        {
-            particle *p = newparticle(e.o, vec(0, 0, 1), 1, e.attr1==5 ? PART_METER : PART_METER_VS, e.attr3 & 0xFFFFFF, 2.0f);
-            int color2 = e.attr4 & 0xFFFFFF;
+            particle *p = newparticle(e.o, vec(0, 0, 1), 1, e.attr[0]==6 ? PART_METER : PART_METER_VS, e.attr[2] & 0xFFFFFF, 2.0f);
+            int color2 = e.attr[3] & 0xFFFFFF;
             p->color2[0] = color2>>16;
             p->color2[1] = (color2>>8)&0xFF;
             p->color2[2] = color2&0xFF;
-            p->progress = clamp(int(e.attr2), 0, 100);
+            p->progress = clamp(int(e.attr[1]), 0, 100);
             break;
         }
-	case 13: // flame <radius> <height> <rgb> - radius=100, height=100 is the classic size
-            regularflame(PART_FLAME, e.o, float(e.attr2)/100.0f, float(e.attr3)/100.0f, e.attr4 & 0xFFFFFF, 3, e.attr5 ? 0.01f * abs(e.attr5) : 2.0f, e.attr6 ? 0.02f * abs(e.attr6): 200.0f, e.attr7 ? abs(e.attr7) : 600.0f, e.attr8 ? e.attr8: -15.0f);
-            break;
-        case 14: // smoke plume <radius> <height> <rgb>
-            regularflame(PART_SMOKE, e.o, float(e.attr2)/100.0f, float(e.attr3)/100.0f, e.attr4 & 0xFFFFFF, 1, e.attr5 ? .02f * abs(e.attr5) : 4.0f, e.attr6 ? 0.01f * abs(e.attr6) : 100.0f, e.attr7 ? abs(e.attr7): 2000.0f, e.attr8 ? e.attr8 - 5 : -20.0f);
+        case 8:
+        	if(editmode || (game::showenthelpers() && showentities >= 1) || showentities==2) break;
+
+		defformatstring(aliasname)("part_text_%d", e.attr[1]);
+		defformatstring(word)("@%s", identexists(aliasname) ?  getalias(aliasname) : "Nothing");
+		partent_text(e.o, word, e.attr[2] & 0xFFFFFF, e.attr[3] > 0 ? e.attr[3] : 200);
+        	break;
+        case 9:  //tape - <dir> <length> <rgb>
+        case 10:  //lightning
+        case 11:  //ball1
+        case 12:  //steam
+        case 13: //water
+        case 14: //snow
+	case 15: //leaves
+        {
+            const int typemap[]   = { PART_STREAK, PART_LIGHTNING, PART_FIREBALL1, PART_STEAM, PART_WATER, PART_SNOW, PART_LEAVES };
+            const float sizemap[] = { 0.28, 1.0, 4.8, 2.4, 0.60, 0.5, 1   };
+            const float velmap[]  = {  200, 200, 200, 200,  200,  40, 100 };
+	    const int gravmap[] =   {    0,   0,  20, -20,    2, 200, 40  };
+            int type = typemap[e.attr[0]-9], gravity = e.attr[7] ? e.attr[7] : gravmap[e.attr[0]-9];
+            float size = e.attr[5] ? abs(e.attr[5]) * .01f : sizemap[e.attr[0]-9], vel = e.attr[6] ? e.attr[6] * .1f : velmap[e.attr[0]-9];
+
+            if(e.attr[1] >= 256) regularshape(type, max(1+e.attr[2], 1), e.attr[3] & 0xFFFFFF, e.attr[1]-256, 5, e.attr[4] != 0 ? e.attr[4] : 200, e.o, size*particlesize/100.0, vel, gravity);
+            else newparticle(e.o, offsetvec(e.o, e.attr[1], max(1+e.attr[2], 1)), 1, type, e.attr[3] & 0xFFFFFF, size, gravity);
             break;
+        }
         case 32: //lens flares - plain/sparkle/sun/sparklesun <red> <green> <blue>
         case 33:
         case 34:
         case 35:
-            flares.addflare(e.o, e.attr2, e.attr3, e.attr4, (e.attr1&0x02)!=0, (e.attr1&0x01)!=0, e.attr5);
+            flares.addflare(e.o, e.attr[1], e.attr[2], e.attr[3], (e.attr[0]&0x02)!=0, (e.attr[0]&0x01)!=0, e.attr[4]);
             break;
         default:
 	    if(editmode) return;
-            defformatstring(ds)("@particles %d?", e.attr1);
+            defformatstring(ds)("@particles %d?", e.attr[0]);
             particle_text(e.o, ds, PART_TEXT, 1, 0x6496FF, 2.0f);
 	    break;
     }
@@ -1378,23 +1406,23 @@ static void makeparticles(entity &e)
 
 bool printparticles(extentity &e, char *buf)
 {
-	switch(e.attr1)
+	switch(e.attr[0])
 	{
-		case 0: case 4: case 7: case 8: case 9: case 10: case 12: case 13: case 14:
+		case 0: case 1: case 2: case 9: case 10: case 11: case 12: case 13: case 14: case 15:
 		{
-			defformatstring(ds)("%d %d %d 0x%.6X %d %d %d %d", e.attr1, e.attr2, e.attr3, e.attr4, e.attr5, e.attr6, e.attr7, e.attr8);
+			defformatstring(ds)("%d %d %d 0x%.6X %d %d %d %d", e.attr[0], e.attr[1], e.attr[2], e.attr[3], e.attr[4], e.attr[5], e.attr[6], e.attr[7]);
 			concatstring(buf, ds);
 			return true;
 		}
-		case 11: case 3:
+		case 5: case 8:
 		{
-			defformatstring(ds)("%d %d 0x%.6X %d %d %d %d %d", e.attr1, e.attr2, e.attr3, e.attr4, e.attr5, e.attr6, e.attr7, e.attr8);
+			defformatstring(ds)("%d %d 0x%.6X %d %d %d %d %d", e.attr[0], e.attr[1], e.attr[2], e.attr[3], e.attr[4], e.attr[5], e.attr[6], e.attr[7]);
 			concatstring(buf, ds);
 			return true;
 		}
-		case 5: case 6:
+		case 6: case 7:
 		{
-			defformatstring(ds)("%d %d 0x%.6X 0x%.6X %d %d %d %d",  e.attr1, e.attr2, e.attr3, e.attr4, e.attr5, e.attr6, e.attr7, e.attr8);
+			defformatstring(ds)("%d %d 0x%.6X 0x%.6X %d %d %d %d", e.attr[0], e.attr[1], e.attr[2], e.attr[3], e.attr[4], e.attr[5], e.attr[6], e.attr[7]);
 			concatstring(buf, ds);
 			return true;
 		}
@@ -1406,7 +1434,7 @@ VARP(showenttext, 0, 1, 1);
 
 void renderentinfo(int i)
 {
-	if(0 > i) return;
+	if(!entities::getents().inrange(i)) return;
 	extentity &e = *entities::getents()[i];
 	vec pos = e.o;
 	string ds, tmp;
@@ -1441,138 +1469,149 @@ void renderentinfo(int i)
 		case ET_LIGHT:
 			pos.z += 6;
 			formatstring(tmp)("Radius %i\n\fs\fRRed: %i\n\fJGreen: %i\n\fDBlue: %i\fr",
-				e.attr1,
-				e.attr2,
-				e.attr3,
-				e.attr4
+				e.attr[0],
+				e.attr[1],
+				e.attr[2],
+				e.attr[3]
 			);
 			break;
 
 		case ET_MAPMODEL:
-			pos.z += 7.5;
-			formatstring(tmp)("Yaw: %i\nModel: %s (%i)\nTrigger Type: %i\nTag: level_trigger_%i\nBonus Radius: %i",
-				e.attr1,
-				mapmodelname(e.attr2), e.attr2,
-				e.attr3,
-				e.attr4,
-				e.attr5
+			pos.z += 9.0;
+			formatstring(tmp)("Yaw: %i\nModel: %s (%i)\nTrigger Type: %i\nTag: level_trigger_%i\nRadius: %i\nColour: \fs\fR%i \fJ%i \fD%i\fr",
+				e.attr[0],
+				mapmodelname(e.attr[1]), e.attr[1],
+				e.attr[2],
+				e.attr[3],
+				e.attr[4],
+				(e.attr[5] >> 16) & 0xFF, (e.attr[5] >> 8) & 0xFF, e.attr[5] & 0xFF
 			);
 			break;
 		case ET_PLAYERSTART:
-			pos.z += 1.5;
-			formatstring(tmp)("Yaw: %i", e.attr1);
+			pos.z += 3.0;
+			formatstring(tmp)("Yaw: %i\nTag: %i", e.attr[0], e.attr[1]);
 			break;
 
 		case ET_ENVMAP:
-			pos.z += 1.5;
-			formatstring(tmp)("Radius: %i", e.attr1);
+			pos.z += 4.5;
+			formatstring(tmp)("Radius: %i\nSize: %i (%i)\nBlur: %i (%i)",
+				e.attr[0],
+				e.attr[1], e.attr[1] ? clamp(e.attr[1], 4, 9) : 0,
+				e.attr[2], e.attr[2] ? clamp(e.attr[2], 1, 2) : 0
+			);
 			break;
 
 		case ET_PARTICLES:
-			switch(e.attr1)
+			switch(e.attr[0])
 			{
 				case 0:
 					pos.z += 12.0f;
 					formatstring(tmp)("Type: Fire (0)\nRadius: %f\nHeight: %f\nColour: \fs\fR%i \fJ%i \fD%i\fr\nSize: %i\nSpeed: %i\nFade: %i\nGravity: %i",
 						//NULL,
-						e.attr2 ? float(e.attr2)/100.0f : 1.5f,
-						e.attr3 ? float(e.attr3)/100.0f : (e.attr2 ? float(e.attr2)/100.0f : 1.5f)/3,
-						(e.attr4 & 0xFF0000) >> 16, (e.attr4 & 0xFF00) >> 8, e.attr4 & 0xFF,
-						e.attr5,
-						e.attr6,
-						e.attr7,
-						e.attr8
+						e.attr[1] ? float(e.attr[1])/100.0f : 1.5f,
+						e.attr[2] ? float(e.attr[2])/100.0f : (e.attr[2] ? float(e.attr[2])/100.0f : 1.5f)/3,
+						(e.attr[3] & 0xFF0000) >> 16, (e.attr[3] & 0xFF00) >> 8, e.attr[3] & 0xFF,
+						e.attr[4],
+						e.attr[5],
+						e.attr[6],
+						e.attr[7]
 					);
 					break;
 				case 1:
 				case 2:
+					pos.z += 12.0f;
+					formatstring(tmp)("Type: %s (%i)\nRadius: %i\nHeight: %i\nColour: \fs\fR%i \fJ%i \fD%i\fr\nSize: %i\nSpeed: %i\nFade: %i\nGravity: %i",
+						e.attr[0]==1 ? "Flame" : "Smoke Plume", e.attr[0],
+							  e.attr[1],
+							  e.attr[2],
+							  (e.attr[3] & 0xFF0000) >> 16, (e.attr[3] & 0xFF00) >> 8, e.attr[3] & 0xFF,
+							  e.attr[4],
+							  e.attr[5],
+							  e.attr[6],
+							  e.attr[7]
+							  );
+
+							  break;
+				case 3:
+				case 4:
 					pos.z += 9.0f;
 					formatstring(tmp)("Type: %s (%i)\nDirection: %i\nRadius: %i\nFade: %i\nSize: %i\nGravity: %i",
-						e.attr1==1 ? "Smoke" : "Fountain", e.attr1,
-						e.attr2,
-						e.attr3,
-						e.attr4,
-						e.attr5,
-						e.attr6
-					);
-					break;
-				case 3:
-					pos.z += 9.0;
-					formatstring(tmp)("Type: Explosion (3)\nSize: %i\nColour: \fs\fR%i \fJ%i \fD%i\fr",
-						//NULL,
-						e.attr2,
-						(e.attr3 & 0xFF0000) >> 16, (e.attr3 & 0xFF00) >> 8, e.attr3 & 0xFF
-					);
-					break;
-				case 4:
-				case 7:
-				case 8:
-				case 9:
-				case 10:
-				case 12:
-					pos.z += 12.0f;
-					formatstring(tmp)("Type: %s (%i)\n%s : %i\nLength: %i\nColour: \fs\fR%i \fJ%i \fD%i\fr\nFadetime w/%s collision: %i\nSize: %i\nVel: %i\nGravity: %i",
-						e.attr1 == 4 ? "Tape/Flare" : e.attr1 == 7 ? "Lightning" : e.attr1 == 8 ? "Fire" : e.attr1 == 9 ? "Smoke" : e.attr1 == 10 ? "Water" : "Snow", /* need a smaller conditional statement... */ e.attr1,
-						e.attr2 >= 256 ? "Effect" : "Direction", e.attr2,
-						e.attr3,
-						(e.attr4 & 0xFF0000) >> 16, (e.attr4 & 0xFF00) >> 8, e.attr4 & 0xFF,
-						e.attr5 < 0 ? "" : "o", abs(e.attr5),
-						e.attr6,
-						e.attr7,
-						e.attr8
+						e.attr[0]==3 ? "Smoke" : "Fountain", e.attr[0],
+						e.attr[1],
+						e.attr[2],
+						e.attr[3],
+						e.attr[4],
+						e.attr[5]
 					);
 					break;
 				case 5:
+					pos.z += 9.0;
+					formatstring(tmp)("Type: Explosion (5)\nSize: %i\nColour: \fs\fR%i \fJ%i \fD%i\fr",
+						//NULL,
+						e.attr[1],
+						(e.attr[2] & 0xFF0000) >> 16, (e.attr[2] & 0xFF00) >> 8, e.attr[2] & 0xFF
+					);
+					break;
 				case 6:
+				case 7:
 					pos.z += 6;
 					formatstring(tmp)("Type: Meter%s (%i)\nPercentage: %i\n1st Colour: \fs\fR%i \fJ%i \fD%i\fr\n2nd Colour: \fs\fR%i \fJ%i \fD%i\fr",
-						e.attr1==6 ? " Versus" : "", e.attr1,
-						e.attr2,
-						(e.attr3 & 0xFF0000) >> 16, (e.attr3 & 0xFF00) >> 8, e.attr3 & 0xFF,
-						(e.attr4 & 0xFF0000) >> 16, (e.attr4 & 0xFF00) >> 8, e.attr4 & 0xFF
-					);
-					break;
-
-				case 11:
+						e.attr[0]==7 ? " Versus" : "", e.attr[0],
+							  e.attr[1],
+							  (e.attr[2] & 0xFF0000) >> 16, (e.attr[2] & 0xFF00) >> 8, e.attr[2] & 0xFF,
+							  (e.attr[3] & 0xFF0000) >> 16, (e.attr[3] & 0xFF00) >> 8, e.attr[3] & 0xFF
+							  );
+							  break;
+				case 8:
 					pos.z += 6;
-					formatstring(tmp)("Type: Text (11)\nTag: part_text_%i\nColour: \fs\fR%i \fJ%i \fD%i\fr\nSize %i",
+					formatstring(tmp)("Type: Text (8)\nTag: part_text_%i\nColour: \fs\fR%i \fJ%i \fD%i\fr\nSize %i",
 						//NULL,
-						e.attr2,
-						(e.attr3 & 0xFF0000) >> 16, (e.attr3 & 0xFF00) >> 8, e.attr3 & 0xFF,
-						e.attr4
+						e.attr[1],
+						(e.attr[2] & 0xFF0000) >> 16, (e.attr[2] & 0xFF00) >> 8, e.attr[2] & 0xFF,
+						e.attr[3]
 					);
 					break;
+				case 9:
+				case 10:
+				case 11:
+				case 12:
+				case 13:
+				case 14:
+				case 15:
+				{
+					pos.z += 12.0f;
+					const static char *typenames[] = {"Flare", "Lightning", "Fire", "Smoke", "Water", "Snow", "Leaves"};
+					formatstring(tmp)("Type: %s (%i)\n%s : %i\nLength: %i\nColour: \fs\fR%i \fJ%i \fD%i\fr\nFadetime w/%s collision: %i\nSize: %i\nVel: %i\nGravity: %i",
+						typenames[e.attr[0] - 9], e.attr[0],
+						e.attr[1] >= 256 ? "Effect" : "Direction", e.attr[1],
+						e.attr[2],
+						(e.attr[3] & 0xFF0000) >> 16, (e.attr[3] & 0xFF00) >> 8, e.attr[3] & 0xFF,
+						e.attr[4] < 0 ? "" : "o", abs(e.attr[4]),
+						e.attr[5],
+						e.attr[6],
+						e.attr[7]
+					);
+					break;
+				}
+
+
+
 				case 32:
 				case 33:
 				case 34:
 				case 35:
 					pos.z += 7.5;
 					formatstring(tmp)("Type: %sLens Flare w/%s Sparkle (%i)\n\fs\fRRed: %i\n\fJGreen: %i\n\fDBlue: %i\n\frSize: %i",
-						(e.attr1==32 || e.attr1 == 33) ? "" : "Fixed-Size ", (e.attr1==32 || e.attr1==34) ? "o" : "", e.attr1,
-						e.attr2,
-						e.attr3,
-						e.attr4,
-						e.attr5
+						(e.attr[0]==32 || e.attr[0] == 33) ? "" : "Fixed-Size ", (e.attr[0]==32 || e.attr[0]==34) ? "o" : "", e.attr[0],
+						e.attr[1],
+						e.attr[2],
+						e.attr[3],
+						e.attr[4]
 					);
-					break;
-				case 13:
-				case 14:
-					pos.z += 12.0f;
-					formatstring(tmp)("Type: %s (%i)\nRadius: %i\nHeight: %i\nColour: \fs\fR%i \fJ%i \fD%i\fr\nSize: %i\nSpeed: %i\nFade: %i\nGravity: %i",
-						e.attr1==13 ? "Flame" : "Smoke Plume", e.attr1,
-						e.attr2,
-						e.attr3,
-						(e.attr4 & 0xFF0000) >> 16, (e.attr4 & 0xFF00) >> 8, e.attr4 & 0xFF,
-						e.attr5,
-						e.attr6,
-						e.attr7,
-						e.attr8
-					);
-
 					break;
 				default:
 					pos.z += 1.5;
-					formatstring(tmp)("Invalid type: %i", e.attr1);
+					formatstring(tmp)("Invalid type: %i", e.attr[0]);
 					break;
 			}
 			break;
@@ -1580,15 +1619,15 @@ void renderentinfo(int i)
 		case ET_SOUND:
 			pos.z += 4.5;
 			formatstring(tmp)("Index: %i\nRadius: %i\nListen Radius: %i",
-				e.attr1,
-				e.attr2,
-				e.attr3
+				e.attr[0],
+				e.attr[1],
+				e.attr[2]
 			);
 			break;
 
 		case ET_SPOTLIGHT:
 			pos.z += 1.5;
-			formatstring(tmp)("Angle: %i", e.attr1);
+			formatstring(tmp)("Angle: %i", e.attr[0]);
 			break;
 		default:
 			entities::renderhelpertext(e, colour, pos, tmp);
@@ -1703,11 +1742,11 @@ void updateparticles()
 			renderentinfo(entgroup[i]);
 
 			extentity &e = *ents[entgroup[i]];
-			if(e.type == ET_PARTICLES && e.attr1 == 11)
+			if(e.type == ET_PARTICLES && e.attr[0] == 11)
 			{
-				defformatstring(aliasname)("part_text_%d", e.attr2);
+				defformatstring(aliasname)("part_text_%d", e.attr[1]);
 				defformatstring(party_text)("@particles:%s", getalias(aliasname));
-				partent_text(e.o, party_text, e.attr3 & 0xFFFFFF, e.attr4 > 0 ? e.attr4 : 200);
+				partent_text(e.o, party_text, e.attr[2] & 0xFFFFFF, e.attr[3] > 0 ? e.attr[3] : 200);
 			}
 			else
 				particle_textcopy(e.o, entname(e), PART_TEXT, 1, 0xFF4B19, 2.0f);
@@ -1718,11 +1757,11 @@ void updateparticles()
 			entity &e = *ents[i];
 			if(e.type==ET_EMPTY) continue;
 
-			if(e.type == ET_PARTICLES && e.attr1 == 11)
+			if(e.type == ET_PARTICLES && e.attr[0] == 11)
 			{
-				defformatstring(aliasname)("part_text_%d", e.attr2);
+				defformatstring(aliasname)("part_text_%d", e.attr[1]);
 				defformatstring(party_text)("@particles:%s", getalias(aliasname));
-				partent_text(e.o, party_text, e.attr3 & 0xFFFFFF, e.attr4 > 0 ? e.attr4 : 200);
+				partent_text(e.o, party_text, e.attr[2] & 0xFFFFFF, e.attr[3] > 0 ? e.attr[3] : 200);
 			}
 			else
 				particle_textcopy(e.o, entname(e), PART_TEXT, 1, 0x1EC850, 2.0f);
