@@ -4,11 +4,25 @@ Texture *sky[6] = { 0, 0, 0, 0, 0, 0 }, *clouds[6] = { 0, 0, 0, 0, 0, 0 };
 
 void loadsky(const char *basename, Texture *texs[6])
 {
+    const char *wildcard = strchr(basename, '*');
     loopi(6)
     {
         const char *side = cubemapsides[i].name;
-	defformatstring(name)("%s_%s", makerelpath("packages", basename), side);
-        if((texs[i] = textureload(name, 3, true, false))==notexture) conoutf(CON_ERROR, "could not load sky texture %s", name);
+        string name;
+        copystring(name, makerelpath("packages", basename));
+        if(wildcard)
+        {
+            char *chop = strchr(name, '*');
+            if(chop) { *chop = '\0'; concatstring(name, side); concatstring(name, wildcard+1); }
+            texs[i] = textureload(name, 3, true, false);
+        }
+        else
+        {
+            defformatstring(ext)("_%s", side);
+            concatstring(name, ext);
+            texs[i] = textureload(name, 3, true, false);
+        }
+        if(texs[i]==notexture) conoutf(CON_ERROR, "could not load side %s of sky texture %s", side, basename);
     }
 }
 
@@ -16,9 +30,16 @@ Texture *cloudoverlay = NULL;
 
 Texture *loadskyoverlay(const char *basename)
 {
-    defformatstring(name)("%s", makerelpath("packages", basename));
-    Texture *t = textureload(name, 0, true, false);
-    if(t==notexture) conoutf(CON_ERROR, "could not load sky overlay texture packages/%s", basename);
+    //const char *ext = strrchr(basename, '.');
+    string name;
+    copystring(name, makerelpath("packages", basename));
+    Texture *t = notexture;
+    //if(ext) t = textureload(name, 0, true, false);
+    //else
+    //{
+        t = textureload(name, 0, true, false);
+    //}
+    if(t==notexture) conoutf(CON_ERROR, "could not load sky overlay texture %s", basename);
     return t;
 }
 
@@ -451,7 +472,7 @@ void drawskybox(int farplane, bool limited)
         if(refracting<0) topclip = 0.5f + 0.5f*(reflectz-camera1->o.z)/float(worldsize);
         else if(reflectz>skyclip) skyclip = reflectz;
     }
-    if(skyclip) skyclip = 0.5f + 0.5f*(skyclip-camera1->o.z)/float(worldsize); 
+    if(skyclip) skyclip = 0.5f + 0.5f*(skyclip-camera1->o.z)/float(worldsize);
 
     if(glaring)
     {
