@@ -82,6 +82,8 @@ struct rendertarget
         glBindFramebuffer_(GL_FRAMEBUFFER_EXT, 0);
     }
 
+    virtual bool shadowcompare() const { return false; }
+
     void setup(int w, int h)
     {
         if(hasFBO)
@@ -115,6 +117,21 @@ struct rendertarget
         while(!colorfmt && colorfmts[++find]);
         if(!colorfmt) colorfmt = colorfmts[find];
 
+        if(attach == GL_DEPTH_ATTACHMENT_EXT && shadowcompare())
+        {
+            if(hasDT && hasSH)
+            {
+                glTexParameteri(target, GL_TEXTURE_COMPARE_MODE_ARB, GL_COMPARE_R_TO_TEXTURE);
+                glTexParameteri(target, GL_TEXTURE_COMPARE_FUNC_ARB, GL_GEQUAL);
+                glTexParameteri(target, GL_DEPTH_TEXTURE_MODE_ARB, GL_LUMINANCE);
+            }
+            else
+            {
+                glTexParameteri(target, GL_TEXTURE_COMPARE_SGIX, GL_TRUE);
+                glTexParameteri(target, GL_TEXTURE_COMPARE_OPERATOR_SGIX, GL_TEXTURE_GEQUAL_R_SGIX);
+            }
+        } 
+
         if(hasFBO && attach != GL_DEPTH_ATTACHMENT_EXT && depthtest())
         {
             if(!renderdb) { glGenRenderbuffers_(1, &renderdb); depthfmt = GL_FALSE; }
@@ -465,11 +482,12 @@ struct rendertarget
             sw = int(0.5f*(scissorx2 - scissorx1)*w),
             sh = int(0.5f*(scissory2 - scissory1)*h);
         if(flipdebug()) { sy = h - sy; sh = -sh; }
-        glBegin(lines ? GL_LINE_LOOP : GL_QUADS);
+        glBegin(lines ? GL_LINE_LOOP : GL_TRIANGLE_STRIP);
         glVertex2i(sx,      sy);
         glVertex2i(sx + sw, sy);
-        glVertex2i(sx + sw, sy + sh);
+        if(lines) glVertex2i(sx + sw, sy + sh);
         glVertex2i(sx,      sy + sh);
+        if(!lines) glVertex2i(sx + sw, sy + sh);
         glEnd();
     }
 
@@ -498,11 +516,12 @@ struct rendertarget
                 loopi(lines ? 1 : 2)
                 {
                     if(!lines) glColor3f(1, 1, i ? 1.0f : 0.5f);
-                    glBegin(lines || i ? GL_LINE_LOOP : GL_QUADS);
+                    glBegin(lines || i ? GL_LINE_LOOP : GL_TRIANGLE_STRIP);
                     glVertex2f(vx,    vy);
                     glVertex2f(vx+vw, vy);
-                    glVertex2f(vx+vw, vy+vh);
+                    if(lines || i) glVertex2f(vx+vw, vy+vh);
                     glVertex2f(vx,    vy+vh);
+                    if(!lines && !i) glVertex2f(vx+vw, vy+vh);
                     glEnd();
                 }
             }
@@ -524,11 +543,11 @@ struct rendertarget
             ty2 /= viewh;
         }
         if(flipdebug()) swap(ty1, ty2);
-        glBegin(GL_QUADS);
+        glBegin(GL_TRIANGLE_STRIP);
         glTexCoord2f(tx1, ty1); glVertex2i(0, 0);
         glTexCoord2f(tx2, ty1); glVertex2i(w, 0);
-        glTexCoord2f(tx2, ty2); glVertex2i(w, h);
         glTexCoord2f(tx1, ty2); glVertex2i(0, h);
+        glTexCoord2f(tx2, ty2); glVertex2i(w, h);
         glEnd();
         notextureshader->set();
         glDisable(target);
