@@ -262,17 +262,26 @@ enum { UNRESOLVED = 0, RESOLVING, RESOLVED };
 
 struct serverinfo
 {
+    enum 
+    { 
+        WAITING = INT_MAX,
+
+        MAXPINGS = 3 
+    };
+
     string name, map, sdesc;
-    int port, numplayers, ping, resolved, lastping;
+    int port, numplayers, resolved, ping, lastping, nextping;
+    int pings[MAXPINGS];
     vector<int> attr;
     ENetAddress address;
     bool keep;
     const char *password;
 
     serverinfo()
-     : port(-1), numplayers(0), ping(INT_MAX), resolved(UNRESOLVED), lastping(-1), keep(false), password(NULL)
+     : port(-1), numplayers(0), resolved(UNRESOLVED), keep(false), password(NULL)
     {
         name[0] = map[0] = sdesc[0] = '\0';
+        clearpings();
     }
 
     ~serverinfo()
@@ -280,6 +289,21 @@ struct serverinfo
         DELETEA(password);
     }
 
+    void clearpings()
+    {
+        ping = WAITING;
+        loopk(MAXPINGS) pings[k] = WAITING;
+        nextping = 0;
+        lastping = -1;
+    }
+
+    void cleanup()
+    {
+        clearpings();
+        attr.setsize(0);
+        numplayers = 0;
+    }
+
     void reset()
     {
         lastping = -1;
@@ -288,19 +312,23 @@ struct serverinfo
     void checkdecay(int decay)
     {
         if(lastping >= 0 && totalmillis - lastping >= decay)
-        {
-            ping = INT_MAX;
-            numplayers = 0;
-            lastping = -1;
-        }
+            cleanup();
         if(lastping < 0) lastping = totalmillis;
     }
 
+    void calcping()
+    {
+        int numpings = 0, totalpings = 0;
+        loopk(MAXPINGS) if(pings[k] != WAITING) { totalpings += pings[k]; numpings++; }
+        ping = numpings ? totalpings/numpings : WAITING;
+    }
+
     void addping(int rtt, int millis)
     {
         if(millis >= lastping) lastping = -1;
-        if(ping == INT_MAX) ping = rtt;
-        else ping = (ping*4 + rtt)/5;
+        pings[nextping] = rtt;
+        nextping = (nextping+1)%MAXPINGS;
+        calcping();
     }
 
     static bool compare(serverinfo *a, serverinfo *b)
@@ -450,6 +478,8 @@ void checkresolver()
     }
 }
 
+static int lastreset = 0;
+
 void checkpings()
 {
     if(pingsock==ENET_SOCKET_NULL) return;
@@ -470,7 +500,7 @@ void checkpings()
         if(!si) continue;
         ucharbuf p(ping, len);
         int millis = getint(p), rtt = clamp(totalmillis - millis, 0, min(servpingdecay, totalmillis));
-        if(rtt < servpingdecay) si->addping(rtt, millis);
+        if(millis >= lastreset && rtt < servpingdecay) si->addping(rtt, millis);
         si->numplayers = getint(p);
         int numattr = getint(p);
         si->attr.shrink(0);
@@ -486,7 +516,11 @@ void refreshservers()
 {
     static int lastrefresh = 0;
     if(lastrefresh==totalmillis) return;
-    if(totalmillis - lastrefresh > 1000) loopv(servers) servers[i]->reset();
+    if(totalmillis - lastrefresh > 1000) 
+    {
+        loopv(servers) servers[i]->reset();
+        lastreset = totalmillis;
+    }
     lastrefresh = totalmillis;
 
     checkresolver();
@@ -514,7 +548,7 @@ char *showservers(g3d_gui *cgui)
               serverinfo &si = *servers[j];
               const char *sdesc = si.sdesc;
               if(si.address.host == ENET_HOST_ANY) sdesc = "[unknown host]";
-                else if(si.ping == INT_MAX) sdesc = "[waiting for response]";
+                else if(si.ping == serverinfo::WAITING) sdesc = "[waiting for response]";
                 if(game::serverinfoentry(cgui, i, si.name, si.port, sdesc, si.map, sdesc == si.sdesc ? si.ping : -1, si.attr, si.numplayers))
                     sc = &si;
             }
@@ -615,7 +649,7 @@ COMMAND(updatefrommaster, "");
 void writeservercfg()
 {
     if(!game::savedservers()) return;
-    stream *f = openfile(path(game::savedservers(), true), "w");
+    stream *f = openutf8file(path(game::savedservers(), true), "wb");
     if(!f) return;
     int kept = 0;
     loopv(servers)
