@@ -111,6 +111,20 @@ namespace UI
         NO_ADJUST     = ALIGN_HNONE | ALIGN_VNONE,
     };
 
+    enum
+    {
+        TYPE_MISC = 0,
+        TYPE_SCROLLER,
+        TYPE_SCROLLBAR,
+        TYPE_SCROLLBUTTON,
+        TYPE_SLIDER,
+        TYPE_SLIDERBUTTON,
+        TYPE_IMAGE,
+        TYPE_TAG,
+        TYPE_WINDOW,
+        TYPE_TEXTEDITOR
+    };
+
     static vector<char *> executelater;
 
     struct Object
@@ -304,31 +318,36 @@ namespace UI
         }
 
         virtual const char *getname() const { return ""; }
+        virtual const int gettype() const { return TYPE_MISC; }
 
         bool isnamed(const char *name) const { return !strcmp(name, getname()); }
 
-        Object *findname(const char *name, bool recurse = true, const Object *exclude = NULL) const
+        Object *findname(int type, const char *name, bool recurse = true, const Object *exclude = NULL) const
         {
             loopchildren(o,
             {
-                if(o != exclude && o->isnamed(name)) return o;
+                if(o != exclude &&
+                    o->gettype() == type &&
+                    (!name || o->isnamed(name))
+                )
+                    return o;
             });
             if(recurse) loopchildren(o,
             {
                 if(o != exclude)
                 {
-                    Object *found = o->findname(name);
+                    Object *found = o->findname(type, name);
                     if(found) return found;
                 }
             });
             return NULL;
         }
 
-        Object *findsibling(const char *name) const
+        Object *findsibling(int type, const char *name) const
         {
             for(const Object *prev = this, *cur = parent; cur; prev = cur, cur = cur->parent)
             {
-                Object *o = cur->findname(name, true, prev);
+                Object *o = cur->findname(type, name, true, prev);
                 if(o) return o;
             }
             return NULL;
@@ -365,14 +384,13 @@ namespace UI
         {
             if(children.empty()) return;
 
-            float childspace = (w - children.last()->x - children.last()->w) / max(children.length() - 1, 1),
-                  offset = 0;
+            float offset = 0;
             loopchildren(o,
             {
                 o->x = offset;
                 offset += o->w;
-                if(i < children.length()) offset += childspace;
                 o->adjustlayout(o->x, 0, offset - o->x, h);
+                offset += space;
             });
         }
     };
@@ -401,15 +419,13 @@ namespace UI
         {
             if(children.empty()) return;
 
-            float childspace = (h - children.last()->y - children.last()->h) / max(children.length() - 1, 1),
-                  offset = 0;
+            float offset = 0;
             loopchildren(o,
             {
                 o->y = offset;
                 offset += o->h;
-                if(i < children.length()) offset += childspace;
                 o->adjustlayout(0, o->y, w, offset - o->y);
-
+                offset += space;
             });
         }
 
@@ -732,7 +748,7 @@ namespace UI
         void sethscroll(float hscroll) { offsetx = clamp(hscroll, 0.0f, hlimit()); }
         void setvscroll(float vscroll) { offsety = clamp(vscroll, 0.0f, vlimit()); }
 
-        const char *getname() const { return "scroller"; }
+        const int gettype() const { return TYPE_SCROLLER; }
     };
 
     struct ScrollBar : Object
@@ -769,7 +785,7 @@ namespace UI
             return target(cx, cy) ? this : NULL;
         }
 
-        const char *getname() const { return "scrollbar"; }
+        const int gettype() const { return TYPE_SCROLLBAR; }
 
         virtual void scrollto(float cx, float cy)
         {
@@ -794,7 +810,7 @@ namespace UI
             }
             else
             {
-                Object *button = findname("scrollbutton", false);
+                Object *button = findname(TYPE_SCROLLBUTTON, NULL, false);
                 if(button && isselected(button))
                 {
                     arrowdir = 0;
@@ -835,8 +851,8 @@ namespace UI
         {
             if(isselected(this))
             {
-                ScrollBar *scrollbar = dynamic_cast<ScrollBar *>(parent);
-                if(!scrollbar) return;
+                if(!parent || parent->gettype() != TYPE_SCROLLBAR) return;
+                ScrollBar *scrollbar = (ScrollBar *) parent;
                 scrollbar->movebutton(this, offsetx, offsety, cx, cy);
             }
         }
@@ -847,7 +863,7 @@ namespace UI
             offsety = cy;
         }
 
-        const char *getname() const { return "scrollbutton"; }
+        const int gettype() const { return TYPE_SCROLLBUTTON; }
     };
 
     struct HorizontalScrollBar : ScrollBar
@@ -863,16 +879,16 @@ namespace UI
 
         void arrowscroll()
         {
-            Scroller *scroller = dynamic_cast<Scroller *>(findsibling("scroller"));
+            Scroller *scroller = (Scroller *) findsibling(TYPE_SCROLLER, NULL);
             if(!scroller) return;
             scroller->addhscroll(arrowdir*arrowspeed*curtime/1000.0f);
         }
 
         void scrollto(float cx, float cy)
         {
-            Scroller *scroller = dynamic_cast<Scroller *>(findsibling("scroller"));
+            Scroller *scroller = (Scroller *) findsibling(TYPE_SCROLLER, NULL);
             if(!scroller) return;
-            ScrollButton *button = dynamic_cast<ScrollButton *>(findname("scrollbutton", false));
+            ScrollButton *button = (ScrollButton *) findname(TYPE_SCROLLBUTTON, NULL, false);
             if(!button) return;
             float bscale = (max(w - 2*arrowsize, 0.0f) - button->w) / (1 - scroller->hscale()),
                   offset = bscale > 1e-3f ? (cx - arrowsize)/bscale : 0;
@@ -881,9 +897,9 @@ namespace UI
 
         void adjustchildren()
         {
-            Scroller *scroller = dynamic_cast<Scroller *>(findsibling("scroller"));
+            Scroller *scroller = (Scroller *) findsibling(TYPE_SCROLLER, NULL);
             if(!scroller) return;
-            ScrollButton *button = dynamic_cast<ScrollButton *>(findname("scrollbutton", false));
+            ScrollButton *button = (ScrollButton *) findname(TYPE_SCROLLBUTTON, NULL, false);
             if(!button) return;
             float bw = max(w - 2*arrowsize, 0.0f)*scroller->hscale();
             button->w = max(button->w, bw);
@@ -913,16 +929,16 @@ namespace UI
 
         void arrowscroll()
         {
-            Scroller *scroller = dynamic_cast<Scroller *>(findsibling("scroller"));
+            Scroller *scroller = (Scroller *) findsibling(TYPE_SCROLLER, NULL);
             if(!scroller) return;
             scroller->addvscroll(arrowdir*arrowspeed*curtime/1000.0f);
         }
 
         void scrollto(float cx, float cy)
         {
-            Scroller *scroller = dynamic_cast<Scroller *>(findsibling("scroller"));
+            Scroller *scroller = (Scroller *) findsibling(TYPE_SCROLLER, NULL);
             if(!scroller) return;
-            ScrollButton *button = dynamic_cast<ScrollButton *>(findname("scrollbutton", false));
+            ScrollButton *button = (ScrollButton *) findname(TYPE_SCROLLBUTTON, NULL, false);
             if(!button) return;
             float bscale = (max(h - 2*arrowsize, 0.0f) - button->h) / (1 - scroller->vscale()),
                   offset = bscale > 1e-3f ? (cy - arrowsize)/bscale : 0;
@@ -931,9 +947,9 @@ namespace UI
 
         void adjustchildren()
         {
-            Scroller *scroller = dynamic_cast<Scroller *>(findsibling("scroller"));
+            Scroller *scroller = (Scroller *) findsibling(TYPE_SCROLLER, NULL);
             if(!scroller) return;
-            ScrollButton *button = dynamic_cast<ScrollButton *>(findname("scrollbutton", false));
+            ScrollButton *button = (ScrollButton *) findname(TYPE_SCROLLBUTTON, NULL, false);
             if(!button) return;
             float bh = max(h - 2*arrowsize, 0.0f)*scroller->vscale();
             button->h = max(button->h, bh);
@@ -950,6 +966,254 @@ namespace UI
         }
     };
 
+    static float getfval(ident *id)
+    {
+        switch(id->type)
+        {
+            case ID_VAR: return *id->storage.i;
+            case ID_FVAR: return *id->storage.f;
+            case ID_SVAR: return parsefloat(*id->storage.s);
+            case ID_ALIAS: return id->getfloat();
+            default: return 0;
+        }
+    }
+
+    struct Slider : Object
+    {
+        ident *var;
+        float vmin, vmax;
+        float arrowsize, arrowspeed;
+        int arrowdir;
+
+        Slider(char *varname, float min = 0, float max = 0, float arrowsize = 0, float arrowspeed = 0) : vmin(min), vmax(max), arrowsize(arrowsize), arrowspeed(arrowspeed), arrowdir(0)
+        {
+            var = getident(varname);
+            if(!var)
+            {
+                alias(varname, "0");
+                var = getident(varname);
+            }
+            if(vmin == 0 && vmax == 0)
+            {
+                if(var->type == ID_VAR)
+                {
+                    vmin = var->minval;
+                    vmax = var->maxval;
+                }
+                else if(var->type == ID_FVAR)
+                {
+                    vmin = var->minvalf;
+                    vmax = var->maxvalf;
+                }
+            }
+        }
+
+        int forks() const { return 5; }
+        int choosefork() const
+        {
+            switch(arrowdir)
+            {
+                case -1: return isselected(this) ? 2 : (ishovering(this) ? 1 : 0);
+                case 1: return isselected(this) ? 4 : (ishovering(this) ? 3 : 0);
+            }
+            return 0;
+        }
+
+        virtual int choosedir(float cx, float cy) const { return 0; }
+
+        Object *hover(float cx, float cy)
+        {
+            Object *o = Object::hover(cx, cy);
+            if(o) return o;
+            return target(cx, cy) ? this : NULL;
+        }
+
+        Object *select(float cx, float cy)
+        {
+            Object *o = Object::select(cx, cy);
+            if(o) return o;
+            return target(cx, cy) ? this : NULL;
+        }
+
+        const int gettype() const { return TYPE_SLIDER; }
+
+        virtual void scrollto(float cx, float cy)
+        {
+        }
+
+        void selected(float cx, float cy)
+        {
+            arrowdir = choosedir(cx, cy);
+            if(!arrowdir) scrollto(cx, cy);
+            else hovering(cx, cy);
+        }
+
+        void arrowscroll()
+        {
+            float pos = (getfval(var) - vmin) / (vmax - vmin);
+            pos += clamp<float>(1 - pos, -pos, arrowdir * arrowspeed * curtime / fabs(vmax - vmin));
+
+            float val = vmin + pos * (vmax - vmin);
+            setval(val);
+        }
+
+        void setval(float val)
+        {
+            string cmd = "";
+            switch(var->type)
+            {
+                case ID_VAR: case ID_FVAR: case ID_SVAR:
+                    formatstring(cmd)("%s %g", var->name, val);
+                    break;
+                case ID_ALIAS:
+                    formatstring(cmd)("alias %s %g", var->name, val);
+                    break;
+            }
+            executelater.add(newstring(cmd));
+        }
+
+        void hovering(float cx, float cy)
+        {
+            if(isselected(this))
+            {
+                if(arrowdir) arrowscroll();
+            }
+            else
+            {
+                Object *button = findname(TYPE_SLIDERBUTTON, NULL, false);
+                if(button && isselected(button))
+                {
+                    arrowdir = 0;
+                    button->hovering(cx - button->x, cy - button->y);
+                }
+                else arrowdir = choosedir(cx, cy);
+            }
+        }
+
+        bool allowselect(Object *o)
+        {
+            return children.find(o) >= 0;
+        }
+
+        virtual void movebutton(Object *o, float fromx, float fromy, float tox, float toy) = 0;
+    };
+
+    struct SliderButton : Object
+    {
+        float offsetx, offsety;
+
+        SliderButton() : offsetx(0), offsety(0) {}
+
+        int forks() const { return 3; }
+        int choosefork() const { return isselected(this) ? 2 : (ishovering(this) ? 1 : 0); }
+
+        Object *hover(float cx, float cy)
+        {
+            return target(cx, cy) ? this : NULL;
+        }
+
+        Object *select(float cx, float cy)
+        {
+            return target(cx, cy) ? this : NULL;
+        }
+
+        void hovering(float cx, float cy)
+        {
+            if(isselected(this))
+            {
+                if(!parent || parent->gettype() != TYPE_SLIDER) return;
+                Slider *slider = (Slider *) parent;
+                slider->movebutton(this, offsetx, offsety, cx, cy);
+            }
+        }
+
+        void selected(float cx, float cy)
+        {
+            offsetx = cx;
+            offsety = cy;
+        }
+
+        const int gettype() const { return TYPE_SLIDERBUTTON; }
+    };
+
+    struct HorizontalSlider : Slider
+    {
+        HorizontalSlider(char *varname, float vmin = 0, float vmax = 0, float arrowsize = 0, float arrowspeed = 0) : Slider(varname, vmin, vmax, arrowsize, arrowspeed) {}
+
+        int choosedir(float cx, float cy) const
+        {
+            if(cx < arrowsize) return -1;
+            else if(cx >= w - arrowsize) return 1;
+            return 0;
+        }
+
+        void scrollto(float cx, float cy)
+        {
+            SliderButton *button = (SliderButton *) findname(TYPE_SLIDERBUTTON, NULL, false);
+            if(!button) return;
+            float pos = clamp(1.0f, 0.0f, (cx - arrowsize) / (w - 2 * arrowsize - button->w));
+            float val = vmin + pos * (vmax - vmin);
+            setval(val);
+        }
+
+        void adjustchildren()
+        {
+            SliderButton *button = (SliderButton *) findname(TYPE_SLIDERBUTTON, NULL, false);
+            if(!button) return;
+            float bw = max(w - 2 * arrowsize, 0.0f) / fabs(vmax - vmin);
+            button->w = max(button->w, bw);
+            float bscale = fabs(vmax - vmin) > 0 ? (max(w - 2 * arrowsize, 0.0f) - button->w) / fabs(vmax - vmin) : 1;
+            button->x = arrowsize + (getfval(var) - vmin) * bscale;
+            button->adjust &= ~ALIGN_HMASK;
+
+            Slider::adjustchildren();
+        }
+
+        void movebutton(Object *o, float fromx, float fromy, float tox, float toy)
+        {
+            scrollto(o->x + tox - fromx, o->y + toy);
+        }
+    };
+
+    struct VerticalSlider : Slider
+    {
+        VerticalSlider(char *varname, float vmin = 0, float vmax = 0, float arrowsize = 0, float arrowspeed = 0) : Slider(varname, vmin, vmax, arrowsize, arrowspeed) {}
+
+        int choosedir(float cx, float cy) const
+        {
+            if(cy < arrowsize) return -1;
+            else if(cy >= h - arrowsize) return 1;
+            return 0;
+        }
+
+        void scrollto(float cx, float cy)
+        {
+            SliderButton *button = (SliderButton *) findname(TYPE_SLIDERBUTTON, NULL, false);
+            if(!button) return;
+            float pos = clamp(1.0f, 0.0f, (cy - arrowsize) / (h - 2 * arrowsize - button->h));
+            float val = vmin + pos * (vmax - vmin);
+            setval(val);
+        }
+
+        void adjustchildren()
+        {
+            SliderButton *button = (SliderButton *) findname(TYPE_SLIDERBUTTON, NULL, false);
+            if(!button) return;
+            float bh = max(h - 2 * arrowsize, 0.0f) / fabs(vmax - vmin);
+            button->h = max(button->h, bh);
+            float bscale = fabs(vmax - vmin) > 0 ? (max(h - 2 * arrowsize, 0.0f) - button->h) / fabs(vmax - vmin) : 1;
+            button->y = arrowsize + (getfval(var) - vmin) * bscale;
+            button->adjust &= ~ALIGN_HMASK;
+
+            Slider::adjustchildren();
+        }
+
+        void movebutton(Object *o, float fromx, float fromy, float tox, float toy)
+        {
+            scrollto(o->x + tox, o->y + toy - fromy);
+        }
+    };
+
     static bool checkalphamask(Texture *tex, float x, float y)
     {
         if(!tex->alphamask)
@@ -1022,6 +1286,8 @@ namespace UI
 
             Object::draw(sx, sy);
         }
+
+        const int gettype() const { return TYPE_IMAGE; }
     };
 
     VAR(thumbtime, 0, 25, 1000);
@@ -1037,23 +1303,27 @@ namespace UI
         {
             Object *o = Object::target(cx, cy);
             if(o || !texmru.inrange(slotnum)) return o;
-            Slot &slot = lookupslot(texmru[slotnum], false);
-            if(slot.sts.length() && (slot.loaded || slot.thumbnail)) return this;
+            VSlot &vslot = lookupvslot(texmru[slotnum], false);
+            if(vslot.slot->sts.length() && (vslot.slot->loaded || vslot.slot->thumbnail)) return this;
             return NULL;
         }
 
-        void drawslot(Slot &slot, float sx, float sy)
+        void drawslot(Slot &slot, VSlot &vslot, float sx, float sy)
         {
             Texture *tex = notexture, *glowtex = NULL, *layertex = NULL;
-	    VSlot &vslot = *slot.variants;
+            VSlot *layer = NULL;
             if(slot.loaded)
             {
                 tex = slot.sts[0].t;
-                if(slot.texmask&(1<<TEX_GLOW)) { loopv(slot.sts) if(slot.sts[i].type==TEX_GLOW) { glowtex = slot.sts[i].t; break; } }
+                if(slot.texmask&(1<<TEX_GLOW)) {
+                    loopv(slot.sts) if(slot.sts[i].type==TEX_GLOW)
+                    { glowtex = slot.sts[i].t; break; }
+                }
                 if(vslot.layer)
                 {
-                    Slot &layer = lookupslot(vslot.layer);
-                    layertex = layer.sts[0].t;
+                    layer = &lookupvslot(vslot.layer);
+                    if(!layer->slot->sts.empty())
+                        layertex = layer->slot->sts[0].t;
                 }
             }
             else if(slot.thumbnail) tex = slot.thumbnail;
@@ -1074,12 +1344,13 @@ namespace UI
                 if(vslot.rotation <= 2 || vslot.rotation == 5) { yoff *= -1; loopk(4) tc[k][1] *= -1; }
             }
             loopk(4) { tc[k][0] = tc[k][0]/xt - float(xoff)/tex->xs; tc[k][1] = tc[k][1]/yt - float(yoff)/tex->ys; }
+            if(slot.loaded) glColor3fv(vslot.colorscale.v);
             glBindTexture(GL_TEXTURE_2D, tex->id);
-            glBegin(GL_QUADS);
+            glBegin(GL_TRIANGLE_STRIP);
             glTexCoord2fv(tc[0]); glVertex2f(sx,   sy);
             glTexCoord2fv(tc[1]); glVertex2f(sx+w, sy);
-            glTexCoord2fv(tc[2]); glVertex2f(sx+w, sy+h);
             glTexCoord2fv(tc[3]); glVertex2f(sx,   sy+h);
+            glTexCoord2fv(tc[2]); glVertex2f(sx+w, sy+h);
             glEnd();
 
             if(glowtex)
@@ -1087,25 +1358,27 @@ namespace UI
                 glBlendFunc(GL_SRC_ALPHA, GL_ONE);
                 glBindTexture(GL_TEXTURE_2D, glowtex->id);
                 glColor3fv(vslot.glowcolor.v);
-                glBegin(GL_QUADS);
+                glBegin(GL_TRIANGLE_STRIP);
                 glTexCoord2fv(tc[0]); glVertex2f(sx,   sy);
                 glTexCoord2fv(tc[1]); glVertex2f(sx+w, sy);
-                glTexCoord2fv(tc[2]); glVertex2f(sx+w, sy+h);
                 glTexCoord2fv(tc[3]); glVertex2f(sx,   sy+h);
+                glTexCoord2fv(tc[2]); glVertex2f(sx+w, sy+h);
                 glEnd();
-                glColor3f(1, 1, 1);
+
                 glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
             }
             if(layertex)
             {
                 glBindTexture(GL_TEXTURE_2D, layertex->id);
-                glBegin(GL_QUADS);
+                glColor3fv(layer->colorscale.v);
+                glBegin(GL_TRIANGLE_STRIP);
                 glTexCoord2fv(tc[0]); glVertex2f(sx+w/2, sy+h/2);
                 glTexCoord2fv(tc[1]); glVertex2f(sx+w,   sy+h/2);
-                glTexCoord2fv(tc[2]); glVertex2f(sx+w,   sy+h);
                 glTexCoord2fv(tc[3]); glVertex2f(sx+w/2, sy+h);
+                glTexCoord2fv(tc[2]); glVertex2f(sx+w,   sy+h);
                 glEnd();
             }
+            glColor3f(1, 1, 1);
 
             defaultshader->set();
         }
@@ -1114,10 +1387,11 @@ namespace UI
         {
             if(texmru.inrange(slotnum))
             {
-                Slot &slot = lookupslot(texmru[slotnum], false);
+                VSlot &vslot = lookupvslot(texmru[slotnum], false);
+                Slot &slot = *vslot.slot;
                 if(slot.sts.length())
                 {
-                    if(slot.loaded || slot.thumbnail) drawslot(slot, sx, sy);
+                    if(slot.loaded || slot.thumbnail) drawslot(slot, vslot, sx, sy);
                     else if(totalmillis-lastthumbnail >= thumbtime)
                     {
                         loadthumbnail(slot);
@@ -1301,6 +1575,12 @@ namespace UI
             : str(newstring(str)), scale(scale), color(r, g, b) {}
         ~Text() { delete[] str; }
 
+        Object *target(float cx, float cy)
+        {
+            Object *o = Object::target(cx, cy);
+            return o ? o : this;
+        }
+
         float drawscale() const { return scale / (FONTH * uitextrows); }
 
         void draw(float sx, float sy)
@@ -1327,6 +1607,60 @@ namespace UI
         }
     };
 
+    static const char *getsval(ident *id)
+    {
+        switch(id->type)
+        {
+            case ID_VAR: return intstr(*id->storage.i);
+            case ID_FVAR: return floatstr(*id->storage.f);
+            case ID_SVAR: return *id->storage.s;
+            case ID_ALIAS: return id->getstr();
+            default: return "";
+        }
+    }
+
+    struct VarText : Object
+    {
+        ident *id;
+        float scale;
+        vec color;
+
+        VarText(const char *var, float scale = 1, float r = 1, float g = 1, float b = 1) : scale(scale), color(r, g, b)
+        {
+            id = getident(var);
+            if(!id)
+            {
+                alias(var, "0");
+                id = getident(var);
+            }
+        }
+
+        float drawscale() const { return scale / (FONTH * uitextrows); }
+
+        void draw(float sx, float sy)
+        {
+            float k = drawscale();
+            glPushMatrix();
+            glScalef(k, k, 1);
+            draw_text(getsval(id), int(sx/k), int(sy/k), color.x * 255, color.y * 255, color.z * 255, 255);
+            glColor3f(1, 1, 1);
+            glPopMatrix();
+
+            Object::draw(sx, sy);
+        }
+
+        void layout()
+        {
+            Object::layout();
+
+            int tw, th;
+            text_bounds(getsval(id), tw, th);
+            float k = drawscale();
+            w = max(w, tw*k);
+            h = max(h, th*k);
+        }
+    };
+
     struct TextEditor : Object
     {
         float scale, offsetx, offsety;
@@ -1450,6 +1784,8 @@ namespace UI
 
             Object::draw(sx, sy);
         }
+
+        const int gettype() const { return TYPE_TEXTEDITOR; }
     };
 
     struct NamedObject : Object
@@ -1465,6 +1801,7 @@ namespace UI
     struct Tag : NamedObject
     {
         Tag(const char *name) : NamedObject(name) {}
+        const int gettype() const { return TYPE_TAG; }
     };
 
     struct World : Object
@@ -1496,6 +1833,8 @@ namespace UI
         {
             if(onhide) execute(onhide);
         }
+
+        const int gettype() const { return TYPE_WINDOW; }
     };
 
     World *world = NULL;
@@ -1514,7 +1853,7 @@ namespace UI
     ICOMMAND(showui, "sss", (char *name, char *contents, char *onhide),
     {
         if(build.length()) { intret(0); return; }
-        Window *oldwindow = dynamic_cast<Window *>(world->findname(name, false));
+        Window *oldwindow = (Window *) world->findname(TYPE_WINDOW, name, false);
         if(oldwindow)
         {
             oldwindow->hidden();
@@ -1528,7 +1867,7 @@ namespace UI
 
     bool hideui(const char *name)
     {
-        Window *window = dynamic_cast<Window *>(world->findname(name, false));
+        Window *window = (Window *) world->findname(TYPE_WINDOW, name, false);
         if(window)
         {
             window->hidden();
@@ -1542,9 +1881,9 @@ namespace UI
     ICOMMAND(replaceui, "sss", (char *wname, char *tname, char *contents),
     {
         if(build.length()) { intret(0); return; }
-        Window *window = dynamic_cast<Window *>(world->findname(wname, false));
+        Window *window = (Window *) world->findname(TYPE_WINDOW, wname, false);
         if(!window) { intret(0); return; }
-        Tag *tag = dynamic_cast<Tag *>(window->findname(tname));
+        Tag *tag = (Tag *) window->findname(TYPE_TAG, tname);
         if(!tag) { intret(0); return; }
         tag->children.deletecontents();
         build.add(tag);
@@ -1615,6 +1954,15 @@ namespace UI
     ICOMMAND(uiscrollbutton, "s", (char *children),
         addui(new ScrollButton, children));
 
+    ICOMMAND(uihslider, "sffffs", (char *var, float *vmin, float *vmax, float *arrowsize, float *arrowspeed, char *children),
+        addui(new HorizontalSlider(var, *vmin, *vmax, *arrowsize, *arrowspeed), children));
+
+    ICOMMAND(uivslider, "sffffs", (char *var, float *vmin, float *vmax, float *arrowsize, float *arrowspeed, char *children),
+        addui(new VerticalSlider(var, *vmin, *vmax, *arrowsize, *arrowspeed), children));
+
+    ICOMMAND(uisliderbutton, "s", (char *children),
+        addui(new SliderButton, children));
+
     ICOMMAND(uioffset, "ffs", (float *offsetx, float *offsety, char *children),
         addui(new Offsetter(*offsetx, *offsety), children));
 
@@ -1638,8 +1986,8 @@ namespace UI
 
     ICOMMAND(uialtimage, "s", (char *texname),
     {
-        if(build.empty()) return;
-        Image *image = dynamic_cast<Image *>(build.last());
+        if(build.empty() || build.last()->gettype() != TYPE_IMAGE) return;
+        Image *image = (Image *) build.last();
         if(image && image->tex==notexture) image->tex = textureload(texname, 3, true, false);
     });
 
@@ -1672,6 +2020,12 @@ namespace UI
     ICOMMAND(uitext, "sfs", (char *text, float *scale, char *children),
         addui(new Text(text, *scale <= 0 ? 1 : *scale), children));
 
+    ICOMMAND(uicolorvartext, "sffffs", (char *var, float *scale, float *r, float *g, float *b, char *children),
+        addui(new VarText(var, *scale <= 0 ? 1 : *scale, *r, *g, *b), children));
+
+    ICOMMAND(uivartext, "sfs", (char *var, float *scale, char *children),
+        addui(new VarText(var, *scale <= 0 ? 1 : *scale), children));
+
     ICOMMAND(uitexteditor, "siifsiss", (char *name, int *length, int *height, float *scale, char *initval, int *keep, char *filter, char *children),
         addui(new TextEditor(name, *length, *height, *scale, initval, *keep ? EDITORFOREVER : EDITORUSED, filter[0] ? filter : NULL), children));
 
@@ -1744,12 +2098,16 @@ namespace UI
         }
     }
 
-	VAR(mainmenu, 1, 1, 0);
-	void clearmainmenu()
-	{
-		if(mainmenu && (isconnected() || haslocalclients()))
-			mainmenu = 0;
-	}
+    VAR(mainmenu, 1, 1, 0);
+    void clearmainmenu()
+    {
+        if(mainmenu && (isconnected() || haslocalclients()))
+        {
+            hideui("main");
+            mainmenu = 0;
+
+        }
+    }
 
     void setup()
     {
@@ -1757,12 +2115,18 @@ namespace UI
     }
 
     TextEditor *textediting = NULL;
+    int showchanges = 0;
 
     void update()
     {
         loopv(executelater) execute(executelater[i]);
         executelater.deletearrays();
 
+        if(showchanges && !world->findname(TYPE_WINDOW, NULL, false))
+        {
+            if(identexists("showchanges")) execute("showchanges");
+        }
+
         if(mainmenu && !isconnected(true) && !world->children.length() && identexists("showmain"))
         {
             execute("showmain");
@@ -1780,7 +2144,11 @@ namespace UI
         world->layout();
 
         bool wastextediting = textediting!=NULL;
-        textediting = dynamic_cast<TextEditor *>(focused);
+        if(!focused || focused->gettype() != TYPE_TEXTEDITOR)
+            textediting = NULL;
+        else
+            textediting = (TextEditor *) focused;
+
         if((textediting!=NULL) != wastextediting)
         {
             SDL_EnableUNICODE(textediting!=NULL);
@@ -1833,7 +2201,7 @@ void addchange(const char *desc, int type)
     if(!applydialog) return;
     loopv(needsapply) if(!strcmp(needsapply[i].desc, desc)) return;
     needsapply.add(change(type, desc));
-    if(identexists("showchanges")) execute("showchanges");
+    UI::showchanges = 1;
 }
 
 void clearchanges(int type)
@@ -1846,7 +2214,11 @@ void clearchanges(int type)
             if(!needsapply[i].type) needsapply.remove(i--);
         }
     }
-    if(needsapply.empty()) UI::hideui("changes");
+    if(needsapply.empty())
+    {
+        UI::showchanges = 0;
+        UI::hideui("changes");
+    }
 }
 
 void applychanges()
@@ -1857,6 +2229,7 @@ void applychanges()
     if(changetypes&CHANGE_SOUND) UI::executelater.add(newstring("resetsound"));
 }
 
+ICOMMAND(pendingchanges, "", (), intret(needsapply.length()));
 ICOMMAND(clearchanges, "", (), clearchanges(CHANGE_GFX|CHANGE_SOUND));
 COMMAND(applychanges, "");
 
@@ -1865,4 +2238,33 @@ ICOMMAND(loopchanges, "ss", (char *var, char *body),
     loopv(needsapply) { alias(var, needsapply[i].desc); execute(body); }
 });
 
+HVARP(fullconcolor, 0, 0x4F4F4F, 0xFFFFFF);
+FVARP(fullconblend, 0, .8, 1);
+
+void consolebox(int x1, int y1, int x2, int y2)
+{
+    glPushMatrix();
+    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
+    glDisable(GL_TEXTURE_2D);
+    notextureshader->set();
+
+    glTranslatef(x1, y1, 0);
+    float r = ((fullconcolor >> 16) & 0xFF) / 255.f,
+        g = ((fullconcolor >> 8) & 0xFF) / 255.f,
+        b = (fullconcolor & 0xFF) / 255.f;
+    glColor4f(r, g, b, fullconblend);
+    glBegin(GL_TRIANGLE_STRIP);
+
+    glVertex2i(x1, y1);
+    glVertex2i(x2, y1);
+    glVertex2i(x1, y2);
+    glVertex2i(x2, y2);
+
+    glEnd();
+    glEnable(GL_TEXTURE_2D);
+    defaultshader->set();
+
+    glPopMatrix();
+}
+
 #endif
