@@ -18,12 +18,12 @@ bool getentboundingbox(extentity &e, ivec &o, ivec &r)
             return false;
         case ET_MAPMODEL:
         {
-            model *m = loadmodel(NULL, e.attr2);
+            model *m = loadmodel(NULL, e.attr[1]);
             if(m)
             {
                 vec center, radius;
                 m->boundbox(0, center, radius);
-                rotatebb(center, radius, e.attr1);
+                rotatebb(center, radius, e.attr[0]);
 
                 o = e.o;
                 o.add(center);
@@ -65,16 +65,12 @@ void modifyoctaentity(int flags, int id, cube *c, const ivec &cor, int size, con
             switch(entities::getents()[id]->type)
             {
                 case ET_MAPMODEL:
-                    if(loadmodel(NULL, entities::getents()[id]->attr2))
+                    if(loadmodel(NULL, entities::getents()[id]->attr[1]))
                     {
                         if(va)
                         {
                             va->bbmin.x = -1;
-                            if(oe.mapmodels.empty())
-                            {
-                                if(!va->mapmodels) va->mapmodels = new vector<octaentities *>;
-                                va->mapmodels->add(&oe);
-                            }
+			    if(oe.mapmodels.empty()) va->mapmodels.add(&oe);
                         }
                         oe.mapmodels.add(id);
                         loopk(3)
@@ -97,17 +93,13 @@ void modifyoctaentity(int flags, int id, cube *c, const ivec &cor, int size, con
             switch(entities::getents()[id]->type)
             {
                 case ET_MAPMODEL:
-                    if(loadmodel(NULL, entities::getents()[id]->attr2))
+                    if(loadmodel(NULL, entities::getents()[id]->attr[1]))
                     {
                         oe.mapmodels.removeobj(id);
                         if(va)
                         {
                             va->bbmin.x = -1;
-                            if(va->mapmodels && oe.mapmodels.empty())
-                            {
-                                va->mapmodels->removeobj(&oe);
-                                if(va->mapmodels->empty()) DELETEP(va->mapmodels);
-                            }
+                            if(oe.mapmodels.empty()) va->mapmodels.removeobj(&oe);
                         }
                         oe.bbmin = oe.bbmax = oe.o;
                         oe.bbmin.add(oe.size);
@@ -156,7 +148,7 @@ static bool modifyoctaent(int flags, int id)
     if(!ents.inrange(id)) return false;
     ivec o, r;
     extentity &e = *ents[id];
-    if((e.inoctanode!=0)==flags || !getentboundingbox(e, o, r)) return false;
+    if((flags&MODOE_ADD ? e.inoctanode : !e.inoctanode) || !getentboundingbox(e, o, r)) return false;
 
     if(!insideworld(e.o))
     {
@@ -273,7 +265,7 @@ int entlooplevel = 0;
 int efocus = -1, enthover = -1, entorient = -1, oldhover = -1;
 bool undonext = true;
 
-VAR(entediting, 0, 0, 1);
+VARF(entediting, 0, 0, 1, { if(!entediting) { entcancel(); efocus = enthover = -1; } });
 
 bool noentedit()
 {
@@ -300,7 +292,7 @@ bool haveselent()
 
 void entcancel()
 {
-    entgroup.setsize(0);
+    entgroup.shrink(0);
 }
 
 void entadd(int id)
@@ -319,7 +311,8 @@ undoblock *newundoent()
     loopv(entgroup)
     {
         e->i = entgroup[i];
-        e->e = *entities::getents()[entgroup[i]];
+	e->e.attr.disown(); //points to random values; this causes problems
+	e->e = *entities::getents()[entgroup[i]];
         e++;
     }
     return u;
@@ -405,7 +398,7 @@ void attachentities()
     f; \
     if(oldtype!=e.type) detachentity(e); \
     if(e.type!=ET_EMPTY) { addentity(n); if(oldtype!=e.type) attachentity(e); } \
-    entities::editent(n)); \
+    entities::editent(n, true)); \
 }
 #define addgroup(exp)   { loopv(entities::getents()) entfocus(i, if(exp) entadd(n)); }
 #define setgroup(exp)   { entcancel(); addgroup(exp); }
@@ -477,10 +470,10 @@ void entselectionbox(const entity &e, vec &eo, vec &es)
         eo.y += e.o.y;
         eo.z = e.o.z - entselradius + es.z;
     }
-    else if(e.type == ET_MAPMODEL && (m = loadmodel(NULL, e.attr2)))
+    else if(e.type == ET_MAPMODEL && (m = loadmodel(NULL, e.attr[1])))
     {
         m->collisionbox(0, eo, es);
-        rotatebb(eo, es, e.attr1);
+        rotatebb(eo, es, e.attr[0]);
 #if 0
         if(m->collide)
             eo.z -= player->aboveeye; // wacky but true. see physics collide
@@ -626,18 +619,18 @@ void renderentradius(extentity &e, bool color)
     switch(e.type)
     {
         case ET_LIGHT:
-            if(color) glColor3f(e.attr2/255.0f, e.attr3/255.0f, e.attr4/255.0f);
-            renderentsphere(e, e.attr1);
+            if(color) glColor3f(e.attr[1]/255.0f, e.attr[2]/255.0f, e.attr[3]/255.0f);
+            renderentsphere(e, e.attr[0]);
             break;
 
         case ET_SPOTLIGHT:
             if(e.attached)
             {
-                if(color) glColor3f(e.attached->attr2/255.0f, e.attached->attr3/255.0f, e.attached->attr4/255.0f);
-                float radius = e.attached->attr1;
+                if(color) glColor3f(e.attached->attr[1]/255.0f, e.attached->attr[2]/255.0f, e.attached->attr[3]/255.0f);
+                float radius = e.attached->attr[0];
                 if(!radius) radius = 2*e.o.dist(e.attached->o);
                 vec dir = vec(e.o).sub(e.attached->o).normalize();
-                float angle = max(1, min(90, int(e.attr1)));
+                float angle = max(1, min(90, int(e.attr[0])));
                 renderentattachment(e);
                 renderentcone(*e.attached, dir, radius, angle);
             }
@@ -645,15 +638,15 @@ void renderentradius(extentity &e, bool color)
 
         case ET_SOUND:
             if(color) glColor3f(0, 1, 1);
-            renderentsphere(e, e.attr2);
-	    renderentsphere(e, min(e.attr2, e.attr3));
+            renderentsphere(e, e.attr[1]);
+	    renderentsphere(e, min(e.attr[1], e.attr[2]));
             break;
 
         case ET_ENVMAP:
         {
             extern int envmapradius;
             if(color) glColor3f(0, 1, 1);
-            renderentsphere(e, e.attr1 ? max(0, min(10000, int(e.attr1))) : envmapradius);
+            renderentsphere(e, e.attr[0] ? max(0, min(10000, int(e.attr[0]))) : envmapradius);
             break;
         }
 
@@ -661,9 +654,9 @@ void renderentradius(extentity &e, bool color)
         case ET_PLAYERSTART:
         {
             if(color) glColor3f(0, 1, 1);
+            entities::entradius(e, color);
             vec dir;
-            vecfromyawpitch(e.attr1, 0, 1, 0, dir);
-            if(e.type==ET_MAPMODEL && e.attr3) renderentring(e, checktriggertype(e.attr3, TRIG_COLLIDE) ? 20 : 12);
+            vecfromyawpitch(e.attr[0], 0, 1, 0, dir);
             renderentarrow(e, dir, 4);
             break;
         }
@@ -687,13 +680,11 @@ void renderhelpers()
 
 		if((showentradius == 2 && entities::radiusent(e))|| (showentdir == 2 && entities::dirent(e)))
 		{
-			glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
 			glDepthFunc(GL_GREATER);
 			glColor3f(0.25f, 0.25f, 0.25f);
 			renderentradius(e, false);
 			glDepthFunc(GL_LESS);
 			renderentradius(e, true);
-			glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
 		}
 	}
 }
@@ -730,7 +721,6 @@ void renderentselection(const vec &o, const vec &ray, bool entmoving)
 		}
 		if(showenthelpers>=1 && (entgroup.length() || enthover >= 0))
 		{
-			glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
 			glDepthFunc(GL_GREATER);
 			glColor3f(0.25f, 0.25f, 0.25f);
 			loopv(entgroup)
@@ -748,7 +738,6 @@ void renderentselection(const vec &o, const vec &ray, bool entmoving)
 				renderentradius(e, true);
 			}
 			if(enthover>=0) entfocus(enthover, renderentradius(e, true));
-			glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
 		}
 	}
 }
@@ -799,7 +788,10 @@ void entpush(int *dir)
     else
         groupedit(e.o[d] += float(s*sel.grid));
     if(entitysurf==1)
+    {
         player->o[d] += float(s*sel.grid);
+        player->resetinterp();
+    }
 }
 
 VAR(entautoviewdist, 0, 25, 100);
@@ -833,9 +825,9 @@ void delent()
     entcancel();
 }
 
-int findtype(char *what)
+int findtype(const char *what)
 {
-    for(int i = 0; *entities::entname(i); i++) if(strcmp(what, entities::entname(i))==0) return i;
+    for(int i = 0; *entities::entname(i); i++) if(!strcmp(what, entities::entname(i))) return i;
     conoutf(CON_ERROR, "unknown entity type \"%s\"", what);
     return ET_EMPTY;
 }
@@ -848,12 +840,12 @@ bool dropentity(entity &e, int drop = -1)
     if(drop<0) drop = entdrop;
     if(e.type == ET_MAPMODEL)
     {
-        model *m = loadmodel(NULL, e.attr2);
+        model *m = loadmodel(NULL, e.attr[1]);
         if(m)
         {
             vec center;
             m->boundbox(0, center, radius);
-            rotatebb(center, radius, e.attr1);
+            rotatebb(center, radius, e.attr[0]);
             radius.x += fabs(center.x);
             radius.y += fabs(center.y);
         }
@@ -903,20 +895,14 @@ void attachent()
 
 COMMAND(attachent, "");
 
-extentity *newentity(bool local, const vec &o, int type, int v1, int v2, int v3, int v4, int v5, int v6, int v7, int v8)
+extentity *newentity(bool local, const vec &o, int type, int *attrs)
 {
     extentity &e = *entities::newentity();
     e.o = o;
-    e.attr1 = v1;
-    e.attr2 = v2;
-    e.attr3 = v3;
-    e.attr4 = v4;
-    e.attr5 = v5;
-    e.attr6 = v6;
-    e.attr7 = v7;
-    e.attr8 = v8;
+    loopi(getattrnum(type))
+    	e.attr.add(attrs[i]);
+
     e.type = type;
-    e.reserved = 0;
     e.spawned = false;
     e.inoctanode = false;
     e.light.color = vec(1, 1, 1);
@@ -926,26 +912,20 @@ extentity *newentity(bool local, const vec &o, int type, int v1, int v2, int v3,
         switch(type)
         {
 		case ET_PARTICLES:
-			switch(e.attr1)
+			switch(e.attr[0])
 			{
-				case 4: case 5: case 6: case 7: case 8: case 9: case 10: case 12: case 13: case 14:
-					if(!e.attr4) e.attr4 |= 0x0F0F0F;
-					if(e.attr1 != 5 && e.attr1 != 6) break;
-				case 3: case 11:
-					if(!e.attr3) e.attr3 |= 0x0F0F0F;
+				case 0: case 1: case 2: case 6: case 7: case 9: case 10: case 11: case 12: case 13: case 14: case 15:
+					if(!e.attr[3]) e.attr[3] |= 0x0F0F0F;
+					if(e.attr[0] != 6 && e.attr[0] != 7) break;
+				case 5: case 8:
+					if(!e.attr[2]) e.attr[2] |= 0x0F0F0F;
 					break;
 			}
 			break;
                 case ET_MAPMODEL:
                 case ET_PLAYERSTART:
-		    e.attr8 = e.attr7;
-		    e.attr7 = e.attr6;
-		    e.attr6 = e.attr5;
-                    e.attr5 = e.attr4;
-                    e.attr4 = e.attr3;
-                    e.attr3 = e.attr2;
-                    e.attr2 = e.attr1;
-                    e.attr1 = (int)camera1->yaw;
+			e.attr.pop();
+			e.attr.insert(0, camera1->yaw);
                     break;
         }
         entities::fixentity(e);
@@ -953,10 +933,35 @@ extentity *newentity(bool local, const vec &o, int type, int v1, int v2, int v3,
     return &e;
 }
 
-void newentity(int type, int a1, int a2, int a3, int a4, int a5, int a6, int a7, int a8)
+const int getattrnum(int type)
+{
+	if(type >= ET_GAMESPECIFIC)
+		return entities::numattrs(type);
+
+	static const int num[] =
+	{
+		0, //empty
+		4, //light
+		6, //mapmodel
+		2, //playerstart
+		3, //envmap
+		8, //particles
+		3, //sound
+		1  //spotlight
+	};
+	return type >= 0 && size_t(type) < sizeof(num)/sizeof(num[0]) ? num[type] : 0;
+}
+
+ICOMMAND(getattrnum, "s", (char *s),
+	int type = findtype(s);
+	intret(getattrnum(type));
+)
+
+void newentity(int type, int *attrs)
 {
     if(entities::getents().length() >= MAXENTS) { conoutf("too many entities"); return; }
-    extentity *t = newentity(true, player->o, type, a1, a2, a3, a4, a5, a6, a7, a8);
+    extentity *t = newentity(true, player->o, type, attrs);
+
     dropentity(*t);
     entities::getents().add(t);
     int i = entities::getents().length()-1;
@@ -966,12 +971,20 @@ void newentity(int type, int a1, int a2, int a3, int a4, int a5, int a6, int a7,
     entedit(i, e.type = type);
 }
 
-void newent(char *what, int *a1, int *a2, int *a3, int *a4, int *a5, int *a6, int *a7, int *a8)
+void newent(tagval *args, int num)
 {
-    if(noentedit()) return;
-    int type = findtype(what);
+    if(noentedit() || num == 0) return;
+    int type = findtype(args[0].getstr());
+    const int numattrs = getattrnum(type);
+    int *attrs = new int[numattrs];
+
+    loopi(numattrs)
+    	attrs[i] = (num - 1 >= i) ? args[i + 1].getint() : 0;
+
     if(type != ET_EMPTY)
-        newentity(type, *a1, *a2, *a3, *a4, *a5, *a6, *a7, *a8);
+        newentity(type, attrs);
+
+    delete[] attrs;
 }
 
 int entcopygrid;
@@ -981,7 +994,7 @@ void entcopy()
 {
     if(noentedit()) return;
     entcopygrid = sel.grid;
-    entcopybuf.setsize(0);
+    entcopybuf.shrink(0);
     loopv(entgroup)
         entfocus(entgroup[i], entcopybuf.add(e).o.sub(sel.o.tovec()));
 }
@@ -998,50 +1011,54 @@ void entpaste()
         entity &c = entcopybuf[i];
         vec o(c.o);
         o.mul(m).add(sel.o.tovec());
-        extentity *e = newentity(true, o, ET_EMPTY, c.attr1, c.attr2, c.attr3, c.attr4, c.attr5, c.attr6, c.attr7, c.attr8);
+        extentity *e = newentity(true, o, ET_EMPTY, c.attr.getbuf());
         entities::getents().add(e);
         entadd(++last);
     }
     int j = 0;
-    groupeditundo(e.type = entcopybuf[j++].type;);
+    groupeditundo(e.attr = entcopybuf[j].attr; e.type = entcopybuf[j++].type;);
 }
 
-COMMAND(newent, "siiiiiiii");
+COMMAND(newent, "V");
 COMMAND(delent, "");
 COMMAND(dropent, "");
 COMMAND(entcopy, "");
 COMMAND(entpaste, "");
 
-void entset(char *what, int *a1, int *a2, int *a3, int *a4, int *a5, int *a6, int *a7, int *a8)
+void entset(tagval *args, int num) //char *what, int *a1, int *a2, int *a3, int *a4, int *a5, int *a6, int *a7, int *a8)
 {
-    if(noentedit()) return;
-    int type = findtype(what);
-    groupedit(e.type=type;
-              e.attr1=*a1;
-              e.attr2=*a2;
-              e.attr3=*a3;
-              e.attr4=*a4;
-              e.attr5=*a5;
-	      e.attr6=*a6;
-	      e.attr7=*a7;
-	      e.attr8=*a8;
-    );
+	if(noentedit()) return;
+	int type = num ? findtype(args[0].getstr()) : 0;
+	int numattrs = getattrnum(type);
+	int *attrs = new int[numattrs];
+
+	loopi(numattrs)
+		attrs[i] = (num - 1 >= i) ? args[i + 1].getint() : 0;
+
+	groupedit(e.type=type;
+	          e.attr.setsize(0);
+	          e.attr.put(attrs, numattrs);
+	);
+	delete[] attrs;
 }
 
 void printent(extentity &e, char *buf)
 {
-    switch(e.type)
-    {
-        case ET_PARTICLES:
-            if(printparticles(e, buf)) return;
-            break;
-
-        default:
-            if(e.type >= ET_GAMESPECIFIC && entities::printent(e, buf)) return;
-            break;
-    }
-    defformatstring(ds)("%d %d %d %d %d %d %d %d", e.attr1, e.attr2, e.attr3, e.attr4, e.attr5, e.attr6, e.attr7, e.attr8);
-    concatstring(buf, ds);
+	switch(e.type)
+	{
+		case ET_PARTICLES:
+			if(printparticles(e, buf)) return;
+			break;
+		default:
+			if(e.type >= ET_GAMESPECIFIC && entities::printent(e, buf)) return;
+			break;
+	}
+	static string tmp;
+	loopv(e.attr)
+	{
+		formatstring(tmp)("%s%d", i ? " " : "", e.attr[i]);
+		concatstring(buf, tmp);
+	}
 }
 
 void nearestent()
@@ -1065,8 +1082,8 @@ void nearestent()
 }
 
 ICOMMAND(enthavesel,"",  (), addimplicit(intret(entgroup.length())));
-ICOMMAND(entselect, "s", (char *body), if(!noentedit()) addgroup(e.type != ET_EMPTY && entgroup.find(n)<0 && execute(body)>0));
-ICOMMAND(entloop,   "s", (char *body), if(!noentedit()) addimplicit(groupeditloop(((void)e, execute(body)))));
+ICOMMAND(entselect, "e", (uint *body), if(!noentedit()) addgroup(e.type != ET_EMPTY && entgroup.find(n)<0 && execute(body)>0));
+ICOMMAND(entloop,   "e", (uint *body), if(!noentedit()) addimplicit(groupeditloop(((void)e, execute(body)))));
 ICOMMAND(insel,     "",  (), entfocus(efocus, intret(pointinsel(sel, e.o))));
 ICOMMAND(entget,    "",  (),
 	entfocus(efocus,
@@ -1077,7 +1094,7 @@ ICOMMAND(entget,    "",  (),
 );
 ICOMMAND(entindex,  "",  (), intret(efocus));
 ICOMMAND(entgetinfo,    "",  (), entfocus(efocus,result(entities::entnameinfo(e))));
-COMMAND(entset, "siiiiiiii");
+COMMAND(entset, "V");
 COMMAND(nearestent, "");
 
 int findentity(int type, int index, int attr1, int attr2)
@@ -1086,13 +1103,19 @@ int findentity(int type, int index, int attr1, int attr2)
     for(int i = index; i<ents.length(); i++)
     {
         extentity &e = *ents[i];
-        if(e.type==type && (attr1<0 || e.attr1==attr1) && (attr2<0 || e.attr2==attr2))
-            return i;
+	if(e.type == type &&
+		(attr1 < 0 || e.attr.length() <= 0 || e.attr[0] == attr1 ) &&
+		(attr1 < 0 || e.attr.length() <= 1 || e.attr[1] == attr2 )
+	)
+		return i;
     }
     loopj(min(index, ents.length()))
     {
         extentity &e = *ents[j];
-        if(e.type==type && (attr1<0 || e.attr1==attr1) && (attr2<0 || e.attr2==attr2))
+	if(e.type == type &&
+		(attr1 < 0 || e.attr.length() <= 0 || e.attr[0] == attr1 ) &&
+		(attr1 < 0 || e.attr.length() <= 1 || e.attr[1] == attr2 )
+	)
             return j;
     }
     return -1;
@@ -1116,13 +1139,13 @@ void findplayerspawn(dynent *d, int forceent, int tag)   // place at random spaw
         for(int attempt = pick;;)
         {
             d->o = entities::getents()[attempt]->o;
-            d->yaw = entities::getents()[attempt]->attr1;
+            d->yaw = entities::getents()[attempt]->attr[0];
             if(entinmap(d, true)) break;
             attempt = findentity(ET_PLAYERSTART, attempt+1, -1, tag);
             if(attempt<0 || attempt==pick)
             {
                 d->o = entities::getents()[attempt]->o;
-                d->yaw = entities::getents()[attempt]->attr1;
+                d->yaw = entities::getents()[attempt]->attr[0];
                 entinmap(d);
                 break;
             }
@@ -1138,7 +1161,7 @@ void findplayerspawn(dynent *d, int forceent, int tag)   // place at random spaw
 
 void splitocta(cube *c, int size)
 {
-    if(size <= VVEC_INT_MASK+1) return;
+    if(size <= 0x1000) return;
     loopi(8)
     {
         if(!c[i].children) c[i].children = newcubes(isempty(c[i]) ? F_EMPTY : F_SOLID);
@@ -1154,6 +1177,7 @@ void resetmap()
     resetblendmap();
     resetlightmaps();
     clearpvs();
+    clearslots();
     clearparticles();
     cleardecals();
     clearsleep();
@@ -1164,7 +1188,7 @@ void resetmap()
     setvar("paused", 0, false);
 
     entities::clearents();
-    outsideents.setsizenodelete(0);
+    outsideents.setsize(0);
 }
 
 void startmap(const char *name)
@@ -1185,26 +1209,31 @@ bool emptymap(int scale, bool force, const char *mname, bool usecfg)    // main
     setvar("mapscale", scale<10 ? 10 : (scale>16 ? 16 : scale), true, false);
     setvar("mapsize", 1<<worldscale, true, false);
 
-    texmru.setsize(0);
+    texmru.shrink(0);
     freeocta(worldroot);
     worldroot = newcubes(F_EMPTY);
     loopi(4) solidfaces(worldroot[i]);
 
-    if(worldsize > VVEC_INT_MASK+1) splitocta(worldroot, worldsize>>1);
+    if(worldsize > 0x1000) splitocta(worldroot, worldsize>>1);
 
+    #ifndef NEWGUI
     clearmainmenu();
+    #else
+    UI::clearmainmenu();
+    #endif
 
     if(usecfg)
     {
-        overrideidents = true;
+        identflags |= IDF_OVERRIDDEN;
         execfile("data/default_map_settings.cfg", false);
-        overrideidents = false;
+        identflags &= ~IDF_OVERRIDDEN;
     }
 
     clearlights();
-    allchanged();
+    allchanged(true);
 
     startmap(mname);
+    game::setwindowcaption();
     return true;
 }
 
@@ -1226,7 +1255,7 @@ bool enlargemap(bool force)
     loopi(3) solidfaces(c[i+1]);
     worldroot = c;
 
-    if(worldsize > VVEC_INT_MASK+1) splitocta(worldroot, worldsize>>1);
+    if(worldsize > 0x1000) splitocta(worldroot, worldsize>>1);
 
     enlargeblendmap();
 
@@ -1235,10 +1264,53 @@ bool enlargemap(bool force)
     return true;
 }
 
+static bool isallempty(cube &c)
+{
+    if(!c.children) return isempty(c);
+    loopi(8) if(!isallempty(c.children[i])) return false;
+    return true;
+}
+
+void shrinkmap()
+{
+    extern int nompedit;
+    if(noedit(true) || (nompedit && multiplayer())) return;
+    if(worldsize <= 1<<10) return;
+
+    int octant = -1;
+    loopi(8) if(!isallempty(worldroot[i]))
+    {
+        if(octant >= 0) return;
+        octant = i;
+    }
+    if(octant < 0) return;
+
+    while(outsideents.length()) removeentity(outsideents.pop());
+
+    if(!worldroot[octant].children) subdividecube(worldroot[octant], false, false);
+    cube *root = worldroot[octant].children;
+    worldroot[octant].children = NULL;
+    freeocta(worldroot);
+    worldroot = root;
+    worldscale--;
+    worldsize /= 2;
+
+    ivec offset(octant, 0, 0, 0, worldsize);
+    vector<extentity *> &ents = entities::getents();
+    loopv(ents) ents[i]->o.sub(offset.tovec());
+
+    shrinkblendmap(octant);
+
+    allchanged();
+
+    conoutf("shrunk map to size %d", worldscale);
+}
+
 void newmap(int *i) { bool force = !isconnected() && !haslocalclients(); if(force) game::forceedit(""); if(emptymap(*i, force, NULL)) game::newmap(max(*i, 0)); }
 void mapenlarge() { if(enlargemap(false)) game::newmap(-1); }
 COMMAND(newmap, "i");
 COMMAND(mapenlarge, "");
+COMMAND(shrinkmap, "");
 
 void mapname()
 {
@@ -1247,13 +1319,13 @@ void mapname()
 
 COMMAND(mapname, "");
 
-void mpeditent(int i, const vec &o, int type, int attr1, int attr2, int attr3, int attr4, int attr5, int attr6, int attr7, int attr8, bool local)
+void mpeditent(int i, const vec &o, int type, int *attrs, bool local)
 {
+    if(i < 0 || i >= MAXENTS) return;
     if(entities::getents().length()<=i)
     {
-        if(i >= MAXENTS) return;
         while(entities::getents().length()<i) entities::getents().add(entities::newentity())->type = ET_EMPTY;
-        extentity *e = newentity(local, o, type, attr1, attr2, attr3, attr4, attr5, attr6, attr7, attr8);
+        extentity *e = newentity(local, o, type, attrs);
         entities::getents().add(e);
         addentity(i);
         attachentity(*e);
@@ -1266,154 +1338,20 @@ void mpeditent(int i, const vec &o, int type, int attr1, int attr2, int attr3, i
         if(oldtype!=type) detachentity(e);
         e.type = type;
         e.o = o;
-        e.attr1 = attr1; e.attr2 = attr2; e.attr3 = attr3; e.attr4 = attr4, e.attr5 = attr5, e.attr6 = attr6, e.attr7 = attr7, e.attr8 = attr8;
+	e.attr.setsize(0);
+	e.attr.put(attrs, getattrnum(type));
         addentity(i);
         if(oldtype!=type) attachentity(e);
     }
+    entities::editent(i, local);
 }
 
 int getworldsize() { return worldsize; }
 int getmapversion() { return mapversion; }
 
-int triggertypes[NUMTRIGGERTYPES] =
-{
-    0,
-    TRIG_ONCE,
-    TRIG_RUMBLE,
-    TRIG_TOGGLE,
-    TRIG_TOGGLE | TRIG_RUMBLE,
-    TRIG_MANY,
-    TRIG_MANY | TRIG_RUMBLE,
-    TRIG_MANY | TRIG_TOGGLE,
-    TRIG_MANY | TRIG_TOGGLE | TRIG_RUMBLE,
-    TRIG_COLLIDE | TRIG_TOGGLE | TRIG_RUMBLE,
-    TRIG_COLLIDE | TRIG_TOGGLE | TRIG_AUTO_RESET | TRIG_RUMBLE,
-    TRIG_COLLIDE | TRIG_TOGGLE | TRIG_LOCKED | TRIG_RUMBLE,
-    TRIG_DISAPPEAR,
-    TRIG_DISAPPEAR | TRIG_RUMBLE,
-    TRIG_DISAPPEAR | TRIG_COLLIDE | TRIG_LOCKED,
-    0 /* reserved */
-};
 
-void resettriggers()
-{
-    const vector<extentity *> &ents = entities::getents();
-    loopv(ents)
-    {
-        extentity &e = *ents[i];
-        if(e.type != ET_MAPMODEL || !e.attr3) continue;
-        e.triggerstate = TRIGGER_RESET;
-        e.lasttrigger = 0;
-    }
-}
 
-void unlocktriggers(int tag, int oldstate = TRIGGER_RESET, int newstate = TRIGGERING)
-{
-    const vector<extentity *> &ents = entities::getents();
-    loopv(ents)
-    {
-        extentity &e = *ents[i];
-        if(e.type != ET_MAPMODEL || !e.attr3) continue;
-        if(e.attr4 == tag && e.triggerstate == oldstate && checktriggertype(e.attr3, TRIG_LOCKED))
-        {
-            if(newstate == TRIGGER_RESETTING && checktriggertype(e.attr3, TRIG_COLLIDE) && overlapsdynent(e.o, 20)) continue;
-            e.triggerstate = newstate;
-            e.lasttrigger = lastmillis;
-            if(checktriggertype(e.attr3, TRIG_RUMBLE)) entities::rumble(e);
-        }
-    }
-}
 
-void trigger(int *tag, int *state)
-{
-    if(*state) unlocktriggers(*tag);
-    else unlocktriggers(*tag, TRIGGERED, TRIGGER_RESETTING);
-}
 
-COMMAND(trigger, "ii");
-
-VAR(triggerstate, -1, 0, 1);
-
-void doleveltrigger(int trigger, int state)
-{
-    defformatstring(aliasname)("level_trigger_%d", trigger);
-    if(identexists(aliasname))
-    {
-        triggerstate = state;
-        execute(aliasname);
-    }
-}
-
-void checktriggers()
-{
-    if(player->state != CS_ALIVE) return;
-    vec o = player->feetpos();
-    const vector<extentity *> &ents = entities::getents();
-    loopv(ents)
-    {
-        extentity &e = *ents[i];
-        if(e.type != ET_MAPMODEL || !e.attr3) continue;
-        switch(e.triggerstate)
-        {
-            case TRIGGERING:
-            case TRIGGER_RESETTING:
-                if(lastmillis-e.lasttrigger>=1000)
-                {
-                    if(e.attr4)
-                    {
-                        if(e.triggerstate == TRIGGERING) unlocktriggers(e.attr4);
-                        else unlocktriggers(e.attr4, TRIGGERED, TRIGGER_RESETTING);
-                    }
-                    if(checktriggertype(e.attr3, TRIG_DISAPPEAR)) e.triggerstate = TRIGGER_DISAPPEARED;
-                    else if(e.triggerstate==TRIGGERING && checktriggertype(e.attr3, TRIG_TOGGLE)) e.triggerstate = TRIGGERED;
-                    else e.triggerstate = TRIGGER_RESET;
-                }
-                break;
-            case TRIGGER_RESET:
-                if(e.lasttrigger)
-                {
-                    if(checktriggertype(e.attr3, TRIG_AUTO_RESET|TRIG_MANY|TRIG_LOCKED) && e.o.dist(o)-player->radius>=(checktriggertype(e.attr3, TRIG_COLLIDE) ? (20 +e.attr5): (12 +e.attr5)))
-                        e.lasttrigger = 0;
-                    break;
-                }
-                else if(e.o.dist(o)-player->radius>=(checktriggertype(e.attr3, TRIG_COLLIDE) ? (20 +e.attr5) : (12 +e.attr5))) break;
-                else if(checktriggertype(e.attr3, TRIG_LOCKED))
-                {
-                    if(!e.attr4) break;
-                    doleveltrigger(e.attr4, -1);
-                    e.lasttrigger = lastmillis;
-                    break;
-                }
-                e.triggerstate = TRIGGERING;
-                e.lasttrigger = lastmillis;
-                if(checktriggertype(e.attr3, TRIG_RUMBLE)) entities::rumble(e);
-                entities::trigger(e);
-                if(e.attr4) doleveltrigger(e.attr4, 1);
-                break;
-            case TRIGGERED:
-                if(e.o.dist(o)-player->radius<(checktriggertype(e.attr3, TRIG_COLLIDE) ? (20 +e.attr5) : (12 +e.attr5)))
-                {
-                    if(e.lasttrigger) break;
-                }
-                else if(checktriggertype(e.attr3, TRIG_AUTO_RESET))
-                {
-                    if(lastmillis-e.lasttrigger<6000) break;
-                }
-                else if(checktriggertype(e.attr3, TRIG_MANY))
-                {
-                    e.lasttrigger = 0;
-                    break;
-                }
-                else break;
-                if(checktriggertype(e.attr3, TRIG_COLLIDE) && overlapsdynent(e.o, 20)) break;
-                e.triggerstate = TRIGGER_RESETTING;
-                e.lasttrigger = lastmillis;
-                if(checktriggertype(e.attr3, TRIG_RUMBLE)) entities::rumble(e);
-                entities::trigger(e);
-                if(e.attr4) doleveltrigger(e.attr4, 0);
-                break;
-        }
-    }
-}
 
 
