@@ -758,6 +758,21 @@ struct captureclientmode : clientmode
 
 extern captureclientmode capturemode;
 ICOMMAND(repammo, "", (), capturemode.replenishammo());
+ICOMMAND(insidebases, "", (),
+{
+    vector<char> buf;
+    if(m_capture && player1->state == CS_ALIVE) loopv(capturemode.bases)
+    {
+        if(capturemode.insidebase(capturemode.bases[i], player1->feetpos()))
+        {
+            if(buf.length()) buf.add(' ');
+            defformatstring(basenum)("%d", i);
+            buf.put(basenum, strlen(basenum));
+        }
+    }
+    buf.add('\0');
+    result(buf.getbuf());
+});
 
 #else
     bool notgotbases;
@@ -770,6 +785,32 @@ ICOMMAND(repammo, "", (), capturemode.replenishammo());
         notgotbases = !empty;
     }
 
+    void cleanup()
+    {
+        reset(false);
+    }
+
+    void setup()
+    {
+        reset(false);
+        if(notgotitems || ments.empty()) return;
+        loopv(ments)
+        {
+            entity &e = ments[i];
+            if(e.type != BASE) continue;
+            int ammotype = e.attr[0];
+            addbase(ammotype>=GUN_SG && ammotype<=GUN_PISTOL ? ammotype : min(ammotype, 0), e.o);
+        }
+        notgotbases = false;
+        sendbases();
+        loopv(clients) if(clients[i]->state.state==CS_ALIVE) entergame(clients[i]);
+    }
+
+    void newmap()
+    {
+        reset(true);
+    }
+
     void stealbase(int n, const char *team)
     {
         baseinfo &b = bases[n];
