@@ -314,10 +314,34 @@ namespace game
     }
     COMMAND(kick, "s");
 
+    vector<int> ignores;
+
+    void ignore(int cn)
+    {
+        fpsent *d = getclient(cn);
+        if(!d || d == player1) return;
+        conoutf("ignoring %s", d->name);
+        if(ignores.find(cn) < 0) ignores.add(cn);
+    }
+
+    void unignore(int cn)
+    {
+        if(ignores.find(cn) < 0) return;
+        fpsent *d = getclient(cn);
+        if(d) conoutf("stopped ignoring %s", d->name);
+        ignores.removeobj(cn);
+    }
+
+    bool isignored(int cn) { return ignores.find(cn) >= 0; }
+
+    ICOMMAND(ignore, "s", (char *arg), ignore(parseplayer(arg)));
+    ICOMMAND(unignore, "s", (char *arg), unignore(parseplayer(arg)));
+    ICOMMAND(isignored, "s", (char *arg), intret(isignored(parseplayer(arg)) ? 1 : 0));
+
     void setteam(const char *arg1, const char *arg2)
     {
         int i = parseplayer(arg1);
-        if(i>=0 && i!=player1->clientnum) addmsg(N_SETTEAM, "ris", i, arg2);
+        if(i>=0) addmsg(N_SETTEAM, "ris", i, arg2);
     }
     COMMAND(setteam, "ss");
 
@@ -676,6 +700,7 @@ namespace game
     void gamedisconnect(bool cleanup)
     {
         if(remote) stopfollowing();
+        ignores.setsize(0);
         connected = remote = false;
         player1->clientnum = -1;
         sessionid = 0;
@@ -821,11 +846,11 @@ namespace game
             messagereliable = false;
             messagecn = -1;
         }
-        if(lastmillis-lastping>250)
+        if(totalmillis-lastping>250)
         {
             putint(p, N_PING);
-            putint(p, lastmillis);
-            lastping = lastmillis;
+            putint(p, totalmillis);
+            lastping = totalmillis;
         }
         sendclientpacket(p.finalize(), 1);
     }
@@ -873,11 +898,11 @@ namespace game
             if(fx<fy) d->o.y += dy<0 ? r-fy : -(r-fy);  // push aside
             else      d->o.x += dx<0 ? r-fx : -(r-fx);
         }
-        int lagtime = lastmillis-d->lastupdate;
+        int lagtime = totalmillis-d->lastupdate;
         if(lagtime)
         {
             if(d->state!=CS_SPAWNING && d->lastupdate) d->plag = (d->plag*5+lagtime)/6;
-            d->lastupdate = lastmillis;
+            d->lastupdate = totalmillis;
         }
     }
 
@@ -1070,6 +1095,7 @@ namespace game
                 int tcn = getint(p), mtype = getint(p);
                 getstring(text, p);
                 filtertext(text, text);
+                if(isignored(d->clientnum)) break;
                 fpsent *t = getclient(tcn);
                 if(t->state!=CS_DEAD && t->state!=CS_SPECTATOR)
                 particle_textcopy(t->abovehead(), text, PART_TEXT, 2000, 0x32FF64, 4.0f, -8);
@@ -1146,7 +1172,7 @@ namespace game
                 if(!text[0]) copystring(text, "unnamed");
                 if(d->name[0])          // already connected
                 {
-                    if(strcmp(d->name, text))
+                    if(strcmp(d->name, text) && !isignored(d->clientnum))
                         conoutf("%s is now known as %s", colorname(d), colorname(d, text));
                 }
                 else                    // new client
@@ -1169,7 +1195,7 @@ namespace game
                     if(!text[0]) copystring(text, "unnamed");
                     if(strcmp(text, d->name))
                     {
-                        conoutf("%s is now known as %s", colorname(d), colorname(d, text));
+                        if(!isignored(d->clientnum)) conoutf("%s is now known as %s", colorname(d), colorname(d, text));
                         copystring(d->name, text, MAXNAMELEN+1);
                     }
                 }
@@ -1307,7 +1333,7 @@ namespace game
             {
                 if(!d) return;
                 int gun = getint(p);
-                d->gunselect = max(gun, 0);
+                d->gunselect = clamp(gun, int(GUN_FIST), int(GUN_PISTOL));
                 playsound(S_WEAPLOAD, &d->o);
                 break;
             }
@@ -1453,7 +1479,7 @@ namespace game
             }
 
             case N_PONG:
-                addmsg(N_CLIENTPING, "i", player1->ping = (player1->ping*5+lastmillis-getint(p))/6);
+                addmsg(N_CLIENTPING, "i", player1->ping = (player1->ping*5+totalmillis-getint(p))/6);
                 break;
 
             case N_CLIENTPING:
@@ -1789,6 +1815,7 @@ namespace game
 
     void parsepacketclient(int chan, packetbuf &p)   // processes any updates from the server
     {
+        if(p.packet->flags&ENET_PACKET_FLAG_UNSEQUENCED) return;
         switch(chan)
         {
             case 0:
@@ -1865,8 +1892,8 @@ namespace game
         stream *cfg = openrawfile(cname, "rb");
         if(map && cfg)
         {
-            int len = map->size();
-            if(len > 1024*1024) conoutf(CON_ERROR, "map is too large: %d", len);
+            stream::offset len = map->size();
+            if(len > 4*1024*1024) conoutf(CON_ERROR, "map is too large: %d", len);
             else if(len <= 0) conoutf(CON_ERROR, "could not read map");
             else
             {
