@@ -25,6 +25,16 @@ namespace entities
                 e.attr[0] = (int(e.attr[0])+180)%360;
                 break;
         }
+        if(ver <= 31) switch(e.type)
+        {
+            case BOX:
+            case BARREL:
+            case PLATFORM:
+            case ELEVATOR:
+                int yaw = (int(e.attr[0])%360 + 360)%360 + 7;
+                e.attr[0] = yaw - yaw%15;
+                break;
+        }
     }
 
 #ifndef STANDALONE
@@ -765,7 +775,7 @@ namespace entities
 			pos.z += 4.5;
 			formatstring(tmp)("Yaw: %i\nMonster: %s (%i)\nTag: monster_dead_%i",
 				e.attr[0],
-				(e.attr[1] >= 0 && e.attr[1] < game::NUMMONSTERTYPES) ? game::monstertypes[e.attr[1]].name : "(null)", e.attr[1],
+				((e.attr[1] >= 0 && e.attr[1] < game::NUMMONSTERTYPES) ? game::monstertypes[e.attr[1]].name : "(null)"), e.attr[1],
 				e.attr[2]
 			);
 			return;
