@@ -249,7 +249,7 @@ namespace game
             }
             else if(d->state==CS_DEAD && d->ragdoll) moveragdoll(d);
 
-            const int lagtime = lastmillis-d->lastupdate;
+            const int lagtime = totalmillis-d->lastupdate;
             if(!lagtime || intermission) continue;
             else if(lagtime>1000 && d->state==CS_ALIVE)
             {
@@ -530,6 +530,7 @@ namespace game
             if(followdir) nextfollow(followdir);
             else stopfollowing();
         }
+        unignore(cn);
         fpsent *d = clients[cn];
         if(!d) return;
         if(notify && d->name[0]) conoutf("player %s disconnected", colorname(d));
@@ -980,7 +981,7 @@ namespace game
     bool serverinfostartcolumn(g3d_gui *g, int i)
     {
         static const char *names[] = { "ping ", "players ", "map ", "mode ", "master ", "host ", "port ", "description " };
-        static const int struts[] =  { 0,       0,          12,     12,      8,         13,      6,       24 };
+        static const int struts[] =  { 7,       7,          14,     13,      8,         14,      7,       24 };
         if(size_t(i) >= sizeof(names)/sizeof(names[0])) return false;
         g->pushlist();
         g->text(names[i], 0xFFFF80, !i ? " " : NULL);
