@@ -1,6 +1,8 @@
 // monster.h: implements AI for single player monsters, currently client only
 #include "game.h"
 
+extern int physsteps;
+
 namespace game
 {
     static vector<int> teleports;
@@ -10,12 +12,12 @@ namespace game
 
     const monstertype monstertypes[NUMMONSTERTYPES] =
     {
-        { GUN_FIST,  4, 100, 3, 0,   100, 800, 50, 0,  5, S_PAINO, S_DIE1,   "A Butterfly",     "butterfly",       "butterfly"}, 
-        { GUN_BITE,  20, 800, 5, 0,   400, 1600, 100, 60,  4000, S_PAINO, S_DIE1,   "A Dragon",     "rpg/characters/dragon",       "Dragon"}, 
-        { GUN_FIST,  8, 800, 5, 0,   400, 1600, 100, 16,  4000, S_PAINO, S_DIE1,   "A Golem",     "rpg/characters/golem",       "golem"}, 
-        { GUN_FIST,  12, 200, 5, 0,   400, 1600, 100, 16,  800, S_PAINO, S_DIE1,   "A Grizzly",     "rpg/characters/grizzly",       "grizzly"}, 
-        { GUN_BITE,  25, 50, 5, 0,   400, 1600, 100, 2,  30, S_PAINO, S_DIE1,   "A Rat",     "rpg/characters/rat",       "rat"}, 
-        { GUN_FIST,  7, 150, 5, 0,   400, 1600, 100, 16,  200, S_PAINO, S_DIE1,   "A Snagon",     "rpg/characters/snagon",       "snagon"}, 
+        { GUN_FIST,  4, 100, 3, 0,   100, 800, 50, 0,  5, S_PAINO, S_DIE1,   "A Butterfly",     "butterfly",       "butterfly"},
+        { GUN_BITE,  20, 800, 5, 0,   400, 1600, 100, 60,  4000, S_PAINO, S_DIE1,   "A Dragon",     "rpg/characters/dragon",       "Dragon"},
+        { GUN_FIST,  8, 800, 5, 0,   400, 1600, 100, 16,  4000, S_PAINO, S_DIE1,   "A Golem",     "rpg/characters/golem",       "golem"},
+        { GUN_FIST,  12, 200, 5, 0,   400, 1600, 100, 16,  800, S_PAINO, S_DIE1,   "A Grizzly",     "rpg/characters/grizzly",       "grizzly"},
+        { GUN_BITE,  25, 50, 5, 0,   400, 1600, 100, 2,  30, S_PAINO, S_DIE1,   "A Rat",     "rpg/characters/rat",       "rat"},
+        { GUN_FIST,  7, 150, 5, 0,   400, 1600, 100, 16,  200, S_PAINO, S_DIE1,   "A Snagon",     "rpg/characters/snagon",       "snagon"},
         { GUN_BITE,  28, 80, 5, 0,   400, 1600, 100, 6,  150, S_PAINO, S_DIE1,   "A Wolf",     "rpg/characters/wolf",       "wolf"},
     };
 
@@ -24,19 +26,24 @@ namespace game
 
     bool monsterhurt;
     vec monsterhurtpos;
-    
+
     struct monster : fpsent
     {
         int monsterstate;                   // one of M_*, M_NONE means human
-    
+
         int mtype, tag;                     // see monstertypes table
         fpsent *enemy;                      // monster wants to kill this entity
         float targetyaw;                    // monster wants to look in this direction
         int trigger;                        // millis at which transition to another monsterstate takes place
         vec attacktarget;                   // delayed attacks
         int anger;                          // how many times already hit by fellow monster
-    
-        monster(int _type, int _yaw, int _tag, int _state, int _trigger, int _move) : monsterstate(_state), tag(_tag)
+        physent *stacked;
+        vec stackpos;
+
+        monster(int _type, int _yaw, int _tag, int _state, int _trigger, int _move) :
+            monsterstate(_state), tag(_tag),
+            stacked(NULL),
+            stackpos(0, 0, 0)
         {
             type = ENT_AI;
             respawn();
@@ -70,7 +77,13 @@ namespace game
             anger = 0;
             copystring(name, t.name);
         }
-        
+
+        void normalize_yaw(float angle)
+        {
+            while(yaw<angle-180.0f) yaw += 360.0f;
+            while(yaw>angle+180.0f) yaw -= 360.0f;
+        }
+
         // monster AI is sequenced using transitions: they are in a particular state where
         // they execute a particular behaviour until the trigger time is hit, and then they
         // reevaluate their situation based on the current state, the environment etc., and
@@ -100,7 +113,7 @@ namespace game
                 if(targetyaw>yaw) yaw = targetyaw;
             }
             float dist = enemy->o.dist(o);
-            if(monsterstate!=M_SLEEP) pitch = asin((enemy->o.z - o.z) / dist) / RAD; 
+            if(monsterstate!=M_SLEEP) pitch = asin((enemy->o.z - o.z) / dist) / RAD;
 
             if(blocked)                                                              // special case: if we run into scenery
             {
@@ -115,9 +128,9 @@ namespace game
                     transition(M_SEARCH, 1, 100, 1000);
                 }
             }
-            
-            float enemyyaw = -(float)atan2(enemy->o.x - o.x, enemy->o.y - o.y)/RAD+180;
-            
+
+            float enemyyaw = -atan2(enemy->o.x - o.x, enemy->o.y - o.y)/RAD;
+
             switch(monsterstate)
             {
                 case M_PAIN:
@@ -125,10 +138,10 @@ namespace game
                 case M_SEARCH:
                     if(trigger<lastmillis) transition(M_HOME, 1, 100, 200);
                     break;
-                    
+
                 case M_SLEEP:                       // state classic sp monster start in, wait for visual contact
                 {
-                    if(editmode) break;          
+                    if(editmode) break;
                     normalize_yaw(enemyyaw);
                     float angle = (float)fabs(enemyyaw-yaw);
                     if(dist<32                   // the better the angle to the player, the further the monster can see/hear
@@ -147,7 +160,7 @@ namespace game
                     }
                     break;
                 }
-                
+
                 case M_AIMING:                      // this state is the delay between wanting to shoot and actually firing
                     if(trigger<lastmillis)
                     {
@@ -167,7 +180,7 @@ namespace game
                         {
                             transition(M_HOME, 1, 800, 500);
                         }
-                        else 
+                        else
                         {
                             bool melee = false, longrange = false;
                             switch(monstertypes[mtype].gun)
@@ -176,9 +189,9 @@ namespace game
                                 case GUN_FIST: melee = true; break;
                                 case GUN_RIFLE: longrange = true; break;
                             }
-                            // the closer the monster is the more likely he wants to shoot, 
+                            // the closer the monster is the more likely he wants to shoot,
                             if((!melee || dist<20) && !rnd(longrange ? (int)dist/12+1 : min((int)dist/12+1,6)) && enemy->state==CS_ALIVE)      // get ready to fire
-                            { 
+                            {
                                 attacktarget = target;
                                 transition(M_AIMING, 0, monstertypes[mtype].lag, 10);
                             }
@@ -189,10 +202,10 @@ namespace game
                         }
                     }
                     break;
-                    
+
             }
 
-            if(move || moving || (onplayer && (onplayer->state!=CS_ALIVE || lastmoveattempt <= onplayer->lastmove)))
+            if(move || maymove() || (stacked && (stacked->state!=CS_ALIVE || stackpos != stacked->o)))
             {
                 vec pos = feetpos();
                 loopv(teleports) // equivalent of player entity touch, but only teleports are used
@@ -202,6 +215,7 @@ namespace game
                     if(dist<16) entities::teleport(teleports[i], this);
                 }
 
+                if(physsteps > 0) stacked = NULL;
                 moveplayer(this, 1, true);        // use physics to move monster
             }
         }
@@ -245,6 +259,12 @@ namespace game
         }
     };
 
+    void stackmonster(monster *d, physent *o)
+    {
+        d->stacked = o;
+        d->stackpos = o->o;
+    }
+
     int nummonsters(int tag, int state)
     {
         int n = 0;
@@ -259,9 +279,9 @@ namespace game
     }
 
     vector<monster *> monsters;
-    
+
     int nextmonster, spawnremain, numkilled, monstertotal, mtimestart, remain;
-    
+
     void spawnmonster()     // spawn a random monster according to freq distribution in DMSP
     {
         int n = rnd(TOTMFREQ), type;
@@ -273,9 +293,9 @@ namespace game
     {
         removetrackedparticles();
         removetrackeddynlights();
-        loopv(monsters) delete monsters[i]; 
+        loopv(monsters) delete monsters[i];
         cleardynentcache();
-        monsters.setsize(0);
+        monsters.shrink(0);
         numkilled = 0;
         monstertotal = 0;
         spawnremain = 0;
@@ -293,7 +313,7 @@ namespace game
             {
                 extentity &e = *entities::ents[i];
                 if(e.type!=MONSTER) continue;
-                monster *m = new monster(e.attr2, e.attr1, e.attr3, M_SLEEP, 100, 0);  
+                monster *m = new monster(e.attr[1], e.attr[0], e.attr[2], M_SLEEP, 100, 0);
                 monsters.add(m);
                 m->o = e.o;
                 entinmap(m);
@@ -301,7 +321,7 @@ namespace game
                 monstertotal++;
             }
         }
-        teleports.setsizenodelete(0);
+        teleports.setsize(0);
         if(m_dmsp || m_classicsp)
         {
             loopv(entities::ents) if(entities::ents[i]->type==TELEPORT) teleports.add(i);
@@ -312,11 +332,11 @@ namespace game
     {
         conoutf(CON_GAMEINFO, allkilled ? "\f2you have cleared the map!" : "\f2you reached the exit!");
         monstertotal = 0;
-        game::addmsg(SV_FORCEINTERMISSION, "r");
+        game::addmsg(N_FORCEINTERMISSION, "r");
     }
     ICOMMAND(endsp, "", (), endsp(false));
 
-    
+
     void monsterkilled()
     {
         numkilled++;
@@ -333,11 +353,11 @@ namespace game
             nextmonster = lastmillis+1000;
             spawnmonster();
         }
-        
+
         if(killsendsp && monstertotal && !spawnremain && numkilled==monstertotal) endsp(true);
-        
+
         bool monsterwashurt = monsterhurt;
-        
+
         loopv(monsters)
         {
             if(monsters[i]->state==CS_ALIVE) monsters[i]->monsteraction(curtime);
@@ -351,7 +371,7 @@ namespace game
                 }
             }
         }
-        
+
         if(monsterwashurt) monsterhurt = false;
     }
 
@@ -360,7 +380,7 @@ namespace game
         loopv(monsters)
         {
             monster &m = *monsters[i];
-            if(m.state!=CS_DEAD || lastmillis-m.lastpain<10000)//m.superdamage<50) 
+            if(m.state!=CS_DEAD || lastmillis-m.lastpain<10000)
             {
                 modelattach vwep[2];
                 vwep[0] = modelattach("tag_weapon", monstertypes[m.mtype].vwepname, ANIM_VWEP_IDLE|ANIM_LOOP, 0);
@@ -385,14 +405,14 @@ namespace game
     {
         conoutf(CON_GAMEINFO, "\f2--- single player time score: ---");
         int pen, score = 0;
-        pen = (totalmillis-maprealtime)/1000; score += pen; if(pen) conoutf(CON_GAMEINFO, "\f2time taken: %d seconds (%d simulated seconds)", pen, (lastmillis-maptime)/1000);
+        pen = ((lastmillis-maptime)*100)/(1000*getvar("gamespeed")); score += pen; if(pen) conoutf(CON_GAMEINFO, "\f2time taken: %d seconds (%d simulated seconds)", pen, (lastmillis-maptime)/1000);
         pen = player1->deaths*60; score += pen; if(pen) conoutf(CON_GAMEINFO, "\f2time penalty for %d deaths (1 minute each): %d seconds", player1->deaths, pen);
         pen = remain*10;          score += pen; if(pen) conoutf(CON_GAMEINFO, "\f2time penalty for %d monsters remaining (10 seconds each): %d seconds", remain, pen);
         pen = (10-skill)*20;      score += pen; if(pen) conoutf(CON_GAMEINFO, "\f2time penalty for lower skill level (20 seconds each): %d seconds", pen);
         pen = 100-accuracy;       score += pen; if(pen) conoutf(CON_GAMEINFO, "\f2time penalty for missed shots (1 second each %%): %d seconds", pen);
         defformatstring(aname)("bestscore_%s", getclientmap());
         const char *bestsc = getalias(aname);
-        int bestscore = *bestsc ? atoi(bestsc) : score;
+        int bestscore = *bestsc ? parseint(bestsc) : score;
         if(score<bestscore) bestscore = score;
         defformatstring(nscore)("%d", bestscore);
         alias(aname, nscore);
