@@ -36,7 +36,7 @@ namespace game
         {
             state = CS_ALIVE;
             type = ENT_INANIMATE;
-            yaw = float((e.attr[0]+7)-(e.attr[0]+7)%15);
+            yaw = e.attr[0];
             if(e.type==PLATFORM || e.type==ELEVATOR)
             {
                 maxspeed = e.attr[3] ? fabs(float(e.attr[3])) : PLATFORMSPEED;
