@@ -10,7 +10,6 @@ namespace game
     VARP(ragdollfade, 0, 1000, 300000);
     VARFP(playermodel, 0, 0, 7, { if(player1->clientnum < 0) player1->playermodel = playermodel; });
     VARP(forceplayermodels, 0, 0, 1);
-    VARP(allplayermodels, 0, 0, 1);
 
     vector<fpsent *> ragdolls;
 
@@ -48,23 +47,19 @@ namespace game
 
      static const playermodelinfo playermodels[8] =
     {
-        { "rc", "rc/blue", "rc/red", NULL, NULL, NULL, { NULL, NULL, NULL }, "rc", "rc_blue", "rc_red", true, true },
-        { "rc/blue", "rc/blue", "rc/red", NULL, NULL, NULL, { NULL, NULL, NULL }, "rc_blue", "rc_blue", "rc_red", true, true },
-        { "rc/red", "rc/blue", "rc/red", NULL, NULL, NULL, { NULL, NULL, NULL }, "rc_red", "rc_blue", "rc_red", true, true },
-        { "rc/pink", "rc/blue", "rc/red", NULL, NULL, NULL, { NULL, NULL, NULL }, "rc_pink", "rc_blue", "rc_red", true, true },
-        { "ogre", "ogre/blue", "ogre/red", NULL, NULL, NULL, { NULL, NULL, NULL }, "ogre", "ogre", "ogre", false, true },
-        { "ogre/blue", "ogre/blue", "ogre/red", NULL, NULL, NULL, { NULL, NULL, NULL }, "ogre", "ogre", "ogre", false, true },
-        { "ogre/red", "ogre/blue", "ogre/red", NULL, NULL, NULL, { NULL, NULL, NULL }, "ogre", "ogre", "ogre", false, true },
-        { "uh/chars/man", "uh/chars/man", "uh/chars/man", NULL, NULL, NULL, { NULL, NULL, NULL }, "uh/chars/man", "uh/chars/man", "uh/chars/man", false, true }
+        { "rc", "rc/blue", "rc/red", NULL, NULL, NULL, { NULL, NULL, NULL }, "rc", "rc_blue", "rc_red", true },
+        { "rc/blue", "rc/blue", "rc/red", NULL, NULL, NULL, { NULL, NULL, NULL }, "rc_blue", "rc_blue", "rc_red", true },
+        { "rc/red", "rc/blue", "rc/red", NULL, NULL, NULL, { NULL, NULL, NULL }, "rc_red", "rc_blue", "rc_red", true },
+        { "rc/pink", "rc/blue", "rc/red", NULL, NULL, NULL, { NULL, NULL, NULL }, "rc_pink", "rc_blue", "rc_red", true },
+        { "ogre", "ogre/blue", "ogre/red", NULL, NULL, NULL, { NULL, NULL, NULL }, "ogre", "ogre", "ogre", false },
+        { "ogre/blue", "ogre/blue", "ogre/red", NULL, NULL, NULL, { NULL, NULL, NULL }, "ogre", "ogre", "ogre", false },
+        { "ogre/red", "ogre/blue", "ogre/red", NULL, NULL, NULL, { NULL, NULL, NULL }, "ogre", "ogre", "ogre", false },
+        { "uh/chars/man", "uh/chars/man", "uh/chars/man", NULL, NULL, NULL, { NULL, NULL, NULL }, "uh/chars/man", "uh/chars/man", "uh/chars/man", false }
     };
 
     int chooserandomplayermodel(int seed)
     {
-        static int choices[sizeof(playermodels)/sizeof(playermodels[0])];
-        int numchoices = 0;
-        loopi(sizeof(playermodels)/sizeof(playermodels[0])) if(i == playermodel || playermodels[i].selectable || allplayermodels) choices[numchoices++] = i;
-        if(numchoices <= 0) return -1;
-        return choices[(seed&0xFFFF)%numchoices];
+        return (seed&0xFFFF)%(sizeof(playermodels)/sizeof(playermodels[0]));
     }
 
     const playermodelinfo *getplayermodelinfo(int n)
@@ -76,7 +71,7 @@ namespace game
     const playermodelinfo &getplayermodelinfo(fpsent *d)
     {
         const playermodelinfo *mdl = getplayermodelinfo(d==player1 || forceplayermodels ? playermodel : d->playermodel);
-        if(!mdl || (!mdl->selectable && !allplayermodels)) mdl = getplayermodelinfo(playermodel);
+        if(!mdl) mdl = getplayermodelinfo(playermodel);
         return *mdl;
     }
 
@@ -91,7 +86,7 @@ namespace game
             if(!forceplayermodels)
             {
                 const playermodelinfo *mdl = getplayermodelinfo(d->playermodel);
-                if(mdl && (mdl->selectable || allplayermodels)) continue;
+                if(mdl) continue;
             }
             cleanragdoll(d);
         }
@@ -102,7 +97,7 @@ namespace game
             if(!forceplayermodels)
             {
                 const playermodelinfo *mdl = getplayermodelinfo(d->playermodel);
-                if(mdl && (mdl->selectable || allplayermodels)) continue;
+                if(mdl) continue;
             }
             cleanragdoll(d);
         }
@@ -114,7 +109,7 @@ namespace game
         {
             const playermodelinfo *mdl = getplayermodelinfo(i);
             if(!mdl) break;
-            if(i != playermodel && (!multiplayer(false) || forceplayermodels || (!mdl->selectable && !allplayermodels))) continue;
+            if(i != playermodel && (!multiplayer(false) || forceplayermodels)) continue;
             if(m_teammode)
             {
                 preloadmodel(mdl->blueteam);
@@ -281,7 +276,7 @@ namespace game
 
     FVAR(swaystep, 1, 35.0f, 100);
     FVAR(swayside, 0, 0.04f, 1);
-    FVAR(swayup, 0, 0.05f, 1);
+    FVAR(swayup, -1, 0.05f, 1);
 
     float swayfade = 0, swayspeed = 0, swaydist = 0;
     vec swaydir(0, 0, 0);
