@@ -16,11 +16,10 @@
 //-------------------------------------------------------
 GenericAction::GenericAction() {}
 GenericAction::~GenericAction() {}
-void GenericAction::finished(Control& target)
+void GenericAction::finished(ControlCharacter& target)
 {
 //	conoutf("DEBUG: GenericAction::finished");
-    ActionInstance& in = dynamic_cast<ActionInstance&>(target);
-    in.event(ACTIONINSTANCE_EVENT_STEP_FINISH);
+    target.event(ACTIONINSTANCE_EVENT_STEP_FINISH);
 }
 
 ActionSpawn::ActionSpawn(int uid)
@@ -34,7 +33,7 @@ ActionSpawn::ActionSpawn(int uid)
 
 ActionSpawn::~ActionSpawn() {}
 
-void ActionSpawn::process(Control& target)
+void ActionSpawn::process(ControlCharacter& target)
 {
     using namespace MapEntities;
 
@@ -42,15 +41,15 @@ void ActionSpawn::process(Control& target)
     vec spawn = vec(e->o);
     droptofloor(spawn, 0, 0);
     target.dynent->o = spawn;
+    setbbfrommodel(target.dynent, game::playermodels[target.dynent->charinfo.playermodel]->name); // Protagoras: bounding box support
     target.dynent->o.z += target.dynent->eyeheight;
     target.dynent->yaw = e->attr[0];
 
-    ControlCharacter& ca = dynamic_cast<ControlCharacter&>(target);
     if(!editmode)
     {
         game::addmsg(SV_TRYSPAWN, "rc", target.dynent);
     }
-    ca.event(ACTIONINSTANCE_EVENT_SPAWN);
+    target.event(ACTIONINSTANCE_EVENT_SPAWN);
     finished(target);
 }
 
@@ -58,7 +57,7 @@ ActionWait::ActionWait(int _millis) : millis(_millis) {}
 
 ActionWait::~ActionWait() {}
 
-void ActionWait::process(Control& target)
+void ActionWait::process(ControlCharacter& target)
 {
     if ( (lastmillis - target.dynent->lastaction ) > millis )
     {
@@ -79,7 +78,7 @@ ActionMove::ActionMove(int uid)
 
 ActionMove::~ActionMove() {}
 
-void ActionMove::process(Control& target)
+void ActionMove::process(ControlCharacter& target)
 {
     using namespace MapEntities;
 
@@ -105,7 +104,7 @@ void ActionMove::process(Control& target)
 	while (dy <= -180.0f) dy += 360.0f; // Normalizing
 	while (dy > 180.0f) dy -= 360.0f;
 	dp -= target.dynent->pitch;
-	dc = (lastmillis - dynamic_cast<ActionInstance&>(target).lastprocess) * target.dynent->maxspeed / (5000.0f * RAD);
+	dc = (lastmillis - target.lastprocess) * target.dynent->maxspeed / (5000.0f * RAD);
 	dy = (dy < -dc) ? -dc : dy; // Clamping
 	dy = (dy > dc) ? dc : dy;
 	dp = (dp < -dc) ? -dc : dp;
@@ -121,7 +120,7 @@ ActionJump::ActionJump() {}
 
 ActionJump::~ActionJump() {}
 
-void ActionJump::process(Control& target)
+void ActionJump::process(ControlCharacter& target)
 {
 	target.dynent->jumping = true;
 	moveplayer(target.dynent, 1, false);
@@ -133,7 +132,7 @@ ActionFace::ActionFace(int _cuid) : cuid(_cuid) {}
 
 ActionFace::~ActionFace() {}
 
-void ActionFace::process(Control& target)
+void ActionFace::process(ControlCharacter& target)
 {
 	if (game::controls[cuid] == &target || cuid >= game::controls.length())
 	{
@@ -154,7 +153,7 @@ void ActionFace::process(Control& target)
 		finished(target);
 		return;
 	}
-	dc = (lastmillis - dynamic_cast<ActionInstance&>(target).lastprocess) * target.dynent->maxspeed / (5000.0f * RAD);
+	dc = (lastmillis - target.lastprocess) * target.dynent->maxspeed / (5000.0f * RAD);
 	dy = (dy < -dc) ? -dc : dy;
 	dy = (dy > dc) ? dc : dy;
 	dp = (dp < -dc) ? -dc : dp;
@@ -169,7 +168,7 @@ ActionExpect::ActionExpect(int _cuid, int _radius) : cuid(_cuid), radius(_radius
 
 ActionExpect::~ActionExpect() {}
 
-void ActionExpect::process(Control& target)
+void ActionExpect::process(ControlCharacter& target)
 {
 	if (game::controls[cuid] == &target || cuid >= game::controls.length())
 	{
@@ -198,7 +197,7 @@ ActionFollow::ActionFollow(int _cuid, int uid) : cuid(_cuid)
 
 ActionFollow::~ActionFollow() {}
 
-void ActionFollow::process(Control& target)
+void ActionFollow::process(ControlCharacter& target)
 {
 	using namespace MapEntities;
 
@@ -227,12 +226,12 @@ void ActionFollow::process(Control& target)
 	dir.normalize();
 	float dy, dp, dc;
 	vectoyawpitch(dir, dy, dp);
-	if (!(target.dynent->inwater)) dp = 0.0f;
+//	if (!(target.dynent->inwater)) dp = 0.0f; // Protagoras: hopefully not needed
 	dy -= target.dynent->yaw;
 	while (dy <= -180.0f) dy += 360.0f;
 	while (dy > 180.0f) dy -= 360.0f;
 	dp -= target.dynent->pitch;
-	dc = (lastmillis - dynamic_cast<ActionInstance&>(target).lastprocess) * target.dynent->maxspeed / (5000.0f * RAD);
+	dc = (lastmillis - target.lastprocess) * target.dynent->maxspeed / (5000.0f * RAD);
 	dy = (dy < -dc) ? -dc : dy;
 	dy = (dy > dc) ? dc : dy;
 	dp = (dp < -dc) ? -dc : dp;
@@ -251,7 +250,7 @@ ActionSpeed::ActionSpeed(int _speed)
 
 ActionSpeed::~ActionSpeed() {}
 
-void ActionSpeed::process(Control& target)
+void ActionSpeed::process(ControlCharacter& target)
 {
 	target.dynent->maxspeed = speed;
 	finished(target);
@@ -261,7 +260,7 @@ ActionDestroy::ActionDestroy() {}
 
 ActionDestroy::~ActionDestroy() {}
 
-void ActionDestroy::process(Control& target)
+void ActionDestroy::process(ControlCharacter& target)
 {
     target.disconnect();
 	finished(target);
@@ -271,10 +270,15 @@ ActionModel::ActionModel(int _model) : model(_model) {}
 
 ActionModel::~ActionModel() {}
 
-void ActionModel::process(Control& target)
+void ActionModel::process(ControlCharacter& target)
 {
 	CharacterInfo& info = target.dynent->getcharinfo();
 	info.setplayermodel(model);
+	// Protagoras: bounding box support
+	target.dynent->o.z -= target.dynent->eyeheight;
+	setbbfrommodel(target.dynent, game::playermodels[info.playermodel]->name);
+	target.dynent->o.z += target.dynent->eyeheight;
+	target.dynent->resetinterp();
 	game::addmsg(SV_SWITCHMODEL, "rci", target.dynent, info.getplayermodel());
 	finished(target);
 }
@@ -283,7 +287,7 @@ ActionSetAO::ActionSetAO(int _ao, int _anim) : ao(_ao), anim(_anim) {}
 
 ActionSetAO::~ActionSetAO() {}
 
-void ActionSetAO::process(Control& target)
+void ActionSetAO::process(ControlCharacter& target)
 {
 	CharacterInfo& info = target.dynent->getcharinfo();
     info.setao(ao, anim);
@@ -294,7 +298,7 @@ ActionSetGesture::ActionSetGesture(int _anim) : anim(_anim) {}
 
 ActionSetGesture::~ActionSetGesture() {}
 
-void ActionSetGesture::process(Control& target)
+void ActionSetGesture::process(ControlCharacter& target)
 {
 	CharacterInfo& info = target.dynent->getcharinfo();
     info.setactivegesture(anim);
@@ -305,7 +309,7 @@ ActionDoGesture::ActionDoGesture(int _millis) : millis(_millis) {}
 
 ActionDoGesture::~ActionDoGesture() {}
 
-void ActionDoGesture::process(Control& target)
+void ActionDoGesture::process(ControlCharacter& target)
 {
 	CharacterInfo& info = target.dynent->getcharinfo();
 	if ( (lastmillis - target.dynent->lastaction ) > millis ) //Protagoras: removed dynamic cast
@@ -330,7 +334,7 @@ ActionAttach::~ActionAttach()
 	delete[] model;
 }
 
-void ActionAttach::process(Control& target)
+void ActionAttach::process(ControlCharacter& target)
 {
    	CharacterInfo& info = target.dynent->getcharinfo();
   	info.attached.add(part,model,rule);
@@ -342,7 +346,7 @@ ActionDetach::ActionDetach(int _part) : part(_part) {}
 
 ActionDetach::~ActionDetach() {}
 
-void ActionDetach::process(Control& target)
+void ActionDetach::process(ControlCharacter& target)
 {
     CharacterInfo& info = target.dynent->getcharinfo();
     info.attached.remove(part);
@@ -357,7 +361,7 @@ ActionCommand::~ActionCommand()
 	delete[] command;
 }
 
-void ActionCommand::process(Control& target)
+void ActionCommand::process(ControlCharacter& target)
 {
 	if(command && !editmode) //#offtools: only execute if not in editmode to prevent recursion
 	{
@@ -374,7 +378,7 @@ ActionCond::~ActionCond()
 	delete[] body;
 }
 
-void ActionCond::process(Control& target)
+void ActionCond::process(ControlCharacter& target)
 {
 	if( (!test || execute(test)) && !editmode )
 	{
@@ -390,7 +394,7 @@ ActionCondWait::~ActionCondWait()
 	delete[] test;
 }
 
-void ActionCondWait::process(Control& target)
+void ActionCondWait::process(ControlCharacter& target)
 {
 	if( (!test || execute(test)) && !editmode )
 	{
@@ -506,8 +510,8 @@ void ActionLib::notify(int ev)
 {
     loopv(game::controls)
     {
-        ControlCharacter* ca = dynamic_cast<ControlCharacter*>(game::controls[i]);
-        if(ca && this == &ca->lib)
+        ControlCharacter* ca = (ControlCharacter*)game::controls[i];
+        if(ca->type() == CONTROL_CHARACTER && this == &ca->lib)
         {
 //            conoutf("DEBUG: ActionLib::notify lib: %s, cn: %d event %d", librarydescr, ca->dynent->clientnum, ev);
             ca->event(ev);
@@ -521,8 +525,8 @@ void ActionLib::notifydelete()
     {
         loopv(game::controls)
         {
-            ControlCharacter* ca = dynamic_cast<ControlCharacter*>(game::controls[i]);
-            if(ca && &(ca->lib) == this)
+            ControlCharacter* ca = (ControlCharacter*)game::controls[i];
+            if(ca->type() == CONTROL_CHARACTER && &(ca->lib) == this)
             {
 //                conoutf("DEBUG: ActionLib::notifydelete Control cn: %d", ca->dynent->clientnum);
                 ca->disconnect();
@@ -635,7 +639,7 @@ void ActionInstance::finished()
 }
 
 //BUG: set correct states in editmode - endless loop through last step
-void ActionInstance::process(Control& target)
+void ActionInstance::process(ControlCharacter& target)
 {
     if(actionstate == ACTION_STATE_IDLE || actionstate == ACTION_STATE_END) return;
 
@@ -654,10 +658,10 @@ void ActionInstance::process(Control& target)
 }
 
 // TODO (offtools#1#): implement nextstep - step through commands
-void ActionInstance::nextstep(Control& target) {}
+void ActionInstance::nextstep(ControlCharacter& target) {}
 
 // TODO (offtools#1#): implement laststep - step to last, process steps between if necessary
-void ActionInstance::laststep(Control& target) {}
+void ActionInstance::laststep(ControlCharacter& target) {}
 //-----------------------------------------------------
 
 vector<ActionLib*> actionlibrary;
@@ -692,7 +696,7 @@ bool newcharacter(const char* s)
     }
 	return false;
 }
-
+//
 int lastuid = 0;
 
 //reset on mapchange
@@ -901,8 +905,8 @@ int clientnumfromuid(int requid)
 {
 	loopv(game::controls)
 	{
-		ControlCharacter* ca = dynamic_cast<ControlCharacter*>(game::controls[i]);
-		if(ca && ca->localuid == requid) { return ca->dynent->clientnum; }
+		ControlCharacter* ca = (ControlCharacter*)game::controls[i];
+		if(ca->type() == CONTROL_CHARACTER && ca->localuid == requid) { return ca->dynent->clientnum; }
 	}
 	return -1;
 }
@@ -911,8 +915,8 @@ void loopcharacter(int id, bool loop = true)
 {
 	loopv(game::controls)
 	{
-		ControlCharacter* ca = dynamic_cast<ControlCharacter*>(game::controls[i]);
-		if(ca && ca->localuid == id)
+		ControlCharacter* ca = (ControlCharacter*)game::controls[i];
+		if(ca->type() == CONTROL_CHARACTER && ca->localuid == id)
 		{
 		    loop ? ca->actionpolicy = ACTION_POLICY_ON_STATE_END_LOOP : ca->actionstate = ACTION_POLICY_ON_STATE_END_WAIT;
         }
