@@ -6,6 +6,31 @@
 
 namespace entities
 {
+    //declared in igame.h
+    int extraentinfosize() { return sizeof(int); }       // size in bytes of what the 2 methods below read/write... so it can be skipped by other games
+    //declared in igame.h
+    void writeent(entity &e, char *buf)   // write any additional data to disk (except for ET_ ents)
+    {
+        memcpy(buf, &((MapEntities::StaticEntity &)e).uid, sizeof(int));
+    }
+    //declared in igame.h
+    void readent(entity &e, char *buf, int ver)     // read from disk, and init
+    {
+// TODO (offtools#1#): check map type (moviecube or others)
+
+//        int ver = getmapversion();
+//        if(ver <= 10)
+//        {
+//            if(e.type >= 7) e.type++;
+//        }
+//        if(ver <= 12)
+//        {
+//            if(e.type >= 8) e.type++;
+//        }
+        memcpy(&((MapEntities::StaticEntity &)e).uid, buf, sizeof(int));
+    }
+
+#ifndef STANDALONE
     using namespace game;
 
     //declared in igame.h
@@ -545,29 +570,6 @@ namespace entities
     	}
     }
     //declared in igame.h
-    int extraentinfosize() { return sizeof(int); }       // size in bytes of what the 2 methods below read/write... so it can be skipped by other games
-    //declared in igame.h
-    void writeent(entity &e, char *buf)   // write any additional data to disk (except for ET_ ents)
-    {
-    	memcpy(buf, &((MapEntities::StaticEntity &)e).uid, sizeof(int));
-    }
-    //declared in igame.h
-    void readent(entity &e, char *buf)     // read from disk, and init
-    {
-// TODO (offtools#1#): check map type (moviecube or others)
-
-//        int ver = getmapversion();
-//        if(ver <= 10)
-//        {
-//            if(e.type >= 7) e.type++;
-//        }
-//        if(ver <= 12)
-//        {
-//            if(e.type >= 8) e.type++;
-//        }
-    	memcpy(&((MapEntities::StaticEntity &)e).uid, buf, sizeof(int));
-    }
-    //declared in igame.h
     void editent(int i, bool local)
     {
         MapEntities::StaticEntity &e = *MapEntities::ents[i];
@@ -617,4 +619,5 @@ namespace entities
     {
     	return;
     }
+#endif
 }
