@@ -1,10 +1,19 @@
 #ifndef __GAME_H__
 #define __GAME_H__
 
+///TODO: preload playermodel in menu
+///TODO: dont register animations twice
+
+
 #include "cube.h"
 
-// console message types
+#define DEBUG_CAMERA  (true)
 
+#ifndef STANDALONE
+extern int dedicated;
+#endif
+
+// console message types
 enum
 {
     CON_CHAT       = 1<<8,
@@ -65,6 +74,7 @@ enum                            // static entity types
     FLAG,                       // attr1 = angle, attr2 = team
     WAYPOINT,
     DYNLIGHT,
+    CAMERA,
     MAXENTTYPES
 };
 
@@ -316,6 +326,10 @@ struct PlayerModelInfo
 
 	void addanimation(char *anim, int num)
 	{
+		loopv(animinfo) if(num == animinfo[i]->index) //#offtools: just a workaround will fix this on lower level in a later version
+		{
+			return;
+		}
 		animinfo.add(new animationinfo(anim, num));
 	}
 
@@ -323,7 +337,7 @@ struct PlayerModelInfo
 	{
 		loopv(animinfo) if(strcmp(animinfo[i]->descr, descr) == 0)
 		{
-			return i;
+			return animinfo[i]->index;
 		}
 		return -1;
 	}
@@ -572,6 +586,11 @@ namespace entities
     extern int getlastmapuid();
 }
 
+namespace CutScene
+{
+	extern void update();
+}
+
 #include "clientmode.h"
 #include "control.h"
 
@@ -683,8 +702,6 @@ namespace game
     extern int chooserandomplayermodel(int seed);
     extern void swayhudgun(int curtime);
     extern vec hudgunorigin(int gun, const vec &from, const vec &to, DynamicEntity *d);
-
-//    extern void debuglog(int level, const char* msg);
 }
 
 namespace server
