@@ -251,6 +251,7 @@ namespace game
     {
         if(cmode) cmode->pickspawn(d);
         else findplayerspawn(d, d==player1 && respawnent>=0 ? respawnent : -1);
+        setbbfrommodel(d, getplayermodelinfo(d).name); // Protagoras: bounding box support
         spawnstate(d);
         if(d==player1)
         {
@@ -623,10 +624,10 @@ namespace game
     {
         switch(index)
         {
-	    case 3: return "data/edit.png";
-            case 2: return "data/hit.png";
-            case 1: return "data/teammate.png";
-            default: return "data/crosshair.png";
+	    case 3: return "packages/crosshairs/edit.png";
+	    case 2: return "packages/crosshairs/hit.png";
+	    case 1: return "packages/crosshairs/teammate.png";
+	    default: return "packages/crosshairs/default.png";
         }
     }
 
@@ -816,7 +817,6 @@ namespace game
     void writegamedata(vector<char> &extras) {}
     void readgamedata(vector<char> &extras) {}
 
-    const char *gameident() { return "movie"; }
     const char *defaultconfig() { return "data/defaults.cfg"; }
     const char *autoexec() { return "autoexec.cfg"; }
     const char *savedservers() { return "servers.cfg"; }
