@@ -150,20 +150,19 @@ namespace game
 	    //if(d->ragdoll && (!ragdoll || anim!=ANIM_DYING)) DELETEP(d->ragdoll);
 
 	    if(!((anim>>ANIM_SECONDARY)&ANIM_INDEX)) anim |= (ANIM_IDLE|ANIM_LOOP)<<ANIM_SECONDARY;
-	    int flags = MDL_LIGHT;
+	    int flags = MDL_LIGHT | MDL_DYNLIGHT | MDL_SHADOW | MDL_DYNSHADOW;
 
 	    if(d!=player1 && !(anim&ANIM_RAGDOLL)) flags |= MDL_CULL_VFC | MDL_CULL_OCCLUDED | MDL_CULL_QUERY;
 
-	    if(d->type==ENT_PLAYER) flags |= MDL_FULLBRIGHT;
-	    else flags |= MDL_CULL_DIST;
+//	    if(d->type==ENT_PLAYER) flags |= MDL_FULLBRIGHT;
+//	    else flags |= MDL_CULL_DIST;
+	    if(d->type!=ENT_PLAYER) flags |= MDL_CULL_DIST;
 
 	    if(d->state==CS_LAGGED) fade = min(fade, 0.3f);
-	    else flags |= MDL_DYNSHADOW;
+//	    else flags |= MDL_DYNSHADOW;
 
 //	    rendermodel(NULL, name   , anim, d->feetpos(), yaw, pitch, 0, flags, d, attached[0].tag ? attached : NULL, basetime, 0, fade);
-//	    rendermodel(NULL, mdlname, anim, d->feetpos(), yaw, pitch, 0, flags, d, attachments, basetime, 0, fade);
-
-	    rendermodel(NULL, mdlname, anim, d->feetpos(), yaw, pitch, 0, 0, d, attachments, basetime, 0, fade);
+	    rendermodel(NULL, mdlname, anim, d->feetpos(), yaw, pitch, 0, flags, d, attachments, basetime, 0, fade); // Fix by Protagoras 20101119
 	}
 
 
