@@ -10,9 +10,10 @@ namespace game
             if(!game::clientoption(args[i]))
 #endif
             if(!server::serveroption(args[i])) {
-				conoutf(CON_ERROR, "unknown command-line option: %s", args[i]);
-			}
-	}
+                conoutf(CON_ERROR, "unknown command-line option: %s", args[i]);
+            }
+    }
+	const char *gameident() { return "movie"; }
 }
 
 extern ENetAddress masteraddress;
@@ -1419,14 +1420,10 @@ namespace server
     {
         int crc, matches;
 
+        crcinfo() {}
         crcinfo(int crc, int matches) : crc(crc), matches(matches) {}
 
-        static int compare(const crcinfo *x, const crcinfo *y)
-        {
-            if(x->matches > y->matches) return -1;
-            if(x->matches < y->matches) return 1;
-            return 0;
-        }
+        static bool compare(const crcinfo &x, const crcinfo &y) { return x.matches > y.matches; }
     };
 
     void checkmaps(int req = -1)
