@@ -622,10 +622,10 @@ namespace game
     {
         switch(index)
         {
-	    case 3: return "data/edit.png";
-            case 2: return "data/hit.png";
-            case 1: return "data/teammate.png";
-            default: return "data/crosshair.png";
+	    case 3: return "packages/crosshairs/edit.png";
+	    case 2: return "packages/crosshairs/hit.png";
+	    case 1: return "packages/crosshairs/teammate.png";
+	    default: return "packages/crosshairs/default.png";
         }
     }
 
