@@ -45,13 +45,49 @@ namespace game
 		player1->secondary = *down != 0 && ! player1->primary;
 	)
 
+	vector<equipment> hotkeys;
+	equipment pending(-1, -1);
+
 	ICOMMAND(hotkey, "i", (int *n),
+		if(*n < 0) return;
+
 		if(rpggui::open())
 			rpggui::hotkey(*n);
-		/*
-		else
-			-equip hot keyed items
-		*/
+		else if(pending.base >= 0)
+		{
+			while(!hotkeys.inrange(*n))
+				hotkeys.add(equipment(-1, -1));
+
+			hotkeys[*n] = pending;
+			pending = equipment(-1, -1);
+		}
+		else if(hotkeys.inrange(*n) && hotkeys[*n].base >= 0 && hotkeys[*n].use >= 0)
+		{
+			int i = player1->getitemcount(hotkeys[*n].base);
+			if(!i) return;
+			if(!player1->dequip(hotkeys[*n].base, -1))
+				player1->equip(hotkeys[*n].base, hotkeys[*n].use);
+		}
+	)
+
+	ICOMMAND(hotkey_get, "i", (int *idx),
+		if(!hotkeys.inrange(*idx))
+		{
+			result("-1 -1");
+			return;
+		}
+		defformatstring(ret)("%i %i", hotkeys[*idx].base, hotkeys[*idx].use);
+		result(ret);
+	)
+
+	ICOMMAND(hotkey_set_pending, "ii", (int *it, int *u),
+		pending.base = *it;
+		pending.use = *u;
+	)
+
+	ICOMMAND(hotkey_clear, "i", (int *idx),
+		if(hotkeys.inrange(*idx))
+			hotkeys[*idx] = equipment(-1, -1);
 	)
 
 	ICOMMAND(include, "s", (const char *pth),
@@ -294,7 +330,7 @@ namespace game
 
 	void respawnself(bool death = false)
 	{
-		player1->respawn();
+		player1->revive();
 	}
 
 	void respawn()
@@ -345,11 +381,14 @@ namespace game
 		map->blips.setsize(0);
 	}
 
+	bool firstupdate = false;
+
 	void startmap(const char *name)
 	{
 		entities::genentlist();
 		ai::clearwaypoints();
 		camera::cleanup(false);
+		firstupdate = true;
 		if(!name)
 			return;
 
@@ -395,8 +434,7 @@ namespace game
 		loopvrev(curmap->loadactions)
 		{
 			curmap->loadactions[i]->exec();
-			delete curmap->loadactions[i];
-			curmap->loadactions.remove(i);
+			delete curmap->loadactions.remove(i);
 		}
 
 		if(DEBUG_WORLD)
@@ -446,6 +484,12 @@ namespace game
 		}
 
 		physicsframe();
+		if(firstupdate)
+		{
+			if(DEBUG_WORLD) conoutf("DEBUG: curmap was recently opened, skipping update");
+			firstupdate = false;
+			return;
+		}
 		if(editmode)
 		{
 			moveplayer(player1, 10, false);
@@ -526,21 +570,12 @@ namespace game
 		ai::trydrop();
 
 		loopvrev(curmap->projs)
-		{
 			if(!curmap->projs[i]->update())
-			{
-				delete curmap->projs[i];
-				curmap->projs.remove(i);
-			}
-		}
+				delete curmap->projs.remove(i);
+
 		loopvrev(curmap->aeffects)
-		{
 			if(!curmap->aeffects[i]->update())
-			{
-				delete curmap->aeffects[i];
-				curmap->aeffects.remove(i);
-			}
-		}
+				delete curmap->aeffects.remove(i);
 
 		camera::update();
 		curmap->getsignal("update");
@@ -709,7 +744,7 @@ namespace game
 
 	bool cansave()
 	{
-		if(curmap->flags & mapinfo::F_NOSAVE || camera::cutscene || rpggui::open())
+		if(editmode || curmap->flags & mapinfo::F_NOSAVE || camera::cutscene || rpggui::open() || rpgscript::stack.length() > 1)
 			return false;
 		return true;
 	}
@@ -775,7 +810,6 @@ namespace game
 	const char *autoexec()		{return "autoexec.cfg";}
 	const char *savedservers()	{return NULL;}
 
-	//TODO write game version
 	void writegamedata(vector<char> &extras) {}
 	void readgamedata (vector<char> &extras) {}
 
@@ -786,5 +820,5 @@ namespace game
 	void mapfailed(const char *name) {}
 	bool allowdoublejump() { return false; }
 	bool detachcamera() { return player1->state == CS_DEAD; }
-	void toserver(char *text) { execute(text); } //since we don't tlak, just execute if the / is forgotten
+	void toserver(char *text) { execute(text); } //since we don't talk, just execute if the / is forgotten
 }
