@@ -219,6 +219,7 @@ namespace ai
 
 		action = attack(victim->getchar(), *p);
 		if(isduplicate(ref->getchar(), &action)) return;
+		if(DEBUG_AI) conoutf(CON_DEBUG, "DEBUG: adding attack directive to entity %p; target %p priority %i", ref->getchar(), victim->getchar(), *p);
 		ref->getchar()->directives.add(new attack(action));
 	)
 
@@ -229,6 +230,7 @@ namespace ai
 
 		action = move(l->o, *p);
 		if(isduplicate(ref->getchar(), &action)) return;
+		if(DEBUG_AI) conoutf(CON_DEBUG, "DEBUG: adding move directive to entity %p; location %i priority %i", ref->getchar(), *loc, *p);
 		ref->getchar()->directives.add(new move(action));
 	)
 
@@ -239,6 +241,7 @@ namespace ai
 
 		action = wander(l->o, max(0, *rad), *p);
 		if(isduplicate(ref->getchar(), &action)) return;
+		if(DEBUG_AI) conoutf(CON_DEBUG, "DEBUG: adding wander directive to entity %p; loc %i, rad %i, priority %i", ref->getchar(), *loc, *rad, *p);
 		ref->getchar()->directives.add(new wander(action));
 	)
 }
