@@ -16,12 +16,20 @@ void use_armour::apply(rpgchar *user)
 	loopv(effects)
 	{
 		if(!game::statuses.inrange(effects[i]->status)) continue;
-		loopvj(game::statuses[effects[i]->status]->effects)
+		statusgroup *sg = game::statuses[effects[i]->status];
+		int resist = 0;
+		int thresh = 0;
+		if(!sg->friendly)
 		{
-			status *st = game::statuses[effects[i]->status]->effects[j];
-			if(st->duration >= 0)
+			resist = user->base.getresistance(effects[i]->element);
+			thresh = user->base.getthreshold(effects[i]->element);
+		}
+
+		loopvj(sg->effects)
+		{
+			if(sg->effects[j]->duration >= 0)
 				conoutf(CON_ERROR, "ERROR: statuses[%i]->effect[%i] has a non negative duration, these will not work properly and some will be ridiculously overpowered", effects[i]->status, j);
-			st->update(user, user, base, extra);
+			sg->effects[j]->update(user, user, resist, thresh, base, extra);
 		}
 	}
 }
@@ -39,7 +47,7 @@ bool rpgchar::checkammo(equipment &eq, equipment *quiver, bool remove)
 			if(base.experience < wep->cost)
 			{
 				if(this == game::player1) conoutf("You lack sufficient exp to perform this action");
-				else if (DEBUG_AI) conoutf(CON_DEBUG, "DEBUG: AI attack interupted for %p, too little exp", this);
+				else if (DEBUG_AI) conoutf(CON_DEBUG, "DEBUG: AI attack interrupted for %p, too little exp", this);
 				return false;
 			}
 			if(remove) base.experience -= wep->cost;
@@ -49,7 +57,7 @@ bool rpgchar::checkammo(equipment &eq, equipment *quiver, bool remove)
 			if((wep->ammo == -1 ? mana : health) < wep->cost)
 			{
 				if(this == game::player1) conoutf(wep->ammo == -1 ? "You lack the mana to cast the full spell" : "Doing this would be suicide");
-				else if (DEBUG_AI) conoutf(CON_DEBUG, "DEBUG: AI attack interupted for %p, too little %s", this, wep->ammo == -1 ? "mana" :"health");
+				else if (DEBUG_AI) conoutf(CON_DEBUG, "DEBUG: AI attack interrupted for %p, too little %s", this, wep->ammo == -1 ? "mana" :"health");
 				return false;
 			}
 			if(remove) (wep->ammo == -1 ? mana : health) -= wep->cost;
@@ -70,14 +78,14 @@ bool rpgchar::checkammo(equipment &eq, equipment *quiver, bool remove)
 			if(base == -1)
 			{
 				if(this == game::player1) conoutf("You have the wrong ammo equipped for the current weapon");
-				else if (DEBUG_AI) conoutf(CON_DEBUG, "DEBUG: AI attack interupted for %p, wrong ammo equipped", this);
+				else if (DEBUG_AI) conoutf(CON_DEBUG, "DEBUG: AI attack interrupted for %p, wrong ammo equipped", this);
 				return false;
 			}
 
 			if(getitemcount(at->items[base]) < wep->cost)
 			{
 				if(this == game::player1) conoutf("You have too little ammo remaining to attack again");
-				else if (DEBUG_AI) conoutf(CON_DEBUG, "DEBUG: AI attack interupted for %p, too little ammo", this);
+				else if (DEBUG_AI) conoutf(CON_DEBUG, "DEBUG: AI attack interrupted for %p, too little ammo", this);
 				return false;
 			}
 
@@ -186,7 +194,7 @@ void rpgchar::doattack(equipment *eleft, equipment *eright, equipment *quiver)
 			case T_SELF:
 				hit(this, attack, ammo, mult, flags, vec(0, 0, 1));
 				if(game::effects.inrange(attack->deatheffect))
-					rpgeffect::drawaura(game::effects[attack->deatheffect], this, mult, rpgeffect::DEATH, 0);
+					game::effects[attack->deatheffect]->drawaura(this, mult, effect::DEATH, 0);
 				break;
 
 			case T_SMULTI:
@@ -239,10 +247,10 @@ void rpgchar::doattack(equipment *eleft, equipment *eright, equipment *quiver)
 				{
 					vec pos = vec(hits[i]).add(orig);
 					if(game::effects.inrange(attack->deatheffect))
-						rpgeffect::drawsplash(game::effects[attack->deatheffect], pos, vec(0, 0, 0), 5, 1, rpgeffect::DEATH, 1);
+						game::effects[attack->deatheffect]->drawsplash(pos, vec(0, 0, 0), 5, 1, effect::DEATH, 1);
 				}
 				if(game::effects.inrange(attack->traileffect))
-					rpgeffect::drawcircle(game::effects[attack->traileffect], this, attack, mult, rpgeffect::TRAIL_SINGLE, 0);
+					game::effects[attack->traileffect]->drawcircle(this, attack, mult, effect::TRAIL_SINGLE, 0);
 
 				break;
 			}
@@ -278,15 +286,27 @@ void rpgchar::doai(equipment *eleft, equipment *eright, equipment *quiver)
 
 	if(aiflags & AI_MOVE)
 	{
-		int start = ai::closestwaypoint(feetpos()) - ai::waypoints.getbuf();
 		int end = ai::closestwaypoint(dest) - ai::waypoints.getbuf();
 		if(!route.length() || route[0] != end)
 		{
 			route.setsize(0);
+			int start = ai::closestwaypoint(feetpos()) - ai::waypoints.getbuf();
 			ai::findroute(start, end, route);
 		}
-		if(route.length() && ai::waypoints[route.last()].o.dist(feetpos()) <= 4)
-			route.pop();
+
+		if(route.length())
+		{
+			waypoint &first = ai::waypoints[route[route.length() - 1]];
+			if(first.o.dist(feetpos()) <= 4)
+				route.pop();
+			else if(route.length() > 1)
+			{
+				waypoint &second = ai::waypoints[route[route.length() - 2]];
+				const vec feet = feetpos();
+				if(first.o.dist(second.o) >= feet.dist(second.o) || feet.dist(second.o) < feet.dist(second.o))
+					route.pop();
+			}
+		}
 	}
 
 	if(target && aiflags & (AI_ATTACK | AI_ALERT))
@@ -486,7 +506,7 @@ void rpgchar::render(bool mainpass)
 		if(use->type < USE_ARMOUR || !use->vwepmdl || use->idlefx < 0 || !use->slots)
 			continue;
 
-		rpgeffect::drawwield(game::effects[use->idlefx], emitter[0], emitter[1], 1, rpgeffect::TRAIL);
+		game::effects[use->idlefx]->drawwield(emitter[0], emitter[1], 1, effect::TRAIL);
 		emitter[0] = emitter[1] = vec(-1, -1, -1);
 		emitter += 2;
 	}
@@ -510,6 +530,11 @@ void rpgchar::equip(int i, int u)
 		conoutf("You can't equip items while attacking!");
 		return;
 	}
+	else if(!game::items.inrange(i) || !game::items[i]->uses.inrange(u))
+	{
+		conoutf(CON_ERROR, "ERROR: unable to equip item %i with use %i; itself or the use does not exist", i, u);
+		return;
+	}
 
 	invstack *item = NULL;
 	loopvj(inventory)
@@ -525,11 +550,6 @@ void rpgchar::equip(int i, int u)
 		conoutf(CON_ERROR, "ERROR: user contains no instances of item %i", i);
 		return;
 	}
-	else if(!game::items[i]->uses.inrange(u))
-	{
-		conoutf(CON_ERROR, "ERROR: unable to equip item, desired use method out of range");
-		return;
-	}
 	else if(game::items[i]->uses[u]->type < USE_ARMOUR)
 	{
 		conoutf(CON_ERROR, "ERROR: invalid use type, needs to be of type weapon or armour to equip");
@@ -552,13 +572,14 @@ void rpgchar::equip(int i, int u)
 	item->quantity--;
 }
 
-void rpgchar::dequip(int i, int slots)
+bool rpgchar::dequip(int i, int slots)
 {
 	if(primary || secondary || lastprimary || lastsecondary)
 	{
 		conoutf("You can't dequip items while attacking!");
-		return;
+		return false;
 	}
+	int rem = 0;
 	loopvj(equipped)
 	{
 		invstack *item = NULL;
@@ -577,13 +598,16 @@ void rpgchar::dequip(int i, int slots)
 			inventory.add(item);
 		}
 
+
 		use_armour *arm = (use_armour *) game::items[equipped[j]->base]->uses[equipped[j]->use];
 		if((i == -1 ? true : equipped[j]->base == i) && (slots ? arm->slots & slots : true))
 		{
 			item->quantity++;
 			delete equipped.remove(j--);
+			rem++;
 		}
 	}
+	return rem != 0;
 }
 
 void rpgchar::die(rpgent *killer)
@@ -602,50 +626,51 @@ void rpgchar::die(rpgent *killer)
 	getsignal("death", true, killer); //in case the player has an item in his inventory that will revive him
 }
 
-void rpgchar::respawn()
+void rpgchar::revive(bool spawn)
 {
 	if(state != CS_DEAD)
 		return;
 
+	health = base.getmaxhp();
+	mana = base.getmaxmp();
+	state = CS_ALIVE;
+	seffects.deletecontents();
+
 	physent::reset();
 	cleanragdoll(this);
-	seffects.deletecontents();
-	state = CS_ALIVE;
-	findplayerspawn(this, -1);
-	health = base.getmaxhp();
-	mana = 0; //dying consumes it all
+	if(spawn) findplayerspawn(this, -1);
+	else entinmap(this);
 }
 
 extern int friendlyfire;
 
-VAR(friendly, 1, 0, -1);
+VAR(hit_friendly, 1, 0, -1);
+VAR(hit_total, 1, 0, -1);
 
 void rpgchar::hit(rpgent *attacker, use_weapon *weapon, use_weapon *ammo, float mul, int flags, vec dir)
 {
-	friendly = 1;
+	hit_friendly = 1;
+	hit_total = 0;
 	loopv(weapon->effects)
 	{
 		if(!game::statuses.inrange(weapon->effects[i]->status)) continue;
-		if(!game::statuses[weapon->effects[i]->status]->friendly) friendly = 0;
+		if(!game::statuses[weapon->effects[i]->status]->friendly) hit_friendly = 0;
 		//TODO friendly fire checks here
-		//TODO resistance and threshold
 		seffects.add(new victimeffect(attacker, weapon->effects[i], weapon->chargeflags, mul));
+		hit_total += game::statuses[weapon->effects[i]->status]->strength() * mul;
 	}
 
 	if(ammo) loopv(ammo->effects)
 	{
 		if(!game::statuses.inrange(ammo->effects[i]->status)) continue;
-		if(!game::statuses[ammo->effects[i]->status]->friendly) friendly = 0;
+		if(!game::statuses[ammo->effects[i]->status]->friendly) hit_friendly = 0;
 		//TODO friendly fire checks here
-		//TODO resistance and threshold
 		seffects.add(new victimeffect(attacker, ammo->effects[i], weapon->chargeflags, mul));
+		hit_total += game::statuses[ammo->effects[i]->status]->strength() * mul;
 	}
 
 	vel.add(dir.mul(weapon->kickback + (ammo ? ammo->kickback : 0)));
-
-	//TODO damage compass
-//	if(total) damagecompass(total, attacker->o);
-
+	if(this == game::player1 && !hit_friendly && hit_total) damagecompass(hit_total, attacker->o);
 	getsignal("hit", false, attacker);
 }
 
@@ -705,7 +730,8 @@ int rpgchar::modinv(int b, int q, bool spawn)
 
 			drop->base = b;
 			drop->quantity = removed;
-			drop->o = drop->newpos = vec(o).add(vec(yaw * RAD, pitch * RAD).mul(radius * 2));
+			drop->o = drop->newpos = vec(yaw * RAD, pitch * RAD).mul(radius * 2).add(o);
+			entinmap(drop);
 		}
 
 		return removed;
@@ -716,10 +742,9 @@ int rpgchar::modinv(int b, int q, bool spawn)
 
 int rpgchar::pickup(rpgitem *item)
 {
-	//TODO allow the player to carry 2x his limit, but reduce his speed on a linear curve after 1x
 	int add = item->quantity;
 	if(game::items.inrange(item->base) && game::items[item->base]->weight)
-		add = clamp<float>(add, 0, (base.getmaxcarry() - getweight()) / game::items[item->base]->weight);
+		add = clamp<float>(add, 0, (base.getmaxcarry() * 2 - getweight()) / game::items[item->base]->weight);
 
 	modinv(item->base, add, false);
 	item->quantity -= add;
