@@ -39,13 +39,22 @@ namespace game
 			loading ## x = NULL; \
 			\
 			reference *obj = rpgscript::searchstack(ref, false); \
-			if(obj) {loading ## x = select;} \
+			if(obj) \
+			{ \
+				loading ## x = select; \
+				if(loading ## x && DEBUG_VCONF) \
+					conoutf("DEBUG: successfully selected \"" #c "\" from reference %s", ref); \
+			} \
 			\
 			if(!loading ##x) \
 			{ \
 				int idx = parseint(ref); \
 				if(vec.inrange(idx)) \
+				{ \
+					if(DEBUG_VCONF) \
+						conoutf("DEBUG: successfully selected \"" #c "\" using index %i", idx); \
 					loading ## x = vec[idx]; \
+				} \
 			} \
 			if(loading ## x) \
 				execute(contents); \
@@ -54,17 +63,30 @@ namespace game
 			loading ## x = old; \
 		) \
 		ICOMMAND(r_num_ ## c, "", (), \
+			if(DEBUG_VCONF) \
+				conoutf("DEBUG: r_num_" #c " requested, returning %i", vec.length()); \
 			intret(vec.length()); \
 		)
 
 	CHECK(script, script, scripts, NULL)
 	CHECK(effect, effect, effects, NULL)
-	CHECK(statusgroup, status, statuses, NULL)
-	CHECK(item_base, item, items, NULL)
+	CHECK(statusgroup, status, statuses,
+		obj->getveffect() ? statuses[obj->getveffect()->group] :
+		obj->getaeffect() ? statuses[obj->getaeffect()->group] :
+		NULL
+	)
+	CHECK(item_base, item, items,
+		obj->getinv() ? items[obj->getinv()->base] :
+		obj->getequip() ? items[obj->getequip()->base] :
+		NULL
+	)
 	CHECK(recipe, recipe, recipes, NULL)
 	CHECK(ammotype, ammo, ammotypes, NULL)
 	CHECK(char_base, char, chars, obj->getchar())
-	CHECK(faction, faction, factions, NULL)
+	CHECK(faction, faction, factions,
+		  obj->getchar() ? factions[obj->getchar()->faction] :
+		  NULL
+	)
 	CHECK(obstacle_base, obstacle, obstacles, obj->getobstacle())
 	CHECK(container_base, container, containers, obj->getcontainer())
 	CHECK(platform_base, platform, platforms, obj->getplatform())
@@ -76,25 +98,38 @@ namespace game
 	use *loadinguse = NULL;
 	static use *checkuse(int type)
 	{
-		if(!loadingitem_base)
-			conoutf(CON_ERROR, "ERROR: item not defined or being loaded");
-		else if(!loadinguse)
+		if(!loadinguse)
 			conoutf(CON_ERROR, "ERROR: use not defined or being loaded");
-		else if(loadingitem_base->uses.find(loadinguse) >= 0);
+		else if(loadinguse->type < type)
+			conoutf(CON_ERROR, "ERROR: trying to set an unavailable use property");
+		else
 			return loadinguse;
-		return (loadinguse = NULL); //for another item if it hits here
+		return NULL;
 	}
 	ICOMMAND(r_select_item_use, "se", (const char *ref, uint *contents),
-		if(!checkitem_base()) return; //just to print a warning
-
 		use *old = loadinguse;
 		loadinguse = NULL;
+		reference *obj = rpgscript::searchstack(ref, false);
 
-		int idx = parseint(ref);
-		if(loadingitem_base->uses.inrange(idx))
-			loadinguse = loadingitem_base->uses[idx];
+		if(obj && obj->getequip())
+		{
+			if(DEBUG_VCONF)
+				conoutf("DEBUG: successfully selected \"use\" from reference %s", ref);
+			loadinguse = items[obj->getequip()->base]->uses[obj->getequip()->use];
+		}
+		else if(loadingitem_base)
+		{
+			int idx = parseint(ref);
+			if(loadingitem_base->uses.inrange(idx))
+			{
+				if(DEBUG_VCONF)
+					conoutf("DEBUG: successfully selected \"use\" using index %i", idx);
+				loadinguse = loadingitem_base->uses[idx];
+			}
+		}
 
 		if(loadinguse) execute(contents);
+		else conoutf(CON_ERROR, "ERROR: unable to select reference %s as type use", ref);
 		loadinguse = old;
 	)
 	ICOMMAND(r_num_item_use, "", (),
@@ -123,6 +158,8 @@ namespace game
 		START(name ## _get, "", (), \
 			INIT \
 			if(!e) return; \
+			if(DEBUG_VCONF) \
+				conoutf("DEBUG: request for " #name " on %p, returning %i", e, e->var); \
 			intret(e->var); \
 		)
 	#define INT(var, min, max) INTN(var, var, min, max)
@@ -140,6 +177,8 @@ namespace game
 		START(name ## _get, "", (), \
 			INIT \
 			if(!e) return; \
+			if(DEBUG_VCONF) \
+				conoutf("DEBUG: request for " #name " on %p, returning %f", e, e->var); \
 			floatret(e->var); \
 		)
 	#define FLOAT(var, min, max) FLOATN(var, var, min, max)
@@ -155,8 +194,10 @@ namespace game
 		) \
 		START(name ## _get, "", (), \
 			INIT \
-			if(e && e->var) result(e->var); \
-			else result(""); \
+			if(!e) return; \
+			if(DEBUG_VCONF) \
+				conoutf("DEBUG: request for " #name " on %p, returning %s", e, e->var); \
+			result(e->var ? e->var : ""); \
 		)
 	#define STRING(var) STRINGN(var, var)
 
@@ -176,6 +217,8 @@ namespace game
 			INIT \
 			if(!e) return; \
 			defformatstring(res)("%g %g %g", e->var.x, e->var.y, e->var.z); \
+			if(DEBUG_VCONF) \
+				conoutf("DEBUG: request for " #name " on %p, returning %s", e, res); \
 			result(res); \
 		)
 	#define VEC(var, l1, l2, l3, h1, h2, h3) VECN(var, var, l1, l2, l3, h1, h2, h3)
@@ -191,6 +234,8 @@ namespace game
 		START(name ## _get, "", (), \
 			INIT \
 			if(!e) return; \
+			if(DEBUG_VCONF) \
+				conoutf("DEBUG: request for " #name " on %p, returning %i", e, e->var); \
 			intret(e->var); \
 		)
 	#define BOOL(var) BOOLN(var, var)
@@ -548,6 +593,7 @@ namespace game
 	#define TYPE USE_ARMOUR
 
 	STRING(vwepmdl)
+	STRING(hudmdl)
 	INT(idlefx, -1, 0xFFFF)
 	STATREQ(reqs)
 	INT(slots, 0, SLOT_MAX - 1)
@@ -558,7 +604,6 @@ namespace game
 	#define CAST use_weapon
 	#define TYPE USE_WEAPON
 
-	STRING(hudmdl)
 	INT(range, 0, 1024)
 	INT(angle, 0, 360)
 	INT(lifetime, 0, 0xFFFF)
