@@ -104,7 +104,8 @@ enum
 
 enum
 {
-	ATTACK_FIRE = 0,
+	ATTACK_NONE = -1,
+	ATTACK_FIRE,
 	ATTACK_WATER,
 	ATTACK_AIR,
 	ATTACK_EARTH,
@@ -191,6 +192,7 @@ struct mapscript
 struct rpgent;
 struct rpgchar;
 struct rpgitem;
+struct use_weapon;
 
 enum
 {
@@ -229,6 +231,23 @@ struct effect
 		lightradius = lightinitradius = 64;
 		lightcol = lightinitcol = vec(1, .9, 0);
 	}
+
+	enum
+	{
+		PROJ,
+		TRAIL, // a trail
+		TRAIL_SINGLE, //a trail which only emits once
+		DEATH, // death cariant which only emits once
+		DEATH_PROLONG // death variant which is intended for a delay that takes a while
+	};
+
+	void drawsphere(vec &o, float radius, float size = 1, int type = PROJ, int elapse = 17);
+	void drawsplash(vec &o, vec dir, float radius, float size = 1, int type = PROJ, int elapse = 17);
+	bool drawline(vec &from, vec &to, float size = 1, int type = PROJ, int elapse = 17);
+	void drawwield(vec &from, vec &to, float size = 1, int type = PROJ, int elapse = 17);
+	void drawaura(rpgent *d, float size = 1, int type = PROJ, int elapse = 17);
+	void drawcircle(vec &o, vec dir, vec &axis, int angle, float radius, float size = 1, int type = PROJ, int elapse = 17);
+	void drawcircle(rpgent *d, use_weapon *wep, float size = 1, int type = PROJ, int elapse = 17);
 };
 
 enum
@@ -300,7 +319,7 @@ struct status
 
 	virtual int value()=0;
 	virtual status *dup()=0; //duplicate
-	virtual void update(rpgent *victim, rpgent *owner, float mul = 1, float extra = 0)=0;
+	virtual void update(rpgent *victim, rpgent *owner, int resist, int thresh, float mul = 1, float extra = 0)=0;
 
 	status() : type(STATUS_HEALTH), duration(0), remain(0), strength(0), variance(0.0f) {}
 	virtual ~status() {}
@@ -310,7 +329,7 @@ struct status_generic : status
 {
 	int value() { return abs(strength); }
 	status *dup() { return new status_generic(*this); }
-	void update(rpgent *victim, rpgent *owner, float mul, float extra);
+	void update(rpgent *victim, rpgent *owner, int resist, int thresh, float mul, float extra);
 
 	status_generic() {}
 	~status_generic() {}
@@ -328,7 +347,7 @@ struct status_polymorph : status
 		p->mdl = mdl ? newstring(mdl) : NULL;
 		return p;
 	}
-	void update(rpgent *victim, rpgent *owner, float mul, float extra);
+	void update(rpgent *victim, rpgent *owner, int resist, int thresh, float mul, float extra);
 
 	status_polymorph() : mdl(NULL) {}
 	~status_polymorph() { delete[] mdl; }
@@ -340,7 +359,7 @@ struct status_light : status
 
 	int value() { return strength * colour.magnitude(); }
 	status *dup() { return new status_light(*this); }
-	void update(rpgent *victim, rpgent *owner, float mul, float extra);
+	void update(rpgent *victim, rpgent *owner, int resist, int thresh, float mul, float extra);
 
 	status_light() : colour(vec(0, 0, 0)) {}
 	~status_light() {}
@@ -357,7 +376,7 @@ struct status_signal : status
 		if(signal) n->signal = newstring(signal);
 		return n;
 	}
-	void update(rpgent *victim, rpgent *owner, float mul, float extra);
+	void update(rpgent *victim, rpgent *owner, int resist, int thresh, float mul, float extra);
 
 	status_signal() : signal(NULL) {}
 	~status_signal() {delete[] signal;}
@@ -376,7 +395,7 @@ struct status_script : status
 		if(script) n->script = newstring(script);
 		return n;
 	}
-	void update(rpgent *victim, rpgent *owner, float mul, float extra);
+	void update(rpgent *victim, rpgent *owner, int resist, int thresh, float mul, float extra);
 
 	status_script() : script(NULL), code(NULL) {}
 	~status_script() {delete[] script; delete[] code;}
@@ -408,14 +427,14 @@ struct areaeffect
 {
 	rpgent *owner;
 	vec o;
-	int lastemit, group, radius, fx;
+	int lastemit, group, elem, radius, fx;
 
 	vector<status *> effects;
 
 	bool update();
 	void render();
 
-	areaeffect() : owner(NULL), o(vec(0, 0, 0)), lastemit(lastmillis), group(0), radius(0), fx(0) {}
+	areaeffect() : owner(NULL), o(vec(0, 0, 0)), lastemit(lastmillis), group(0), elem(ATTACK_NONE), radius(0), fx(0) {}
 	~areaeffect() { effects.deletecontents(); }
 };
 
@@ -423,12 +442,13 @@ struct inflict;
 struct victimeffect
 {
 	rpgent *owner;
-	int group; //just for descriptions
+	int group;
+	int elem;
 	vector<status *> effects;
 
 	bool update(rpgent *victim);
 	victimeffect(rpgent *o, inflict *inf, int chargeflags, float mul);
-	victimeffect() : owner(NULL), group(0) {}
+	victimeffect() : owner(NULL), group(0), elem(ATTACK_NONE) {}
 	~victimeffect() { effects.deletecontents(); }
 };
 
@@ -487,9 +507,9 @@ struct rpgent : dynent
 	///character/AI
 	virtual void givexp(int xp) {}
 	virtual void equip(int i, int u = 0) {}
-	virtual void dequip(int i, int slots = 0) {}
+	virtual bool dequip(int i, int slots = 0) {return false;}
 	virtual void die(rpgent *killer = NULL) {}
-	virtual void respawn() {}
+	virtual void revive(bool spawn = true) {}
 	virtual void hit(rpgent *attacker, use_weapon *weapon, use_weapon *ammo, float mul, int flags, vec dir) = 0;
 
 	///inventory
@@ -538,6 +558,7 @@ enum
 
 struct stats
 {
+	rpgchar *parent;
 	//note delta vars are for spell effects
 	///SPECIAL
 	int experience, level, points;
@@ -561,134 +582,25 @@ struct stats
 	float deltahregen, deltamregen;
 
 	///QUERIES
-	static int neededexp(int level) { return level * (level + 1) * 500; } //standard EXP formula, 0... 1000.... 3000.... 6000.... 10000....
-	void givexp(int xp)
-	{
-		experience = max(0, experience + xp);
-		while(experience >= neededexp(level))
-		{
-			level++;
-			points += 5;
-		}
-	}
-
-	//no safety checks are done, make sure to use the ENUM
+	//no safety checks are done for these 3, make sure to use the ENUM!!
 	inline int getattr(int n) const {return max(0, baseattrs[n] + deltaattrs[n]);}
 	inline int getskill(int n) const {return max(0, baseskills[n] + deltaskills[n]);}
 	inline int critchance() const {return bonuscrit + deltacrit + getattr(STAT_LUCK) / 10;}
+	
+	static int neededexp(int level) { return level * (level + 1) * 500; } //standard EXP formula, 0... 1000.... 3000.... 6000.... 10000....
+	void givexp(int xp);
+	int getmaxhp();
+	float gethpregen();
+	int getmaxmp();
+	float getmpregen();
+	int getmaxcarry();
+	int getthreshold(int n);
+	int getresistance(int n);
+	void skillpotency(int n, float &amnt, float &extra);
+	void setspeeds(float &maxspeed, float &jumpvel);
+	void resetdeltas();
 
-	int getmaxhp()
-	{
-		int amnt = bonushealth + deltahealth + (getattr(STAT_ENDURANCE) * 4 + getattr(STAT_STRENGTH) * 2) * (25 + level) / 40.0f;
-		return max(1, amnt); ///TODO allow maxhp of 0 and just kill the character?
-	}
-
-	float gethpregen()
-	{
-		return bonushregen + deltahregen + getattr(STAT_ENDURANCE) * 0.0025 + getattr(STAT_STRENGTH) * 0.00125;
-	}
-
-	int getmaxmp()
-	{
-		int amnt = bonusmana + deltamana + (getattr(STAT_INTELLIGENCE) * 2 + getattr(STAT_WISDOM) * 4) * (25 + level) / 40.0f;
-		return max(0, amnt);
-	}
-
-	float getmpregen()
-	{
-		return bonusmregen + deltamregen + getattr(STAT_WISDOM)  * 0.05 + getattr(STAT_INTELLIGENCE) * 0.025;
-	}
-
-	int getmaxcarry()
-	{
-		int amnt = bonuscarry + deltacarry + getattr(STAT_STRENGTH) * 5;
-		return max(0, amnt);
-	}
-
-	int getthreshold(int n)
-	{
-		//unlike resistances, thresholds are not defined by stats
-		return bonusthresh[n] + deltathresh[n];
-	}
-
-	int getresistance(int n)
-	{
-		float amnt = bonusresist[n] + deltaresist[n] + getattr(STAT_LUCK) / 15.0f;
-		switch(n)
-		{
-			case ATTACK_FIRE:
-			case ATTACK_WATER:
-				amnt += getattr(STAT_AGILITY) / 10.0f; break;
-			case ATTACK_AIR:
-			case ATTACK_EARTH:
-				amnt += getattr(STAT_STRENGTH) / 10.0f; break;
-			case ATTACK_MAGIC:
-				amnt += (getattr(STAT_WISDOM) + getattr(STAT_INTELLIGENCE)) / 20.0f; break;
-			case ATTACK_SLASH:
-				amnt += getattr(STAT_AGILITY) / 20.0f; break;
-			case ATTACK_BLUNT:
-				amnt += getattr(STAT_STRENGTH) / 20.0f; break;
-			case ATTACK_PIERCE:
-				amnt += getattr(STAT_ENDURANCE) / 20.0f; break;
-		}
-		return min<int>(95, amnt);
-	}
-
-	void skillpotency(int n, float &amnt, float &extra)
-	{
-		amnt = .25;
-		switch(n)
-		{
-			case -1: amnt = 1; extra = rnd(50) / 100.0f; return; //special NO MODIFIERs case
-			case SKILL_ARMOUR:
-				amnt += (getattr(STAT_STRENGTH)      + getattr(STAT_ENDURANCE) * 2       + getskill(SKILL_ARMOUR) * 5) / 100.0f;
-				break;
-			case SKILL_DIPLOMACY:
-				amnt += (getattr(STAT_CHARISMA) * 3                                      + getskill(SKILL_DIPLOMACY) * 5) / 100.0f;
-				break;
-			case SKILL_MAGIC:
-				amnt += (getattr(STAT_WISDOM) * 1.75 + getattr(STAT_INTELLIGENCE) * 1.25 + getskill(SKILL_MAGIC) * 5) / 100.0f;
-				break;
-			case SKILL_MARKSMAN:
-				amnt += (getattr(STAT_STRENGTH) * .5 + getattr(STAT_AGILITY) * 2.5       + getskill(SKILL_MARKSMAN) * 5) / 100.0f;
-				break;
-			case SKILL_MELEE:
-				amnt += (getattr(STAT_AGILITY) * .5  + getattr(STAT_STRENGTH) * 2.5      + getskill(SKILL_MELEE) * 5) / 100.0f;
-				break;
-			case SKILL_STEALTH:
-				amnt += (getattr(STAT_LUCK) * 1      + getattr(STAT_AGILITY) * 2         + getskill(SKILL_STEALTH) * 5) / 100.0f;
-				break;
-		}
-		amnt += logf(1 + (amnt / 5));
-
-		extra = amnt * (50 + rnd(101)) / 100.0f;
-		if(rnd(100) < critchance())
-			extra = extra * (1.10 + (rnd(191) / 100.0f)) + (rnd(150) / 100.0f) * (rnd(150) / 100.0f);
-		extra -= amnt;
-	}
-
-	void setspeeds(float &maxspeed, float &jumpvel)
-	{
-		maxspeed = 40 + bonusmovespeed + deltamovespeed + getattr(STAT_AGILITY) / 4.f;
-		jumpvel = 80 + bonusjumpvel + deltajumpvel + getattr(STAT_AGILITY) / 2.f;
-	}
-
-	void resetdeltas()
-	{
-		loopi(STAT_MAX)
-			deltaattrs[i] = 0;
-		loopi(SKILL_MAX)
-			deltaskills[i] = 0;
-		loopi(ATTACK_MAX)
-		{
-			deltathresh[i] = 0;
-			deltaresist[i] = 0;
-		}
-		deltahealth = deltamana = deltamovespeed = deltajumpvel = deltacarry = deltacrit = 0;
-		deltahregen = deltamregen = 0;
-	}
-
-	stats() : experience(0), level(1), points(0)
+	stats(rpgchar *p) : parent(p), experience(0), level(1), points(0)
 	{
 		loopi(STAT_MAX)
 			baseattrs[i] = 30;
@@ -798,19 +710,19 @@ struct use
 struct use_armour : use
 {
 	const char *vwepmdl;
+	const char *hudmdl;
 	int idlefx;
 	statreq reqs;
 	int slots;
 	int skill;
 
 	void apply(rpgchar *user);
-	use_armour(int s) : use(s), vwepmdl(NULL), idlefx(-1), reqs(statreq()), slots(0), skill(SKILL_ARMOUR) {type = USE_ARMOUR;}
-	~use_armour() {}
+	use_armour(int s) : use(s), vwepmdl(NULL), hudmdl(NULL), idlefx(-1), reqs(statreq()), slots(0), skill(SKILL_ARMOUR) {type = USE_ARMOUR;}
+	~use_armour() {delete[] vwepmdl; delete[] hudmdl;}
 };
 
 struct use_weapon : use_armour
 {
-	const char *hudmdl;
 	int range;
 	int angle;
 	int lifetime;
@@ -828,7 +740,7 @@ struct use_weapon : use_armour
 	float elasticity, speed;
 
 	void apply(rpgchar *user);
-	use_weapon(int s) : use_armour(s), hudmdl(NULL), range(256), angle(60), lifetime(10000), gravity(0), projeffect(0), traileffect(0), deatheffect(0), cost(10), pflags(P_DIST|P_TIME), ammo(-1), target(T_SINGLE), radius(32), kickback(10), recoil(0), charge(0), basecharge(.5f), mincharge(.5f), maxcharge(1.0f), elasticity(0.8), speed(1.0f) {type = USE_WEAPON;}
+	use_weapon(int s) : use_armour(s), range(256), angle(60), lifetime(10000), gravity(0), projeffect(0), traileffect(0), deatheffect(0), cost(10), pflags(P_DIST|P_TIME), ammo(-1), target(T_SINGLE), radius(32), kickback(10), recoil(0), charge(0), basecharge(.5f), mincharge(.5f), maxcharge(1.0f), elasticity(0.8), speed(1.0f) {type = USE_WEAPON;}
 	~use_weapon() {}
 };
 
@@ -930,9 +842,7 @@ struct rpgobstacle : rpgent, obstacle_base
 	///character/AI
 	void givexp(int xp) {}
 	void equip(int i, int u = 0) {}
-	void dequip(int i, int slots = 0) {}
 	void die(rpgent *killer = NULL) {}
-	void respawn() {}
 	void hit(rpgent *attacker, use_weapon *weapon, use_weapon *ammo, float mul, int flags, vec dir);
 
 
@@ -985,9 +895,7 @@ struct rpgcontainer : rpgent, container_base
 	///character/AI
 	void givexp(int xp) {}
 	void equip(int i, int u = 0) {}
-	void dequip(int i, int slots = 0) {}
 	void die(rpgent *killer = NULL) {}
-	void respawn() {}
 	void hit(rpgent *attacker, use_weapon *weapon, use_weapon *ammo, float mul, int flags, vec dir);
 
 	///inventory
@@ -1048,9 +956,7 @@ struct rpgplatform : rpgent, platform_base
 	///character/AI
 	void givexp(int xp) {}
 	void equip(int i, int u = 0) {}
-	void dequip(int i, int slots = 0) {}
 	void die(rpgent *killer = NULL) {}
-	void respawn() {}
 	void hit(rpgent *attacker, use_weapon *weapon, use_weapon *ammo, float mul, int flags, vec dir);
 
 	rpgplatform() : target(-1) {physent::type = ENT_INANIMATE;}
@@ -1103,9 +1009,7 @@ struct rpgtrigger : rpgent, trigger_base
 	///character/AI
 	void givexp(int xp) {}
 	void equip(int i, int u = 0) {}
-	void dequip(int i, int slots = 0) {}
 	void die(rpgent *killer = NULL) {}
-	void respawn() {}
 	void hit(rpgent *attacker, use_weapon *weapon, use_weapon *ammo, float mul, int flags, vec dir);
 
 	rpgtrigger() : lasttrigger(lastmillis) {physent::type = ENT_INANIMATE;}
@@ -1193,7 +1097,9 @@ struct char_base
 		delete[] o.mdl;
 		delete[] o.name;
 
+		rpgchar *p = o.base.parent;
 		o = *this;
+		o.base.parent = p;
 		if(name) o.name = newstring(name);
 		o.mdl = newstring(mdl);
 		o.inventory.deletecontents();
@@ -1201,7 +1107,7 @@ struct char_base
 		loopv(inventory) o.inventory.add(new invstack(*inventory[i]));
 	}
 
-	char_base() : name(NULL), mdl(newstring(DEFAULTMODEL)), base(stats()), script(1), faction(0) {}
+	char_base(rpgchar *p = NULL) : name(NULL), mdl(newstring(DEFAULTMODEL)), base(stats(p)), script(1), faction(0) {}
 	~char_base()
 	{
 		delete[] name;
@@ -1255,9 +1161,9 @@ struct rpgchar : rpgent, char_base
 	///character/AI
 	void givexp(int xp);
 	void equip(int i, int u = 0);
-	void dequip(int i, int slots = 0);
+	bool dequip(int i, int slots = 0);
 	void die(rpgent *killer);
-	void respawn();
+	void revive(bool spawn = true);
 	void hit(rpgent *attacker, use_weapon *weapon, use_weapon *ammo, float mul, int flags, vec dir);
 
 	///inventory
@@ -1272,7 +1178,7 @@ struct rpgchar : rpgent, char_base
 
 	void cleanup();
 
-	rpgchar() : lastaction(0), charge(0), primary(false), secondary(false), lastprimary(false), lastsecondary(false), attack(NULL), aiflags(0), target(NULL)
+	rpgchar() : char_base(this), lastaction(0), charge(0), primary(false), secondary(false), lastprimary(false), lastsecondary(false), attack(NULL), aiflags(0), target(NULL)
 	{
 		health = base.getmaxhp();
 		mana = base.getmaxmp();
@@ -1529,6 +1435,17 @@ struct reference
 	~reference();
 };
 
+struct delayscript
+{
+	hashtable<const char *, reference> refs;
+	const char *script;
+	int remaining;
+
+	bool update();
+	delayscript() : refs(hashtable<const char *, reference>(128)), script(NULL), remaining(0) {}
+	~delayscript() { delete[] script; }
+};
+
 #ifdef NO_DEBUG
 
 #define DEBUG_WORLD   (false)
@@ -1539,6 +1456,7 @@ struct reference
 #define DEBUG_AI      (false)
 #define DEBUG_IO      (false)
 #define DEBUG_CAMERA  (false)
+#define DEBUG_STATUS  (false)
 #define DEBUG_VERBOSE (false)
 
 #else
@@ -1551,7 +1469,8 @@ struct reference
 #define DEBUG_AI      (game::debug & (1 << 5))
 #define DEBUG_IO      (game::debug & (1 << 6))
 #define DEBUG_CAMERA  (game::debug & (1 << 7))
-#define DEBUG_VERBOSE (game::debug & (1 << 8))
+#define DEBUG_STATUS  (game::debug & (1 << 8))
+#define DEBUG_VERBOSE (game::debug & (1 << 9))
 
 #endif
 
@@ -1564,27 +1483,9 @@ struct reference
 #define DEBUG_VAI      (DEBUG_VERBOSE && DEBUG_AI)
 #define DEBUG_VIO      (DEBUG_VERBOSE && DEBUG_IO)
 #define DEBUG_VCAMERA  (DEBUG_VERBOSE && DEBUG_CAMERA)
+#define DEBUG_VSTATUS  (DEBUG_VERBOSE && DEBUG_STATUS)
 
-#define DEBUG_MAX ((1 << 9) - 1)
-
-namespace rpgeffect
-{
-	enum
-	{
-		PROJ,
-		TRAIL, // a trail
-		TRAIL_SINGLE, //a trail which only emits once
-		DEATH, // death cariant which only emits once
-		DEATH_PROLONG // death variant which is intended for a delay that takes a while
-	};
-	extern void drawsphere(effect *e, vec &o, float radius, float size = 1, int type = PROJ, int elapse = 17);
-	extern void drawsplash(effect *e, vec &o, vec dir, float radius, float size = 1, int type = PROJ, int elapse = 17);
-	extern bool drawline(effect *e, vec &from, vec &to, float size = 1, int type = PROJ, int elapse = 17);
-	extern void drawwield(effect *e, vec &from, vec &to, float size = 1, int type = PROJ, int elapse = 17);
-	extern void drawaura(effect *e, rpgent *d, float size = 1, int type = PROJ, int elapse = 17);
-	extern void drawcircle(effect *e, vec &o, vec dir, vec &axis, int angle, float radius, float size = 1, int type = PROJ, int elapse = 17);
-	extern void drawcircle(effect *e, rpgent *d, use_weapon *wep, float size = 1, int type = PROJ, int elapse = 17);
-}
+#define DEBUG_MAX ((1 << 10) - 1)
 
 namespace ai
 {
@@ -1618,6 +1519,8 @@ namespace game
 	extern vector<mapscript *> mapscripts;
 	extern vector<recipe *> recipes;
 
+	extern vector<equipment> hotkeys;
+
 	//config
 	extern script *loadingscript;
 	extern effect *loadingeffect;
@@ -1670,7 +1573,9 @@ namespace rpgscript
 	extern void domapscript(mapinfo *invokee, rpgent *invoker, uint *code);
 
 	extern void pushstack();
+	extern void popstack();
 	extern vector<hashtable<const char*, reference> *> stack;
+	extern vector<delayscript *> delaystack;
 }
 
 namespace entities
