@@ -4,7 +4,7 @@ extern bool reloadtexture(const char *name); //texture.cpp
 
 namespace rpgio
 {
-	#define GAME_VERSION 23
+	#define GAME_VERSION 27
 	#define GAME_MAGIC "RPGS"
 
 	/**
@@ -173,18 +173,40 @@ namespace rpgio
 
 				loading->name = readstring(f);
 				loading->mdl = readstring(f);
-
-				f->read(&loading->base, sizeof(stats));
-				lilswap(&loading->base.experience, 3);
-				lilswap(loading->base.baseattrs, STAT_MAX * 2 + SKILL_MAX * 2);
-
-				lilswap(loading->base.bonusthresh, ATTACK_MAX * 2);
-				lilswap(&loading->base.bonushealth, 6);
-				lilswap(&loading->base.bonushregen, 2);
-
-				lilswap(loading->base.deltathresh, ATTACK_MAX * 2);
-				lilswap(&loading->base.deltahealth, 6);
-				lilswap(&loading->base.deltahregen, 2);
+				
+				#define x(var, type) loading->base.var = f->getlil<type>();
+	
+				x(experience, int)
+				x(level, int)
+				x(points, int)
+				loopi(STAT_MAX) x(baseattrs[i], short)
+				loopi(SKILL_MAX) x(baseskills[i], short)
+				loopi(STAT_MAX) x(deltaattrs[i], short)
+				loopi(SKILL_MAX) x(deltaskills[i], short)
+				
+				loopi(ATTACK_MAX) x(bonusthresh[i], short)
+				loopi(ATTACK_MAX) x(bonusresist[i], short)
+				x(bonushealth, int)
+				x(bonusmana, int)
+				x(bonusmovespeed, int)
+				x(bonusjumpvel, int)
+				x(bonuscarry, int)
+				x(bonuscrit, int)
+				x(bonushregen, float)
+				x(bonusmregen, float)
+				
+				loopi(ATTACK_MAX) x(deltathresh[i], short)
+				loopi(ATTACK_MAX) x(deltaresist[i], short)
+				x(deltahealth, int)
+				x(deltamana, int)
+				x(deltamovespeed, int)
+				x(deltajumpvel, int)
+				x(deltacarry, int)
+				x(deltacrit, int)
+				x(deltahregen, float)
+				x(deltamregen, float)
+					
+				#undef x
 
 				int items = f->getlil<int>();
 				loopi(items)
@@ -319,6 +341,7 @@ namespace rpgio
 
 			updates.add(reference(f->getlil<int>(), lastmap, eff->owner));
 			eff->group = f->getlil<int>();
+			eff->elem = f->getlil<int>();
 			int numstat = f->getlil<int>();
 			loopj(numstat)
 			{
@@ -404,21 +427,40 @@ namespace rpgio
 
 				writestring(f, saving->name);
 				writestring(f, saving->mdl);
-
-				stats tmp = saving->base;
-
-				lilswap(&tmp.experience, 3);
-				lilswap(tmp.baseattrs, STAT_MAX * 2 + SKILL_MAX * 2);
-
-				lilswap(tmp.bonusthresh, ATTACK_MAX * 2);
-				lilswap(&tmp.bonushealth, 6);
-				lilswap(&tmp.bonushregen, 2);
-
-				lilswap(tmp.deltathresh, ATTACK_MAX * 2);
-				lilswap(&tmp.deltahealth, 6);
-				lilswap(&tmp.deltahregen, 2);
-
-				f->write(&tmp, sizeof(stats));
+				
+				#define x(var) f->putlil(saving->base.var);
+	
+				x(experience)
+				x(level)
+				x(points)
+				loopi(STAT_MAX) x(baseattrs[i])
+				loopi(SKILL_MAX) x(baseskills[i])
+				loopi(STAT_MAX) x(deltaattrs[i])
+				loopi(SKILL_MAX) x(deltaskills[i])
+				
+				loopi(ATTACK_MAX) x(bonusthresh[i])
+				loopi(ATTACK_MAX) x(bonusresist[i])
+				x(bonushealth)
+				x(bonusmana)
+				x(bonusmovespeed)
+				x(bonusjumpvel)
+				x(bonuscarry)
+				x(bonuscrit)
+				x(bonushregen)
+				x(bonusmregen)
+				
+				loopi(ATTACK_MAX) x(deltathresh[i])
+				loopi(ATTACK_MAX) x(deltaresist[i])
+				x(deltahealth)
+				x(deltamana)
+				x(deltamovespeed)
+				x(deltajumpvel)
+				x(deltacarry)
+				x(deltacrit)
+				x(deltahregen)
+				x(deltamregen)
+					
+				#undef x
 
 				f->putlil(saving->inventory.length());
 				loopv(saving->inventory)
@@ -532,6 +574,7 @@ namespace rpgio
 		{
 			f->putlil(enttonum(d->seffects[i]->owner));
 			f->putlil(d->seffects[i]->group);
+			f->putlil(d->seffects[i]->elem);
 			f->putlil(d->seffects[i]->effects.length());
 
 			loopvj(d->seffects[i]->effects)
@@ -700,6 +743,7 @@ namespace rpgio
 			aeff->owner = entfromnum(f->getlil<int>());
 			aeff->lastemit = 0; //should emit immediately
 			aeff->group = f->getlil<int>();
+			aeff->elem = f->getlil<int>();
 			aeff->radius = f->getlil<int>();
 			aeff->fx = f->getlil<int>();
 
@@ -850,6 +894,7 @@ namespace rpgio
 			f->putlil(enttonum(aeff->owner));
 			writevec(aeff->o);
 			f->putlil(aeff->group);
+			f->putlil(aeff->elem);
 			f->putlil(aeff->radius);
 			f->putlil(aeff->fx);
 
@@ -1030,7 +1075,20 @@ namespace rpgio
 					return;
 			}
 		}
+	}
 
+	void readdelayscript(stream *f, vector<mapinfo *> &maps, delayscript *loading)
+	{
+		readreferences(f, maps, loading->refs);
+		loading->script = readstring(f);
+		loading->remaining = f->getlil<int>();
+	}
+
+	void writedelayscript(stream *f, vector<mapinfo *> &maps, delayscript *saving)
+	{
+		writereferences(f, maps, saving->refs);
+		writestring(f, saving->script);
+		f->putlil(saving->remaining);
 	}
 
 	void loadgame(const char *name)
@@ -1098,6 +1156,12 @@ namespace rpgio
 				b; \
 			}
 
+		READ(hotkey,
+			int b = f->getlil<int>();
+			int u = f->getlil<int>();
+			game::hotkeys.add(equipment(b, u));
+		)
+
 		READ(faction,
 			if(!game::factions.inrange(i)) game::factions.add(new faction());
 			readfaction(f, game::factions[i]);
@@ -1118,6 +1182,11 @@ namespace rpgio
 			readreferences(f, maps, *rpgscript::stack[i]);
 		)
 
+		READ(delayscript stack,
+			rpgscript::delaystack.add(new delayscript());
+			readdelayscript(f, maps, rpgscript::delaystack[i]);
+		)
+
 		#undef READ
 
 		delete f;
@@ -1179,6 +1248,11 @@ namespace rpgio
 				b; \
 			}
 
+		WRITE(hotkey, game::hotkeys,
+			f->putlil(game::hotkeys[i].base);
+			f->putlil(game::hotkeys[i].use);
+		)
+
 		WRITE(faction, game::factions,
 			writefaction(f, game::factions[i]);
 		)
@@ -1197,6 +1271,10 @@ namespace rpgio
 			writereferences(f, maps, *rpgscript::stack[i]);
 		)
 
+		WRITE(delayscript stack, rpgscript::delaystack,
+			writedelayscript(f, maps, rpgscript::delaystack[i]);
+		)
+
 		game::curmap->objs.add(game::player1);
 		DELETEP(f);
 
