@@ -193,9 +193,20 @@ void rpgent::getsignal(const char *sig, bool prop, rpgent *sender)
 		if(listen)
 			rpgscript::doentscript(this, sender, listen->code);
 	}
-	if(prop)
+	if(prop && (type() == ENT_CHAR || type() == ENT_CONTAINER))
 	{
-		//TODO propagate to inventory
+		vector<invstack *> &inv = type() == ENT_CHAR ? ((rpgchar *) this)->inventory : ((rpgcontainer *) this)->inventory;
+		loopv(inv)
+		{
+			if(inv[i]->quantity > 0)
+				inv[i]->getsignal(sig, prop, this, -1);
+			if(type() == ENT_CHAR)
+			{
+				vector<equipment*> &equips = ((rpgchar *) this)->equipped;
+				loopvj(equips) if(equips[j]->base == inv[i]->base)
+					inv[i]->getsignal(sig, prop, this, equips[j]->use);
+			}
+		}
 	}
 }
 
@@ -216,6 +227,18 @@ void mapinfo::getsignal(const char *sig, bool prop, rpgent *sender)
 	}
 }
 
+bool delayscript::update()
+{
+	if((remaining -= curtime) > 0 || camera::cutscene)
+		return true;
+
+	rpgscript::stack.add(&refs);
+	execute(script);
+	rpgscript::stack.pop();
+
+	return false;
+}
+
 /*
 	start the actual script stuff!
 */
@@ -232,6 +255,7 @@ namespace rpgscript
 
 	vector<rpgent *> obits; //r_destroy
 	vector<hashtable<const char*, reference> *> stack;
+	vector<delayscript *> delaystack;
 	reference *player = NULL;
 	reference *hover = NULL;
 	reference *map = NULL;
@@ -327,6 +351,37 @@ namespace rpgscript
 		return r;
 	}
 
+	void copystack(hashtable<const char *, reference> &dst)
+	{
+		static reference dummy("");
+		loopvj(stack)
+		{
+			enumerate(*stack[j], reference, ref,
+				reference *newref = dst.access(ref.name);
+				if(!newref)
+				{
+					const char *n = newstring(ref.name);
+					newref = &dst.access(n, dummy);
+					newref->name = n;
+				}
+				newref->setnull();
+				if(!temporary(ref))
+				{
+					newref->type = ref.type;
+					newref->ref = ref.ref;
+				}
+			)
+		}
+	}
+
+	ICOMMAND(r_sleep, "is", (int *rem, const char *scr),
+		delayscript *ds = delaystack.add(new delayscript());
+		ds->remaining = *rem;
+		ds->script = newstring(scr);
+		copystack(ds->refs);
+	)
+
+
 	void clean()
 	{
 		stack.deletecontents();
@@ -391,6 +446,12 @@ namespace rpgscript
 			);
 		}
 
+		loopv(delaystack)
+		{
+			if(!delaystack[i]->update())
+				delete delaystack.remove(i--);
+		}
+
 		vec dir = vec(worldpos).sub(player1->o);
 		if(dir.magnitude() > player1->radius + 8)
 		{
@@ -407,7 +468,7 @@ namespace rpgscript
 	void doitemscript(invstack *invokee, rpgent *invoker, uint *code)
 	{
 		if(!invokee || !code) return;
-		if(DEBUG_SCRIPT)
+		if(DEBUG_VSCRIPT)
 			conoutf(CON_DEBUG, "DEBUG: conditions met, executing code with invokee %p and invoker %p", invokee, invoker);
 
 		pushstack();
@@ -422,7 +483,7 @@ namespace rpgscript
 	void doentscript(rpgent *invokee, rpgent *invoker, uint *code)
 	{
 		if(!invokee || !code) return;
-		if(DEBUG_SCRIPT)
+		if(DEBUG_VSCRIPT)
 			conoutf(CON_DEBUG, "DEBUG: conditions met, executing code with invokee %p and invoker %p", invokee, invoker);
 
 		pushstack();
@@ -437,7 +498,7 @@ namespace rpgscript
 	void domapscript(mapinfo *invokee, rpgent *invoker, uint *code)
 	{
 		if(!invokee || !code) return;
-		if(DEBUG_SCRIPT)
+		if(DEBUG_VSCRIPT)
 			conoutf(CON_DEBUG, "DEBUG: conditions met, executing code with invokee %p and invoker %p", invokee, invoker);
 
 		pushstack();
@@ -888,7 +949,13 @@ namespace rpgscript
 		intret(ent->getchar()->faction)
 	)
 
-	//add effect
+	ICOMMAND(r_add_effect, "sifi", (const char *ref, int *st, float *mul, int *flags),
+		if(!statuses.inrange(*st)) return;
+		getreference(ref, victim, victim->getent(), , r_add_effect)
+
+		inflict inf(*st, ATTACK_NONE, 1); //resistances aren't applied
+		victim->getent()->seffects.add(new victimeffect(NULL, &inf, *flags, *mul));
+	)
 
 	ICOMMAND(r_kill, "ss", (const char *ref, const char *killer), //I was framed!
 		getreference(ref, victim, victim->getchar(), , r_kill)
@@ -898,7 +965,12 @@ namespace rpgscript
 		victim->getent()->die(nasty ? nasty->getent() : NULL);
 	)
 
-	//resurrect
+	ICOMMAND(r_resurrect, "s", (const char *ref),
+		getreference(ref, target, target->getchar(), , r_resurrect)
+
+		if(DEBUG_SCRIPT) conoutf(CON_DEBUG, "DEBUG: r_resurrect; trying to revive \"%s\"", ref);
+		target->getchar()->revive(false);
+	)
 
 	ICOMMAND(r_pickup, "ss", (const char *finder, const char *keepsake), //finders keepers :P
 		getreference(finder, actor, actor->getchar(), , r_pickup)
@@ -1286,6 +1358,54 @@ namespace rpgscript
 		defformatstring(str)("%i %i %i %i %f", s->type, s->strength, s->duration, s->remain, s->variance);
 		result(str);
 	)
+	
+	ICOMMAND(r_get_status_duration, "si", (const char *ref, int *idx),
+		getreference(ref, eff, eff->getaeffect() || eff->getveffect(), intret(0), r_get_status_duration)
+		vector<status *> &effects = eff->getaeffect() ? eff->getaeffect()->effects : eff->getveffect()->effects;
+		int val = 0;
+		
+		if(*idx >= 0)
+		{
+			if(!effects.inrange(*idx))
+			{
+				val = 0;
+				conoutf(CON_ERROR, "ERROR: r_get_status_duration, invalid index");
+			}
+			else 
+				val = effects[*idx]->duration;
+		}
+		else loopv(effects)
+		{
+			val = max(val, effects[i]->duration);
+		}
+		
+		if(DEBUG_VSCRIPT) conoutf("DEBUG: r_get_status_duration; returning %i for \"%s\" %i", val, ref, *idx);
+		intret(val);
+	)
+		 
+	ICOMMAND(r_get_status_remain, "si", (const char *ref, int *idx),
+		getreference(ref, eff, eff->getaeffect() || eff->getveffect(), intret(0), r_get_status_remain)
+		vector<status *> &effects = eff->getaeffect() ? eff->getaeffect()->effects : eff->getveffect()->effects;
+		int val = 0;
+		
+		if(*idx >= 0)
+		{
+			if(!effects.inrange(*idx))
+			{
+				val = 0;
+				conoutf(CON_ERROR, "ERROR: r_get_status_remain, invalid index");
+			}
+			else 
+				val = effects[*idx]->remain;
+		}
+		else loopv(effects)
+		{
+			val = max(val, effects[i]->remain);
+		}
+		
+		if(DEBUG_VSCRIPT) conoutf("DEBUG: r_get_status_remain; returning %i for \"%s\" %i", val, ref, *idx);
+		intret(val);
+	)
 
 	ICOMMAND(r_sound, "ss", (const char *p, const char *r),
 		if(*r)
