@@ -3,6 +3,7 @@
 victimeffect::victimeffect(rpgent *o, inflict *inf, int chargeflags, float mul) : owner(o)
 {
 	group = inf->status;
+	elem = game::statuses[group]->friendly ? ATTACK_NONE : inf->element;
 	mul *= inf->mul;
 
 	statusgroup *sg = game::statuses[inf->status];
@@ -18,9 +19,18 @@ victimeffect::victimeffect(rpgent *o, inflict *inf, int chargeflags, float mul)
 
 bool victimeffect::update(rpgent *victim)
 {
+	int resist = 0;
+	int thresh = 0;
+	if(victim->type() == ENT_CHAR)
+	{
+		rpgchar *d = (rpgchar *) victim;
+		resist = d->base.getresistance(elem);
+		thresh = d->base.getthreshold(elem);
+	}
+
 	loopvrev(effects)
 	{
-		effects[i]->update(victim, owner);
+		effects[i]->update(victim, owner, resist, thresh);
 
 		if(effects[i]->duration >= 0 && (effects[i]->remain -= curtime) <= 0)
 		{
@@ -33,19 +43,29 @@ bool victimeffect::update(rpgent *victim)
 
 bool areaeffect::update()
 {
-	vector<rpgent *> targets;
-	loopv(game::curmap->objs)
+	loopvj(game::curmap->objs)
 	{
 		//checking the head and the feet should be sufficiently accurate for gaming purposes
-		rpgent *d = game::curmap->objs[i];
-		if(d->o.dist(o) <= radius || d->feetpos().dist(o) <= radius)
-			targets.add(d);
+		rpgent *victim = game::curmap->objs[j];
+		if(victim->o.dist(o) <= radius || victim->feetpos().dist(o) <= radius)
+		{
+			int resist = 0;
+			int thresh = 0;
+			if(victim->type() == ENT_CHAR)
+			{
+				rpgchar *d = (rpgchar *) victim;
+				resist = d->base.getresistance(elem);
+				thresh = d->base.getthreshold(elem);
+			}
+
+			loopvrev(effects)
+				effects[i]->update(victim, owner, resist, thresh);
+
+		}
 	}
 
 	loopvrev(effects)
 	{
-		loopvj(targets) effects[i]->update(targets[j], owner);
-
 		if(effects[i]->duration >= 0 && (effects[i]->remain -= curtime) <= 0)
 		{
 			delete effects.remove(i);
@@ -65,22 +85,23 @@ void areaeffect::render()
 		default:
 			if((lastmillis - lastemit) >= emitmillis)
 			{
-				rpgeffect::drawsphere(e, o, radius, e->size, rpgeffect::DEATH_PROLONG, lastmillis - lastemit);
+				e->drawsphere(o, radius, e->size, effect::DEATH_PROLONG, lastmillis - lastemit);
 				lastemit = lastmillis;
 			}
 			break;
 
 		case PART_EXPLOSION:
 		case PART_EXPLOSION_BLUE:
-			rpgeffect::drawsphere(e, o, radius, e->size, rpgeffect::DEATH_PROLONG, 0);
+			e->drawsphere(o, radius, e->size, effect::DEATH_PROLONG, 0);
 			break;
 	}
 }
 
-void status_generic::update(rpgent *victim, rpgent *owner, float mul, float extra)
+void status_generic::update(rpgent *victim, rpgent *owner, int resist, int thresh, float mul, float extra)
 {
 	//time based multiplier, dwindles on last 20% of duration
-	float mult = mul + extra * variance;
+	int strength = this->strength < 0 ? min(0, this->strength + thresh) : max(0, this->strength - thresh);
+	float mult = (mul + extra * variance) * (100 - resist) / 100.f;
 	if(duration > 0)
 		mult *= clamp<float>(1, 0, 5.f * remain/duration);
 
@@ -217,32 +238,49 @@ void status_generic::update(rpgent *victim, rpgent *owner, float mul, float extr
 	}
 }
 
-void status_polymorph::update(rpgent *victim, rpgent *owner, float mul, float extra)
+void status_polymorph::update(rpgent *victim, rpgent *owner, int resist, int thresh, float mul, float extra)
 {
 	if(duration == 0) return;
 
 	victim->temp.mdl = mdl;
 }
 
-void status_light::update(rpgent *victim, rpgent *owner, float mul, float extra)
+void status_light::update(rpgent *victim, rpgent *owner, int resist, int thresh, float mul, float extra)
 {
 	if(duration == 0) return;
-	float mult = mul + extra * variance;
+
+	int strength = this->strength < 0 ? min(0, this->strength + thresh) : max(0, this->strength - thresh);
+	float mult = (mul + extra * variance) * (100 - resist) / 100.f;
 	if(duration > 0)
 		mult *= clamp<float>(1, 0, 5.f * remain/duration);
 
 	victim->temp.light.add(vec4(colour, strength * mult));
 }
 
-//TODO set the remaining, mul and duration variables... somehow
-void status_signal::update(rpgent *victim, rpgent *owner, float mul, float extra)
+VAR(eff_strength, 1, -1, -1);
+VAR(eff_remain, 1, -1, -1);
+VAR(eff_mult, 1, -1, -1);
+VAR(eff_duration, 1, -1, -1);
+
+void status_signal::update(rpgent *victim, rpgent *owner, int resist, int thresh, float mul, float extra)
 {
+	eff_strength = strength < 0 ? min(0, strength + thresh) : max(0, strength - thresh);
+	eff_remain = remain;
+	eff_duration = duration;
+	eff_mult = (mul + extra * variance) * (100 - resist) / 100.f;
+
 	victim->getsignal(signal, true, owner);
 }
 
-void status_script::update(rpgent *victim, rpgent *owner, float mul, float extra)
+void status_script::update(rpgent *victim, rpgent *owner, int resist, int thresh, float mul, float extra)
 {
 	if(!script) return;
+
 	if(!code) code = compilecode(script);
+	eff_strength = strength < 0 ? min(0, strength + thresh) : max(0, strength - thresh);
+	eff_remain = remain;
+	eff_duration = duration;
+	eff_mult = (mul + extra * variance) * (100 - resist) / 100.f;
+
 	rpgscript::doentscript(victim, owner, code);
 }
\ No newline at end of file
