@@ -1,10 +1,15 @@
 #ifndef __CUBE_H__
 #define __CUBE_H__
 
+#define _FILE_OFFSET_BITS 64
+
 #ifdef __GNUC__
 #define gamma __gamma
 #endif
 
+#ifdef WIN32
+#define _USE_MATH_DEFINES
+#endif
 #include <math.h>
 
 #ifdef __GNUC__
@@ -18,16 +23,20 @@
 #include <stdarg.h>
 #include <limits.h>
 #include <assert.h>
-#ifdef __GNUC__
-#include <new>
-#else
-#include <new.h>
-#endif
 #include <time.h>
 
 #ifdef WIN32
-#define WIN32_LEAN_AND_MEAN
-#include "windows.h"
+  #define WIN32_LEAN_AND_MEAN
+  #define _WIN32_WINNT 0x0500
+  #include "windows.h"
+  #ifndef _WINDOWS
+    #define _WINDOWS
+  #endif
+  #ifndef __GNUC__
+    #include <eh.h>
+    #include <dbghelp.h>
+  #endif
+  #define ZLIB_DLL
 #endif
 
 #ifndef STANDALONE
@@ -44,14 +53,6 @@
 
 #include <enet/enet.h>
 
-#ifdef WIN32
-  #define _WINDOWS
-  #ifndef __GNUC__
-    #define ZLIB_DLL
-    #include <eh.h>
-    #include <dbghelp.h>
-  #endif
-#endif
 #include <zlib.h>
 
 #ifdef __sun__
