@@ -241,7 +241,12 @@ struct vec2
     float cross(const vec2 &o) const { return x*o.y - y*o.x; }
 
     vec2 &mul(float f)       { x *= f; y *= f; return *this; }
+    vec2 &mul(const vec2 &o) { x *= o.x; y *= o.y; return *this; }
+    vec2 &div(float f)       { x /= f; y /= f; return *this; }
+    vec2 &div(const vec2 &o) { x /= o.x; y /= o.y; return *this; }
+    vec2 &add(float f)       { x += f; y += f; return *this; }
     vec2 &add(const vec2 &o) { x += o.x; y += o.y; return *this; }
+    vec2 &sub(float f)       { x -= f; y -= f; return *this; }
     vec2 &sub(const vec2 &o) { x -= o.x; y -= o.y; return *this; }
     vec2 &neg()              { x = -x; y = -y; return *this; }
 };
@@ -400,23 +405,9 @@ struct dualquat
 
     dualquat &invert()
     {
-        float rr = real.squaredlen();
-        if(rr > 0)
-        {
-            float invrr = 1/rr,
-                  invrd = -2*real.dot(dual)*invrr*invrr;
-
-            dual.mul3(-invrr);
-            dual.w *= invrr;
-            quat tmp(real);
-            tmp.mul3(-invrd);
-            tmp.w *= invrd;
-            dual.add(tmp);
-
-            real.mul3(-invrr);
-            real.w *= invrr;
-        }
-        else { real = dual = quat(0, 0, 0, 0); }
+        real.invert();
+        dual.invert();
+        dual.sub(quat(real).mul(2*real.dot(dual)));
         return *this;
     }
 
@@ -570,7 +561,7 @@ struct matrix3x3
         angle = acosf(clamp(0.5f*(a.x + b.y + c.z - 1), -1.0f, 1.0f));
 
 		if(angle <= 0) axis = vec(0, 0, 1);
-        else if(angle < M_PI) 
+        else if(angle < M_PI)
         {
             axis = vec(c.y - b.z, a.z - c.x, b.x - a.y);
             float r = axis.magnitude();
@@ -665,7 +656,7 @@ struct matrix3x4
         b.mul(k);
         c.mul(k);
     }
-    
+
     void translate(const vec &p)
     {
         a.w += p.x;
@@ -975,7 +966,7 @@ struct ivec
 
     ivec() {}
     ivec(const vec &v) : x(int(v.x)), y(int(v.y)), z(int(v.z)) {}
-    ivec(int i)
+    explicit ivec(int i)
     {
         x = ((i&1)>>0);
         y = ((i&2)>>1);
@@ -1374,3 +1365,5 @@ extern bool raysphereintersect(const vec &center, float radius, const vec &o, co
 extern bool rayrectintersect(const vec &b, const vec &s, const vec &o, const vec &ray, float &dist, int &orient);
 extern bool linecylinderintersect(const vec &from, const vec &to, const vec &start, const vec &end, float radius, float &dist);
 
+extern const vec2 sincos360[];
+
