@@ -3,6 +3,7 @@
 extern int curtime;                     // current frame time
 extern int lastmillis;                  // last time
 extern int totalmillis;                 // total elapsed time
+extern uint totalsecs;
 extern int gamespeed, paused;
 extern bvec watercolor;
 extern char *version;
@@ -357,7 +358,6 @@ extern void cleanragdoll(dynent *d);
 #define MAXTRANS 5000                  // max amount of data to swallow in 1 go
 
 extern int maxclients;
-extern int dedi;
 
 enum { DISC_NONE = 0, DISC_EOP, DISC_CN, DISC_KICK, DISC_TAGT, DISC_IPBAN, DISC_PRIVATE, DISC_MAXCLIENTS, DISC_TIMEOUT, DISC_OVERFLOW, DISC_NUM };
 
