@@ -8,7 +8,7 @@ namespace entities
     extern const int numattrs(int type);
     extern int extraentinfosize();
     extern void writeent(entity &e, char *buf);
-    extern void readent(entity &e, char *buf);
+    extern void readent(entity &e, char *buf, int ver);
     extern float dropheight(entity &e);
     extern void fixentity(extentity &e);
     extern void entradius(extentity &e, bool &color);
