@@ -595,14 +595,26 @@ namespace game
 	const char *defaultcrosshair(int index)
 	{
 		//memo to self, make 1^2 px crosshair
-		return editmode ? "data/edit" : "data/crosshair";
+		switch(index)
+		{
+			case 2: return "packages/crosshairs/default";
+			case 1: return "packages/crosshairs/edit";
+			default: return "data/items";
+		}
 	}
 
 	int selectcrosshair(float &r, float &g, float &b)
 	{
-		r = b = 0.5;
-		g = 1.0f;
-		return editmode;
+		if(editmode)
+		{
+			r = b = 0.5;
+			g = 1.0f;
+			return 1;
+		}
+		if(mousetarget)
+			return 2;
+
+		return 0;
 	}
 
 	void initclient()
