@@ -637,10 +637,9 @@ namespace entities
 
 	void writeent(entity &e, char *buf) {}  // write any additional data to disk (except for ET_ ents)
 
-	void readent(entity &e, char *buf)     // read from disk, and init
+	void readent(entity &e, char *buf, int ver)     // read from disk, and init
 	{
-		int version = getmapversion();
-		if(version <= 30)
+		if(ver <= 30)
 		{
 			switch(e.type)
 			{
