@@ -6,22 +6,22 @@ namespace game
 
 	vector<enemy *> monstertypes;
 
-	ICOMMAND(addmonster, "siiii", (char *s, int *t, int *x, int *y, int *z), {monstertypes.add(new enemy(s, *t, *x, *y, *z));});
-	ICOMMAND(resetmonsters, "", (), monstertypes.deletecontentsp(); );
+	ICOMMAND(addmonster, "siiii", (char *s, int *t, int *x, int *y, int *z), {monstertypes.add(new enemy(s, *t, *x, *y, *z));})
+	ICOMMAND(resetmonsters, "", (), monstertypes.deletecontents(); )
 
 
 	void initialisemonsters()
 	{
-		intents.setsizenodelete(0);
+		intents.setsize(0);
 
 		loopv(entities::ents)
 		{
 			extentity &e = *entities::ents[i];
 			if(e.type==ENEMY)
 			{
-				if(monstertypes.inrange(e.attr2))
+				if(monstertypes.inrange(e.attr[1]))
 				{
-					sspmonster *mon = new sspmonster(e, *monstertypes[e.attr2]);
+					sspmonster *mon = new sspmonster(e, *monstertypes[e.attr[1]]);
 					sspobjs.add(mon);
 				}
 			}
@@ -41,9 +41,9 @@ namespace game
 
 			float dist = e.o.dist(d->o);
 
-			if(e.type==TELEPORT && dist <= (e.attr2 ? abs(e.attr2) : 16))
+			if(e.type==TELEPORT && dist <= (e.attr[1] ? abs(e.attr[1]) : 16))
 				entities::trypickup(intents[i], d);
-			if(e.type==JUMPPAD && dist <= (e.attr4 ? abs(e.attr4) : 12))
+			if(e.type==JUMPPAD && dist <= (e.attr[3] ? abs(e.attr[3]) : 12))
 				entities::trypickup(intents[i], d);
 		}
 	}
